/**
 * @author Qing Guo
 */

 dojo.provide("com.ibm.pb.dnd.layout.LayoutFreeformSource");
 
 dojo.require("com.ibm.dnd.TunedLayoutSource");
 dojo.require("com.ibm.pb.logging.Loggable");
 dojo.require("com.ibm.mashups.builder.IEvent");
 dojo.require("com.ibm.mm.builder.utils.PageWidthChanger");
 dojo.require("com.ibm.mm.builder.dnd.QuickWireHelper");
 
 dojo.declare("com.ibm.pb.dnd.layout.LayoutFreeformSource", 
 	[com.ibm.dnd.TunedLayoutSource, com.ibm.mm.builder.dnd.QuickWireHelper, com.ibm.pb.logging.Loggable], {
 	mouseDown:false,
	topPos: null,
	height: 700,
	managerOverrides: {
		OFFSET_X: 1,
		OFFSET_Y: 1,
		makeAvatar: function(){
			return new com.ibm.pb.dnd.layout.LayoutControlAvatar(this);
		}
	},
	layoutParser: com.ibm.pb.layout.LayoutParser,
	constructor: function() {
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.resizeWidgetEvt = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.resizeWidget", this, "__widgetResized");
		this.addNewWidget = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.addNewWidget", this, "_addNewWidget");
		var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		this.lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
		if(this.node.style){
			this.height = dojo.style(this.node,"height");
			if(this.node.style.width){
				if (dojo.isIE) {
					dojo.style(this.node, "width", "100%");
				}
				else {
					this.node.style.removeProperty("width");
				}
			}
		}
		this.count = dojo.query(".component-control", this.node).length;
		//following codes are used for MoveWidget Dialog
		var cst = dojo.coords(this.node, true);
		var pageWidthChanger = com.ibm.mm.builder.utils.pageWidthChanger;
		pageWidthChanger.setWidthRangeProvider(this.createFixedWidthRangeProvider(cst.w));
		//end
   		this.onOverEvent();
	},
		
	onDrop: function(source, nodes, copy,draggedOut){	
		this.LOGGER.entering("onDrop",[source, nodes, copy]);
		this.inherited(arguments);
		var currentNode = nodes[0];
		
		//dojo.style(dojo.body(),{cursor:"default"});
		var sourceItem = source.getItem(currentNode.id);
		if (sourceItem && (sourceItem.type == ibmCfg.controllers.dnd.SHELF_TYPE)) {
			this.layoutContainer = dojo.dnd.manager().target.node;
		}else{
			if(source.anchor){
				this.layoutContainer = source.anchor.parentNode;
			}else{
				this.layoutContainer = dojo.dnd.manager().target.node;
			}
		}
		var _containerId = this.layoutParser.extractID(this.node);
		var _currentNodeId = this.layoutParser.extractID(currentNode);
		var lcCoords = dojo.coords(this.node, true);
		
		if (sourceItem && (sourceItem.type == ibmCfg.controllers.dnd.SHELF_TYPE)) {
			com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = true;
			var widget = dijit.byId(nodes[0].id);
			widget.apply(_containerId, null);
		} else {
			widgetNode = currentNode;
			var lctrl = this.lm.find(_currentNodeId).start();

			var avatarPosition1 = dojo.position(dojo.dnd.manager().avatar.node);
			var xMax = Math.max(avatarPosition1.x - lcCoords.x, 0);

			var _position = {
				x: Math.min(Math.max(this._last_drop_X - lcCoords.x - this._last_layerX, 0), xMax),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0)
			};
			if(draggedOut){//user's mouse has moved out of the layout.
				var avatarPosition = dojo.position(dojo.dnd.manager().avatar.node);
				_position = {
					x: Math.max(avatarPosition.x - lcCoords.x, 0),
					y: Math.max(avatarPosition.y - lcCoords.y, 0)
				};
			}
			
			if(source != this){
				_position = {
				x: Math.max(this._last_drop_X - lcCoords.x, 0),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0)
				};
				if (lctrl) {
					var styleTxt = lctrl.getMetaData("css-style-backup");
					if (styleTxt) 
						widgetNode.style.cssText = styleTxt;
				}
			}
			
			widgetNode.style.cssText += "; position: absolute; "
									+ "left: " + _position.x + "px; "
									+ "top: " + _position.y + "px;"
			if (lctrl) {
				lctrl.setMetaData("css-style", widgetNode.style.cssText);
			}
			this._saveWidgetStyles();//to update the zindex for all widgets.
			
			if(dojo.hasClass(source.node, "hiddenWidgetsContainer")){
				var containerDef = this.lm.find(_containerId);
				if (containerDef) {
					containerDef.setFinishedCallback(dojo.hitch(this, "_insert"), {
					layoutModel: this.lm,
					controls: { target: null, current: currentNode }});
					containerDef.start(false);
				}
				else {
					this.LOGGER.severe("onDrop", "Cannot find container in the layout model");
				}
			}			
		}
		
		// 20637 remove unused space at the bottom of the freeform layout by updating bottom layout border to be 
		// bottom of bottommost widget + 10px
        // also as the default is to have a freeForm of 700px, the default bottommostWidgetBorder is 690
        var bottommostWidgetBorder = 690;

        dojo.query(".component-control", this.node ).forEach(function(widgetNode, index, arr){
            // dojo style should always return a number of px
            var widgetTop = dojo.style(widgetNode, "top");
            var widgetHeight = dojo.style(widgetNode, "height");

            var widgetBottom = widgetTop + widgetHeight;

            // check if this widget is below any other widget
            if (widgetBottom > bottommostWidgetBorder) {
                bottommostWidgetBorder = widgetBottom;
            }   
        }, this);
        
        // calculate the old and the new height of the container
        var oldContainerHeight = dojo.style(this.node, "height");
        var newContainerHeight = bottommostWidgetBorder + 10;

        // if the heights are different or updateContainer is forced, do so
        if (this.updateContainer || oldContainerHeight != newContainerHeight){
            // set the style
            dojo.style(this.node, "height", newContainerHeight+"px");

            var ctrol = this.lm.find(_containerId).start();
            this._saveContainerSize(ctrol);
            this.updateContainer = false;
        }
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.widgetMoved", _currentNodeId);
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		
		this.isDragging = false;
				
		this.LOGGER.exiting("onDrop",[source, nodes, copy]);
	},
	
	_addNewWidget: function(iEvent){
		var newPlaceHolderID = iEvent;
		//judget whether widget is added to freeform layout rather than hidden area
		var _nws = dojo.query(".id-"+newPlaceHolderID, this.node);
		if (!_nws || !_nws[0]){
			return;
		}
		var lcCoords = dojo.coords(this.node, true);
		var lc = com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation(this.count, lcCoords);
		var _position;
		if (com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop) {
			_position = {
				x: Math.max(this._last_drop_X - lcCoords.x, 0),
				y: Math.max(this._last_drop_Y - lcCoords.y, 0),
				zIndex: this.count
			};
		}else{
			_position = {x:lc.x, y:lc.y, zIndex: lc.zIndex};
		}
		if(lcCoords.h < _position.y+300){
			this._saveContainerHeight(_position.y+300);		
		}
		com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition(newPlaceHolderID, _position);
		com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight(newPlaceHolderID);
		this.refreshCallback = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.layoutRefreshed", this, dojo.hitch(this, "_loadDefaultSize", newPlaceHolderID));
		
		com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = false;
		this.count++;
	},
	_saveContainerHeight:function(newHeight){
		dojo.style(this.node, "height", newHeight + "px");
		var _containerId =this.layoutParser.extractID(this.node);
		var ctrol = this.lm.find(_containerId).start();
		this._saveContainerSize(ctrol);	
	},
	
	_saveWidgetStyles:function(){
		dojo.query(".component-control", this.node).forEach(function(node, index, arr){
			var id =this.layoutParser.extractID(node);
			if(id){
				var lctrl = this.lm.find(id).start();
				if (lctrl) {
					lctrl.setMetaData("css-style", node.style.cssText);
				}
			}
		},this);
	},
	/**
	 * Synchronizes each z-Index in order to make sure that the current widget is always "on-top"
	 */
	_syncZindex:function(ctrlnode){
		var currentZ = ctrlnode.style.zIndex;
		// check if Z is set and if it is a number
		if (currentZ && !isNaN(currentZ)) {
			// get the integer value of current z
			var currentZInt = parseInt(currentZ);
			
			// iterate through all the other widgets
			dojo.query(".component-control", this.node).forEach(function(node, index, arr){
				var tempZ = node.style.zIndex;
				// check if temp Z is set and if it is a number
				if (tempZ && !isNaN(tempZ)) {
					var tempZInt = parseInt(tempZ);
					// check if tempZInt > 0 and Y= currentZInt
					if (tempZInt > 0 && tempZInt >= currentZInt) {
						tempZInt--;
						node.style.zIndex=tempZInt;
					}
				}
			});
		}
		ctrlnode.style.zIndex = this.count;
	},
	_loadDefaultSize: function(id){
		try{
			var newPlaceHolderDiv = this.layoutParser.findById(id);
			var widgetBody = dojo.query(".lotusWidgetBody", newPlaceHolderDiv)[0];
			var wb = dojo.query(".iw-iWidget",widgetBody)[0];
			if (wb && wb.id) {
                var wrapper = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel().find(wb.id);
                if (wrapper) {
					if (wrapper.getIWidgetDefinition().widgetDef) {
						var iitems = wrapper.getIDescriptorItems();
						var defaultHeight = iitems.getItemValue("defaultHeight");
						var defaultWidth = iitems.getItemValue("defaultWidth");
						var payload = null;
						if (defaultHeight || defaultWidth) {
							payload = {};
							if (defaultHeight) {
								payload.newHeight = defaultHeight;
							}
							if (defaultWidth) {
								payload.newWidth = defaultWidth;
							}
						}
						if (payload) {
							this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + wb.id, payload);
						}
					}
					else {
                        var that = this;
                        setTimeout(function() {
                            dojo.hitch(that, that._loadDefaultSize)(id)
                        }, (dojo.isIE<=7)?1000:100);
					}
				}
			}
			if(this.refreshCallback){
				this.evtSvc.unsubscribeEvent(this.refreshCallback);
				this.refreshCallback = null;
				this.newWidgetId = null;
			}
		}catch(er){
			this.LOGGER.severe("_loadDefaultSize", er.message);
		}
	},
	_insert: function(container, status, x){
		// summary: inserts a control to the new location
		// container: layout container into which to put the new control
		// status: HTTP status returned from request to get container
		// x: {layoutModel, controls {target, current dragging node}}
		this.LOGGER.entering("_insert",[container, status, x]);
		if(status != 200) {
			this.LOGGER.severe("_insert","Could not find container in the layout model.  Cannot move control.",[container, status, x]);
			return;
		}

		if(container instanceof com.ibm.mashups.enabler.layout.LayoutContainer){
			var current = x.layoutModel.find(this.layoutParser.extractID(x.controls.current)).start();
			
			if(!current){
				this.LOGGER.severe("_insert","Cannot find current dragging node in the layout model", [x.controls.current]);
			}
				
			var target = null;
			if (x.controls.target) { 
				target = x.layoutModel.find(this.layoutParser.extractID(x.controls.target)).start();
				if(!target){
					this.LOGGER.severe("_insert","Cannot find target node in the layout model", [x.controls.target]);
				}
			}
	
			if( x.layoutModel.confirmInsert(current,container,target) ){
				x.layoutModel.insert(current,container,target);
			}
			
			com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
			
		} else { 
			this.LOGGER.severe("_insert","Container is not an instance of com.ibm.mashups.enabler.layout.LayoutContainer",[container, status, x]); 
		}
		
		this.LOGGER.exiting("_insert",[container, status, x]);
	},
	onDndCancel: function(){
		if (dojo.dnd.manager().source !== this) { 
			return;
		}
		
		if (this.isDragging) {
            // get the selected nodes. This will be empty in case moved to another drop zone
            var selectedNodes = this.getSelectedNodes();
            
            // only simulate a drop if this was canceled and still is part of our area
            if (selectedNodes.length > 0) {
                //mouse move out of the layout, ondrop will not be called by dnd manager, manuall call it.
                this.onDrop(this,selectedNodes,false,true);
            
                ibmCfg.controllers.dnd.ghostHeight = 0;
            }
        }
        this.inherited(arguments);
		
		if (ibmCfg.controllers.dnd.currentNode && !ibmCfg.controllers.dnd.currentNode._isCanceled) {
					
			if (!dojo.isIE) {
				dojo.style(ibmCfg.controllers.dnd.currentNode, {display: ""});
			}
			
			ibmCfg.controllers.dnd.currentNode._isCanceled = true;
		}
		
		try{
			if(dojo.dnd.manager().target && dojo.dnd.manager().target.declaredClass != this.declaredClass && dojo.hasClass(dojo.dnd.manager().source.node, "ibmFreeform")){
				var currentId = this.layoutParser.extractID(dojo.dnd.manager().nodes[0]);	
				this.evtSvc.broadcastEvent("com.ibm.mashups.builder.leaveFreeformContainer"+"."+currentId);
			}
		}catch(e){}
		
		try{
			if(dojo.dnd.manager().target && dojo.dnd.manager().target.declaredClass == this.declaredClass && !dojo.hasClass(dojo.dnd.manager().source.node, "ibmFreeform")){
				var currentId = this.layoutParser.extractID(dojo.dnd.manager().nodes[0]);
				this.evtSvc.broadcastEvent("com.ibm.mashups.builder.enterFreeformContainer"+"."+currentId);
			}
		}catch(e){}
		
		this.current = null;
		this.destroyAllQuickWireTarget();
	},
	destroy: function(){
		this.evtSvc.unsubscribeEvent(this.resizeWidgetEvt);
		this.evtSvc.unsubscribeEvent(this.addNewWidget);
		this.count = 0;
		this.onOutEvent();
		this.inherited(arguments);
	},
	onDndStart: function(source, nodes, copy){
		this.inherited(arguments);
		this.isDragging = true;
		this._syncZindex(nodes[0]);
		
		var currentNode = dojo.byId(nodes[0].id);
		// current node stored in controller for use in creator
		ibmCfg.controllers.dnd.currentNode = currentNode;
		ibmCfg.controllers.dnd.currentNode._isCanceled = false;
		
		var sourceItem = source.getItem(nodes[0].id);
		if(sourceItem && (sourceItem.type != ibmCfg.controllers.dnd.SHELF_TYPE)){
			if (!dojo.isIE) {
				dojo.style(currentNode, {display: "none"});
			}
		}
	},
	onOverEvent: function(){
		if(!this.onmousemoveEvent){
			this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
		}
		
		dojo.dnd.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
		this.active = true;
	},
	
	onOutEvent: function(){
		if(this.onmousemoveEvent){
			dojo.disconnect(this.onmousemoveEvent);
			delete this.onmousemoveEvent;
		}
		
		dojo.dnd.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
		this.active = false;
	},
	

	onMouseUp: function(e){
		try {
			this._last_drop_X = e.pageX;
			this._last_drop_Y = e.pageY;
			com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseUp.call(this, e);
		} catch (e) {
			this.LOGGER.severe("onMouseUp", e.message);
		}
	},
	
	onMouseDown: function(e){
		com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseDown.call(this, e);
		this._last_layerX =  dojo.isOpera ? e.offsetX: e.layerX;
		this._last_layerY =  dojo.isOpera ? e.offsetY: e.layerY;
		
		// 19610: make sure z-index is respected when title bar is clicked
		var node = e.originalTarget;
        if (!node) {
			//20527 ie doesnt support originalTarget
			node = e.srcElement;
		}
		// work around "Permission denied to access property 'className'"
		try {
			while(!!node){
				if (dojo.hasClass(node, "component-control")) {
					this._syncZindex(node);
					break;
				}	
				node = node.parentNode;
			}
		} catch (e) {
			//console.warn("caught exception: ", e);
 		}
 
	},
	
	onMouseMove: function(e){
			if(this.isDragging && this.targetState == "Disabled"){ return; }
			dojo.dnd.Source.superclass.onMouseMove.call(this, e);
			var m = dojo.dnd.manager();
			if(!this.isDragging){
				if(this.mouseDown && this.isSource &&
						(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
					var nodes = this.getSelectedNodes();
					if(nodes.length){
						m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
					}
				}
			}
		
			//com.ibm.pb.dnd.layout.LayoutFreeformSource.superclass.onMouseMove.call(this, e); //don't call this for performance concern
			if (this.isDragging) {
				var m = dojo.dnd.manager();
				if (this === m.source && m.avatar && m.avatar.node && m.avatar.node.style.marginLeft === "0px" && (dojo.isOpera || e.layerX > 0)) 
				{
//					var ml = builderConfig.isBidi ? (parseInt(m.nodes[0].style.width, 10) - this._last_layerX) : this._last_layerX;
					var ml = this._last_layerX;
					dojo.style(m.avatar.node, {
						'marginLeft': -ml + "px",
						'marginTop' : "0px"
						//'marginTop': 34-this._last_layerY + "px",
					});
					//dojo.query(".dojoDndItem", m.avatar.node).style({	display: ""});
				}				
				
				if(!this.targetBox){
					this.targetBox = dojo.position(m.avatar.node, true);
				}
				this.topPos = e.pageY - this.targetBox.y;
				
				m.canDrop(true);
			}
	},
	
	_addItemClass: function(node, type){
		if(this.isDragging){
			if(type!="Anchor") {
				var _id = this.layoutParser.extractID(node);
				//20754:
				//  this.topPos is the top position of the item you are dragging
				//  node.style.top is the top position of the target
				//  this.quickWireHandlerHeight is the height of the target's title bar
				if(this.topPos > (node.style.top + this.quickWireHandlerHeight)) { //check if dropping on the target's title bar
					this.destroyQuickWireTarget(_id);
				} else {
					if(this.quickWireHandlers[_id]) {
						// do nothing
					} else {
						this.createQuickWireTarget(_id);
					}
				}
			}
		}
	},
	
	_removeItemClass: function(node, type){
		if(this.isDragging){
		}
	},
	
	moving: function(e){
		try {
			var m = dojo.dnd.manager();
			var avatar = m.avatar;
			if(!avatar){
				return false;
			}
			
			dojo.dnd.autoScroll(e);
			
			var a = dojo.coords(avatar.node);
			var ml = parseInt(avatar.node.style.marginLeft, 10) || 0;
			var mt = parseInt(avatar.node.style.marginTop, 10) || 0;
			//avatar container coords
			var ac = {
				l: (e.pageX + m.OFFSET_X),
				t: (e.pageY + m.OFFSET_Y),
				w: a.w - ml,
				h: a.h
			};
			
			var cst = dojo.coords(this.node, true);
			//source constrained area
			var c = {
				l:cst.x + 1 - ml,
				r:cst.x + cst.w - ml - ac.w, 
				t:cst.y - mt,
				b:cst.y + cst.h - ac.h
			};
			//limit four direction.
			avatar.node.style.left = (ac.l < c.l ? c.l : c.r < ac.l ? c.r : ac.l) + "px";
			avatar.node.style.top = (ac.t < c.t ? c.t : c.b < ac.t ? c.b : ac.t) + "px";
			//limit only top-left direction.
			//var al = ac.l < c.l ? c.l : ac.l;
			//var at = ac.t < c.t ? c.t : ac.t;
			//dojo.marginBox(avatar.node, {l: al, t: at});
			//adjust the width
			com.ibm.mm.builder.utils.pageWidthChanger.widgetToBeMoved(null, ac.l, ac.w);
			//adjust the height
			if (e.pageY > c.b) {
				this.updateContainer = true;
			   var newHeight = (e.pageY + ac.h);
			   dojo.style(this.node, "height", newHeight + "px");
			}
		} catch (err) {
			this.LOGGER.severe("moving", err.message);
		}
	},

	_saveContainerSize: function(ctrl){
		var cheight = dojo.style(this.node,"height");
		var sstyle = "height:"+cheight+"px;";
		ctrl.setMetaData("css-style", sstyle);
	},
	
	_saveWidgetSize: function(widgetId, layoutControlId, control){
		var node = dojo.byId(widgetId).parentNode;
		var layoutCtl = this.lm.find(layoutControlId).start();
		var tp = control.style.overflow;
		if(tp){
			control.style.removeProperty("overflow");
		};
		layoutCtl.setMetaData("css-style", control.style.cssText);
		layoutCtl.setMetaData("wrapper-css-style", node.style.cssText);
		if(tp){
			control.style.overflow = tp;
		};
	},
	
	__widgetResized: function(iEvent){
		var widgetId = iEvent.widgetId;
		var newH = iEvent.newHeight + 50;
		var control = this.layoutParser.findById(widgetId);
		var top = 0;
		if (control.style) {
			var top = dojo.style(control, "top");
			if (!top) 
				top = 0;
		}
		
		if (top + newH > this.height) {
			this.updateContainer = true;
			var newheight = top + newH + 50;
			this.height = newheight;
			this.node.style.height = newheight + "px";
		}
		if (this.updateContainer) {
			var ctrol = this.lm.find(this.layoutParser.extractID(this.node)).start();
			this._saveContainerSize(ctrol);
			this.updateContainer = false;
		}
	},
	
	createFixedWidthRangeProvider : function(width) {
	   return {
	      getMinimum : function() {
		     return width;
		  },
		  getMaximum : function() {
		     return width;
		  }
	   };
	}
 });
 
com.ibm.pb.dnd.layout.LayoutFreeformSource.creator = function(item, hint){
	if(hint=="avatar"){
		var _node = dojo.doc.createElement("div");
		if(dojo.dnd.manager().source.current){
			// use the portlet markup as avatar
			_node.innerHTML = String(item);
			var elem = dojo.byId(dojo.dnd.manager().source.current.id);
			var isPlaceHolder = false;
			//if(!dojo.hasClass(elem, "ibmPlaceholder")){
			var _id = this.layoutParser.extractID(dojo.dnd.manager().source.current);
			var widget = dojo.byId(_id);
			if(!widget){
				widget = dojo.query(".iw-iWidget", elem)[0];
				isPlaceHolder = true;
			}
			var m = dojo.contentBox(widget.parentNode);
			var ava_widget = dojo.query("[id~="+_id+"]",_node)[0]; 
			if(!ava_widget && isPlaceHolder){
				ava_widget = dojo.query(".iw-iWidget", _node)[0];
			}
			if(dojo.isFF){
				dojo._setMarginBox(ava_widget.parentNode, 0, 0, m.w + 17, m.h);
			}else{
				dojo._setMarginBox(ava_widget.parentNode, 0, 0, m.w, m.h);
			}
			if(isPlaceHolder){
				var size = dojo.contentBox(elem);
				dojo.style(_node, { width: size.w+"px", height: size.h+"px", opacity: 0.5});
			}else{
				dojo.style(_node, { width: elem.style.width, height: elem.style.height, opacity: 0.5});			
			}				
		}
		return {node: _node, data: item, type: [ibmCfg.controllers.dnd.DND_TYPE]};
	} else {  // this works the same way as the _normalizedCreator/onDropExternal functions in dojo.dnd.Source when there is no given creator function
		var node;
		if(dojo.isOpera) {
			node = dojo.byId(ibmCfg.controllers.dnd.currentNode.id).parentNode;  // opera returned nothing using dojo.query below
		} else {
			node = dojo.query("[id~="+ibmCfg.controllers.dnd.currentNode.id+"]")[0];
		}
		if(node){
			var source = ibmCfg.controllers.dnd.dndSources[this.layoutParser.extractID(node.parentNode)];
			if(source){
				var t = source.getItem(node.id);
				source.delItem(node.id);
				source.selection = {};  // prevents node from being deleted in dojo.dnd.Selector.deleteSelectedNodes called by onDropExternal()
				return {node: node, data: t.data, type: t.type};
			}
		} 
		return {node:dojo.create("div"),data:item,type:ibmCfg.controllers.dnd.SHELF_TYPE}
	}
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.withHandles = true;
com.ibm.pb.dnd.layout.LayoutFreeformSource.addWidgetByDrop = false;

com.ibm.mashups.services.ServiceManager.getService("eventService").subscribeEvent("com.ibm.mashups.builder.initWidgets", null, function(iEvent){
	try {
		var container = com.ibm.pb.layout.LayoutParser.findById(iEvent.containerId);
		var lcCoords = dojo.coords(container, true);
		for(var i = 0; i < iEvent.data.length; i++){
			var item = iEvent.data[i];
			var widgetId = item.widgetId;
			var idx = item.index;
			var lc = com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation(idx, lcCoords);
			com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition(widgetId, lc);
			com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight(widgetId);
		}
		com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().commit().start();
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(false);
	}catch(e){
		this.LOGGER.severe("com.ibm.mashups.builder.initWidgets", e.message);
	}
});

com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation = function(index, coords){
	try{
		var maxWidth = coords.w;
		if(!index) index = 0;
		if (index * 30 < maxWidth-400) {
			return {
				x: 30 * index,
				y: 30 * index,
				zIndex: index+1
			}
		}else{
			return {
				x: (30 * index) % (maxWidth-400),
				y: 30 * index,
				zIndex: index+1
			}
		}
	}catch(e){
		this.LOGGER.severe("com.ibm.pb.dnd.layout.LayoutFreeformSource.getWidgetLocation", e.message);
	}
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition = function(widgetId, position){
	try {
		var _style = "position:absolute;left:" + position.x + "px;top:" + position.y + "px;width:400px;z-index:" + position.zIndex + ";";
		var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
		var ctl = lm.find(widgetId).start();
		ctl.setMetaData("css-style", _style);
		var control = com.ibm.pb.layout.LayoutParser.findById(widgetId);
		if (control) {
			control.style.position = "absolute";
			control.style.left = position.x + "px";
			control.style.top = position.y + "px";
			control.style.width = "400px";
			control.style.zIndex = position.zIndex;
		}
	}catch(e){
		this.LOGGER.severe("com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetPosition", e.message);
	};
};
com.ibm.pb.dnd.layout.LayoutFreeformSource.setWidgetInitHeight = function(widgetId){
	var currentPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
	var lm = ibmCfg.controllers.dnd.nm.getLayoutModel(currentPageId);
	var control = com.ibm.pb.layout.LayoutParser.findById(widgetId);
	var outheight = 300;
	if (control.style.height){
		outheight = parseInt(control.style.height);
	}else{
		control.style.height = outheight + "px";
	}
	if(control){
		var wb = dojo.query(".lotusWidgetBody", control)[0];
		var innerheight = outheight - 50;
		if(wb) wb.style.height = innerheight+"px";
		var layoutCtl = lm.find(widgetId).start();
		layoutCtl.setMetaData("widget-body-height", innerheight+"px");
		layoutCtl.setMetaData("css-style", control.style.cssText);
	}
};
