#************************************************************************
# Licensed Materials - Property of IBM
# 5724-M24
# Copyright IBM Corporation 2011.  All rights reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#************************************************************************

###########################################################################
# Run the ConfigForHTTP admin task to update the web.xml in order to allow
# Business Space to use HTTP in addition to HTTPS
#
###########################################################################

import sys, os
import zipfile
import shutil
import AdminUtilities
import tempfile
from java.util import Properties
import java.io.File as File;
from java.io import *
from java.lang import *
from java.util.zip import *

#*******************************************************************************************************
# Get application deployment target 
#*******************************************************************************************************
def getAppDeploymentTarget (appName, failonerror=AdminUtilities._BLANK_ ):
        if (failonerror==AdminUtilities._BLANK_): 
	        failonerror=AdminUtilities._FAIL_ON_ERROR_
	#endIf
	msgPrefix = "getAppDeploymentTarget("+`appName`+", "+`failonerror`+"): "
        
        try:
                #--------------------------------------------------------------------
		# Set up globals
		#--------------------------------------------------------------------
		global AdminApp
                global AdminConfig
                
                #--------------------------------------------------------------------
		# Get application deployment target
		#--------------------------------------------------------------------
                targets = ""
                
                # Check the required parameters
                if (appName == ""):
                   raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6041E", ["appName", appName]))
                
                # Retrieve application deployment configuration object
                deployment = AdminConfig.getid("/Deployment:"+appName+"/" )

                # Check if deployment target exists
                if (len(deployment) == 0):
                    raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6040E", ["appName", appName]))
                else:
                   targets = AdminConfig.showAttribute(deployment, "deploymentTargets")
                   targets = AdminUtilities.convertToList(targets)
                #endIf
                return targets
        except:
		typ, val, tb = sys.exc_info()
		if (typ==SystemExit):  raise SystemExit,`val`
                if (failonerror != "true"):
                   print "Exception: %s %s " % (sys.exc_type, sys.exc_value)
		   val = "%s %s" % (sys.exc_type, sys.exc_value)
	           raise "ScriptLibraryException: ", `val`
                   return -1
                else:   
                   return AdminUtilities.fail(msgPrefix+AdminUtilities.getExceptionText(typ, val, tb), failonerror)
                #endIf
        #endTry
	AdminUtilities.infoNotice(AdminUtilities._OK_+msgPrefix)
	#return 1  # succeed
#endDef

##################################################################
# exportAnAppToFile
##################################################################
def exportAnAppToFile( appName, exportFile, failonerror=AdminUtilities._BLANK_):
        if(failonerror==AdminUtilities._BLANK_): 
		failonerror=AdminUtilities._FAIL_ON_ERROR_
	#endIf
	msgPrefix = "exportAnAppToFile("+`appName`+", "+`exportFile`+", "+`failonerror`+"): "

        try:
                #--------------------------------------------------------------------
		# Set up globals
		#--------------------------------------------------------------------
		global AdminApp
                
                #--------------------------------------------------------------------
		# export an application to a file 
		#--------------------------------------------------------------------
                if (appName == ""):
                   raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6041E", ["appName", appName]))
                
                if (exportFile == ""):
                   raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6041E", ["exportFile", exportFile]))
                   
                # Identify deployment configuration object
                deployment = AdminConfig.getid("/Deployment:"+appName+"/")
                if (len(deployment) == 0):
                   raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6040E", ["appName", appName]))
                
                AdminApp.export(appName, exportFile)
                if (AdminUtilities._AUTOSAVE_ == "true"):
                   AdminConfig.save()
        except:
		typ, val, tb = sys.exc_info()
		if (typ==SystemExit):  raise SystemExit,`val`
                if (failonerror != "true"):
                   print "Exception: %s %s " % (sys.exc_type, sys.exc_value)
		   val = "%s %s" % (sys.exc_type, sys.exc_value)
	           raise "ScriptLibraryException: ", `val`
                   return -1
                else:   
                   return AdminUtilities.fail(msgPrefix+AdminUtilities.getExceptionText(typ, val, tb), failonerror)
                #endIf
                
	#endTry
	return 1  # succeed
#endDef


##################################################################
# syncActiveNodes method for use with ND 
##################################################################
def syncActiveNodes( failonerror=AdminUtilities._BLANK_ ):
    if (failonerror==AdminUtilities._BLANK_): 
        failonerror=AdminUtilities._FAIL_ON_ERROR_
    #endIf
    msgPrefix = "syncActiveNodes("+`failonerror`+"): "

    try:
    	print "Updating the following Nodes:"
    	print " "
        dp = AdminControl.queryNames("type=DeploymentManager,*")
        # WASL6044E=WASL6044E: The {0}:{1} mbean is not running.
	if (len(dp) == 0):
	    raise AttributeError(AdminUtilities._formatNLS(resourceBundle, "WASL6044E", ["DeploymentManager", "DeploymentManager"]))
        else:
            result = AdminControl.invoke(dp, "syncActiveNodes", "true")
	    print result
            return 1
    except:
        typ, val, tb = sys.exc_info()
        if (typ==SystemExit):  raise SystemExit,`val`
        if (failonerror != AdminUtilities._TRUE_):
            print "Exception: %s %s " % (sys.exc_type, sys.exc_value)
	    val = "%s %s" % (sys.exc_type, sys.exc_value)
	    raise "ScriptLibraryException: ", `val`
            return -1
        else:   
            return AdminUtilities.fail(msgPrefix+AdminUtilities.getExceptionText(typ, val, tb), failonerror)
        #endIf
    #endTry
    AdminAdminUtilities.infoNotice(AdminUtilities._OK_+msgPrefix)
#endDef

#*******************************************************************************************************
#This function converts path to unix style
#*******************************************************************************************************
def convertPathToUnixStyle(pathName):
  newPath = File(pathName);
  newPath = newPath.getAbsolutePath()
  return newPath.replace('\\',"/");
#endDef

#*******************************************************************************************************
#This function extracts the web.xml from an ear 
# making generic so it works for mm.runtime.was.war, process-portal-support.war or process-portal.war 
#*******************************************************************************************************
def extractWebXml(warFileName, earFileName):
	print 'extractWebXml::warFile =' + warFileName + ' earFileName =' + earFileName 
	earFile = ZipFile(earFileName)
    	warEntry = ZipEntry(warFileName)
    	input1 = earFile.getInputStream(warEntry)
    	warFile = File(washome + '/temp/BusinessSpace/scripts/working/' + warFileName);
    	outputStream1 = FileOutputStream(warFile);
        read1 = input1.read();
        while (read1 >= 0) :
            outputStream1.write(read1);
            read1 = input1.read();
        
        input1.close();
        outputStream1.close();
        earFile.close();
        
        warZipFile = ZipFile(washome + '/temp/BusinessSpace/scripts/working/' + warFileName);
    	webXMLEntry = ZipEntry('WEB-INF/web.xml');
    	input2 = warZipFile.getInputStream(webXMLEntry);
    	XMLFile = File(washome + '/temp/BusinessSpace/scripts/working/web.xml');
    	outputStream2 = FileOutputStream(XMLFile);
	read2 = input2.read();
        while (read2 >= 0) :
            outputStream2.write(read2);
            read2 = input2.read();
    	
        input2.close();
        outputStream2.close();
        warZipFile.close();
 
#endDef


#*******************************************************************************************************
# Is this an https only web.xml
#*******************************************************************************************************
def isHTTPSOnly():

	returnValue = 'false'

	webfile = open(washome + '/temp/BusinessSpace/scripts/working/web.xml')
	while 1:
		line = webfile.readline()
		if not line: break
		if (line.find("<transport-guarantee>CONFIDENTIAL</transport-guarantee>") > -1):
			returnValue = 'true'	    
	webfile.close()

	return returnValue;
#endDef


#*******************************************************************************************************
# Is this an https only web.xml
#*******************************************************************************************************
def configureWebXml(transportSetting):

	webfile = open(washome + '/temp/BusinessSpace/scripts/working/web.xml')
	newlines = []
	while 1:
		line = webfile.readline()
		if not line: break
		if transportSetting == 'httpsonly':
			line = line.replace("<transport-guarantee>NONE</transport-guarantee>","<transport-guarantee>CONFIDENTIAL</transport-guarantee>")
		else:
			line = line.replace("<transport-guarantee>CONFIDENTIAL</transport-guarantee>","<transport-guarantee>NONE</transport-guarantee>")
		newlines.append(line)

	webfile.close()

	outfile = open(washome + '/temp/BusinessSpace/scripts/working/web.xml', 'w')
	outfile.writelines(newlines)
	outfile.close()	

#endDef


#*******************************************************************************************************
# Get the bspace.transport setting
#*******************************************************************************************************
def getBSpaceTransportConfig(appName, altClusterName):

# BSpace and Mashups apps store a config property in the ConfgiService.properties file
# Process Portal also needs to use the same config property to keep both sets of wars in sync
# This is needed to maintain the same transport(https/http) for BSpace and Process Portal.
# In a cluster, it is not always possible to have the BSpace config path on the cluster where Process Portal is deployed
# In the golden topology, for e.g. the Process Portal is on the AppTarget cluster while BSpace is not.
# Therefore, we will use only Mashups ear to determine the path to config service properties.
# This way, we keep most of the script's logic untouched
	for app in apps:
		if (app.startswith('mm.was_')):
			appName=app
			print 'configBSpaceTransport::getBSpaceTransportConfig - using app name =' + appName
		

	returnValue = ''

	transportProp = None

	prop = Properties()
				
	prop.load(FileInputStream(getConfigDirectory(appName, altClusterName)))
	transportProp = prop.getProperty("bspace.transport")
	print "transportProp=" + transportProp
		
	return transportProp	
			
#endDef


#*******************************************************************************************************
# Get the configuration directory
#*******************************************************************************************************
def getConfigDirectory (appName, altClusterName):

	deployTargets = getAppDeploymentTarget(appName)
	
	directoryPath = None
	
	profileRoot = System.getProperty("user.install.root")
	profileRoot = convertPathToUnixStyle(profileRoot)

	for deployTarget in deployTargets:
			
		if (AdminConfig.getObjectType(deployTarget) == 'ServerTarget'):


			nodeName = AdminConfig.showAttribute(deployTarget, "nodeName")

			serverName = AdminConfig.showAttribute(deployTarget, "name")

			if (serverName != None and nodeName != None):

				if (appName.find(nodeName + "_" + serverName) > -1):

					directoryPath = profileRoot + "/BusinessSpace/" + nodeName + "/" + serverName + "/mm.runtime.prof/config/ConfigService.properties"

			continue
		

		if (AdminConfig.getObjectType(deployTarget) == 'ClusteredTarget'):

			clusterName = AdminConfig.showAttribute(deployTarget, "name")
			
			if (altClusterName != None):
				directoryPath = profileRoot + "/BusinessSpace/" + altClusterName + "/mm.runtime.prof/config/ConfigService.properties"
			elif (appName.find(clusterName) > -1):
				directoryPath = profileRoot + "/BusinessSpace/" + clusterName + "/mm.runtime.prof/config/ConfigService.properties"			
			
			continue


	return directoryPath
#endDef


#*******************************************************************************************************
# Set the bspace.transport setting
#*******************************************************************************************************
def setBSpaceTransportConfig(transportPropSetting, appName, altClusterName):

	propFileName = getConfigDirectory(appName, altClusterName)

	propfile = open(propFileName)
	newlines = []
	while 1:
		line = propfile.readline()
		if not line: break
		
		if line.strip().startswith("bspace.transport="):
			line = "bspace.transport=" + transportPropSetting + "\n"
		
		newlines.append(line)

	propfile.close()

	outfile = open(propFileName, 'w')
	outfile.writelines(newlines)
	outfile.close()	
	
	deployTargets = getAppDeploymentTarget(appName)
	
	#change the path for the os
	propFileName = os.path.normcase(propFileName)
	
	for deployTarget in deployTargets:
			
		if (AdminConfig.getObjectType(deployTarget) == 'ServerTarget'):


			nodeName = AdminConfig.showAttribute(deployTarget, "nodeName")

			serverName = AdminConfig.showAttribute(deployTarget, "name")

			if (serverName != None and nodeName != None):

				if (appName.find(nodeName + "_" + serverName) > -1):
					print 'update property config' 
				
					AdminTask.updatePropertyConfig('[-serverName ' + serverName + ' -nodeName ' + nodeName + ' -propertyFileName "' + propFileName + '" -prefix "Mashups_"]')
					
				continue
		

		if (AdminConfig.getObjectType(deployTarget) == 'ClusteredTarget'):

			clusterName = AdminConfig.showAttribute(deployTarget, "name")
			
			if (appName.find(clusterName) > -1):
				AdminTask.updatePropertyConfig('[-clusterName ' + clusterName + ' -propertyFileName "' + propFileName + '" -prefix "Mashups_"]')
				
			continue
					
			
#endDef


##################################################################
# Check the arguments
##################################################################


validArgs = 'true'

transportType = ''
useDefaultSetting = 'false'

clusterName = None
nodeName = None
serverName = None
bspaceClusterName = None
	
counter = 0

try:

	while counter < len(sys.argv):
		if sys.argv[counter] == '-clusterName': 
			clusterName = sys.argv[counter+1]
			counter = counter + 2
		elif sys.argv[counter] == '-bspaceClusterName': 
			bspaceClusterName = sys.argv[counter+1]
			counter = counter + 2		
		elif sys.argv[counter] == '-nodeName': 
			nodeName = sys.argv[counter+1]
			counter = counter + 2		
		elif sys.argv[counter] == '-serverName': 
			serverName = sys.argv[counter+1]
			counter = counter + 2
		elif sys.argv[counter] == '-httpsonly': 
			transportType='httpsonly'
			counter = counter + 1
		elif sys.argv[counter] == '-allowhttp': 
			transportType='allowhttp'
			counter = counter + 1
		elif sys.argv[counter] == '-usedefault': 
			transportType='usedefault'
			useDefaultSetting = 'true'
			counter = counter + 1
		else: 
			validArgs = 'false'
			break
	
except:
	validArgs = 'false'	


if (transportType == None):
	validArgs = 'false'

if ((nodeName == None and serverName != None) or (nodeName != None and serverName == None)):
	validArgs = 'false'

if ((clusterName != None and (nodeName != None or serverName != None))):
	validArgs = 'false'
		
	
if validArgs == 'false':
	print ''
	print 'Usage:  configBSpaceTransport -option -nodeName node -serverName server -clusterName cluster'
	print '        where -option is one of the following:'
	print '        -httpsonly   -allow only https connections'
	print '        -allowhttp   -allow both https and http connections'
	print '        -usedefault  -update the connection transport with the BSpace default'
	print ''
	print '        -nodeName   -optional parameter to specifiy the node'
	print '        -serverName   -optional parameter to specifiy the server'
	print '        -clusterName   -optional parameter to specifiy the cluster'
	print '        -bspaceClusterName   - optional parameter to specify the Business Space cluster'
	print ''
	print '        Note:   serverName and nodeName must be used together'
	
	sys.exit(0)


##################################################################
# Get the WAS Home
##################################################################

# get the WAS Home directory
washome = System.getProperty('was.install.root')


##################################################################
# Get the Profile Root
##################################################################

profileRoot = System.getProperty("user.install.root")
profileRoot = convertPathToUnixStyle(profileRoot)

##################################################################
# Get the cell type
##################################################################

cellID = AdminConfig.list("Cell");
cellType = AdminConfig.showAttribute(cellID, 'cellType');


##################################################################
# Get the list of all apps and convert to a list
##################################################################

apps = AdminApp.list()
apps = AdminUtilities.convertToList(apps)


##################################################################
# Create a working directory
##################################################################

#create a working directory for extracting the war file
if not os.path.exists(washome + '/temp/BusinessSpace/scripts/working'):
	os.makedirs(washome + '/temp/BusinessSpace/scripts/working')


#########################################################################
# Update each mm.was.ear and IBM_BPM_Portal ears with the updated web.xml
#########################################################################

updateRequired = 'false'

specifiedAppName = ''

if (nodeName != None and serverName != None):
	specifiedAppName = nodeName + '_' + serverName
	print 'Attempting to update: mm.was_' + specifiedAppName + ' and IBM_BPM_Portal'+specifiedAppName

if (clusterName != None):
	specifiedAppName = clusterName
	print 'Attempting to update: mm.was_' + specifiedAppName + ' and IBM_BPM_Portal'+specifiedAppName

def getWarsInEar(ear):
	modules = AdminApp.listModules(ear)
	modules = AdminUtilities.convertToList(modules)
	wars = []
	for module in modules:
		if module.find('.war') > 0 :
			start = module.find('#') +1
			end =  module.find('+')
			wars.insert(len(wars),module[start:end])
	return wars	
	
for app in apps:
	if (app.startswith('mm.was_' + specifiedAppName) or app.startswith('IBM_BPM_Portal_' + specifiedAppName)):
		
		print 'Default transport config: ' + getBSpaceTransportConfig(app, bspaceClusterName)
		
		if (useDefaultSetting == 'true'):
			transportType = getBSpaceTransportConfig(app, bspaceClusterName)
			print 'Retrieved default transport: ' + transportType
	
		print 'Updating application ' + app
		exportAnAppToFile(app, washome + '/temp/BusinessSpace/scripts/working/' + app + '.ear')
		
		#get war modules
		
		wars = getWarsInEar(app)
		
		for war in wars:
		
			extractWebXml(war, washome + '/temp/BusinessSpace/scripts/working/' + app + '.ear')
		
			if (transportType == 'allowhttp' and isHTTPSOnly() == 'false'):
				continue

			if (transportType == 'httpsonly' and isHTTPSOnly() == 'true'):
				continue

			if (transportType == 'allowhttp' and isHTTPSOnly() == 'true'):
				configureWebXml(transportType)
				print 'about to update web module '+ war + ' to allowhttp in application named'+ app
				AdminApp.update(app, 'file', '[-operation update -contents "' + washome + '/temp/BusinessSpace/scripts/working/web.xml " -contenturi ' + war +'/WEB-INF/web.xml]')
				updateRequired = 'true'
		
			if (transportType == 'httpsonly' and isHTTPSOnly() == 'false'):
				configureWebXml(transportType)
				print 'about to update web module '+ war + ' to httpsonly in application named'+ app
				AdminApp.update(app, 'file', '[-operation update -contents "' + washome + '/temp/BusinessSpace/scripts/working/web.xml " -contenturi ' + war + '/WEB-INF/web.xml]')
				updateRequired = 'true'
			
			if (useDefaultSetting != 'true'):	
				setBSpaceTransportConfig(transportType, app, bspaceClusterName)
			

##################################################################
# Save the changes
##################################################################

if (updateRequired == 'true'):
	AdminConfig.save()
		
	##################################################################
	# If this is a DMGR, then synchronize the active nodes
	##################################################################
	if (cellType == "DISTRIBUTED" and useDefaultSetting == 'false'):
		syncActiveNodes()		
else:
	print 'No configuration is necessary.'

##################################################################
# Remove the working directory
##################################################################

shutil.rmtree(washome + '/temp/BusinessSpace/scripts/working')


##################################################################
# Script complete
##################################################################

print " "
print "Configuration complete"
