# BEGIN COPYRIGHT
# *************************************************************************
#
# Licensed Materials - Property of IBM                              
#                                                                   
# 5724-M24                                                         
#                                                                   
# Copyright IBM Corp. 2009,2010  All Rights Reserved.                    
#                                                                   
# US Government Users Restricted Rights - Use, duplication or       
# disclosure restricted by GSA ADP Schedule Contract with           
# IBM Corp.                          
#
# *************************************************************************
# END COPYRIGHT

#This script can be called from a command-line wrapper script or from the profile upgrade framework.
#If called from the framework, then the System properties will hold the relevant parameters.
#If called from the command-line then the expected parameters are either: 
#[wasHome, clusterName] or [wasHome, nodeName, serverName]

from java.lang import *
from java.io import *
from java.util.zip import *
from java.util import Properties
import os
import shutil
import zipfile
import re
#import configBSpaceTransport

False = 0
True = 1

#*******************************************************************************************************
#This function implements the profile upgrade framework definition that accepts
#an appPrefix, nodeName, serverName, and clusterName).  It will be called by the
#framework function updateDeploymentTargetsOf.  Either the nodeName/serverName pair will be specified
#or the clusterName parameter will be specified.  The others will be None
#wasHome is expected to be previously defined
#*******************************************************************************************************
def upgradeBSpaceProfile(appName, nodeName, serverName, clusterName):
  print "Enter upgradeBSpaceProfile()... "

  if ( clusterName is not None ):
    scope = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName
    updateScopeArgs = '-clusterName "' + clusterName + '"'
    #mmRuntimeProfPath = profileRoot + '/BusinessSpace/' + clusterName + '/mm.runtime.prof'
    
  elif ( (nodeName is not None) and (serverName is not None) ):
    scope = 'WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName
    updateScopeArgs = '-serverName "' + serverName + '" -nodeName "' + nodeName + '"'
    #mmRuntimeProfPath = profileRoot + '/BusinessSpace/' + nodeName + '/' + serverName+ '/mm.runtime.prof'
    #check serverType, upgrade is only required for application server
    server = AdminConfig.getid(getScopePath(cellName, nodeName, serverName))
    if AdminConfig.showAttribute(server, "serverType") != "APPLICATION_SERVER":
      print "Target is not an application server, skipping Business space profile upgrade for " + nodeName + "_" + serverName
      return
  #endif

  #get subfix name of application  
  appSubfixName = appName[appName.index("_"):]
 
  #get the version numbers of the deployed and installed ears for comparison
  deployedVersion = getDeployedVersionId(appSubfixName)
  upgradeVersion = getInstalledVersionId()

  #check to see if the profile should be upgraded
  upgradeProfileFlag = 1;
  if ( (deployedVersion != "") and (upgradeVersion != "") ):
    upgradeProfileFlag = int(deployedVersion) < int(upgradeVersion)

  print "Upgrade the profile?: " + repr(upgradeProfileFlag)
  
  if (upgradeProfileFlag):
    if ( clusterName is not None ):
      AdminTask.updateBusinessSpaceRuntimeForMigration('[-clusterName ' + clusterName + ' -sourceVersion v8011]')
    elif ( (nodeName is not None) and (serverName is not None) ):
      AdminTask.updateBusinessSpaceRuntimeForMigration('[-nodeName ' + nodeName + ' -serverName ' + serverName + ' -sourceVersion v8011]')
    #endif   
    
    #save
    AdminConfig.save()    
    
    sys.argv = ['-usedefault']
    execfile(wasHome + '/BusinessSpace/scripts/configBSpaceTransport.py', {})
    #configBSpaceTransport.main('-allowhttp')
    
  else:
    #The profile upgrade has already completed for this target, no need to rerun.
    print "The profile upgrade is not necessary"
  #endif
  
  print "Exit upgradeBSpaceProfile()... "
#enddef

#------------------------------------------------------------------------------
# This function updates the profile-level configuration for all nodes
# affected by this invocation (may include co-located managed nodes)
#------------------------------------------------------------------------------
def updateProfileConfiguration():
  print "Entering updateProfileConfiguration"
  nodeNames = getNodeNames()
  profileRoots = map(lambda nodeName: AdminTask.showVariables(["-variableName", "USER_INSTALL_ROOT", "-scope", "Cell=" + cellName + ",Node=" + nodeName]), nodeNames)
  print "Need to update the following profile locations: " + str(profileRoots)
  for profileRoot in profileRoots:
    print "Profile root -- " + profileRoot
    updateImportingFlags(profileRoot)
    copyBusinessSpaceSystemFiles(profileRoot)
    clearBSpaceHelpCache(profileRoot)

  
  print "Exiting updateProfileConfiguration"
#enddef

#------------------------------------------------------------------------------
# Get the list of nodes which are to be updated by the current invocation
#------------------------------------------------------------------------------
def getNodeNames():
  nodeNames = [ System.getProperty("local.node") ]
  	
  managedNodes = System.getProperty("bpm.upgrade.managed.nodes")
  
  if managedNodes:
    nodeNames.extend(managedNodes[1:-1].split())
  #endif  
  
  return nodeNames
#enddef

#-----------------------------------------------------------------
# Update Business Space v80 importing flags
#-----------------------------------------------------------------
def updateImportingFlags(profileRoot):
        print "Entering updateImportingFlags"
        
        fileName = "oobLoadedStatus.properties"
        
	if existsPath(profileRoot + '/BusinessSpace'):
		find_file_by_pattern(profileRoot + '/BusinessSpace')
	#endif
	
	print final_file_list
	
	for refile in final_file_list:
		if File(refile).getName() == fileName:
			prop = Properties()				
			prop.load(FileInputStream(refile))			
			importThemes = prop.getProperty("importThemes.txt")
			prop.setProperty("importThemes.txt", 'true')
			importSpaces = prop.getProperty("importSpaces.txt")
			prop.setProperty("importSpaces.txt", 'true')
			importTemplates = prop.getProperty("importTemplates.txt")
			prop.setProperty("importTemplates.txt", 'true')
			#print "importThemes=" + importThemes
			prop.save(FileOutputStream(refile), '')
		#endif
	
	print "Exiting updateImportingFlags"
#endDef

#-----------------------------------------------------------------
# 
#-----------------------------------------------------------------
def find_file_by_pattern(parent):
	pattern='.*'
	re_file = re.compile(pattern)
	
	cur_list = os.listdir(parent) 
	
	for item in cur_list:
		full_path = os.path.join(parent, item)
		#print full_path 
		if os.path.isfile(full_path):
			final_file_list.append(full_path) 
		else: 
			find_file_by_pattern(full_path) 
	
	#return final_file_list
#endDef

#-----------------------------------------------------------------
#
#-----------------------------------------------------------------
def find_dir_by_pattern(parent):
	pattern='.*'
	re_file = re.compile(pattern)
	
	cur_list = os.listdir(parent) 
	
	for item in cur_list:
		full_path = os.path.join(parent, item)
		#print full_path 
		if not os.path.isfile(full_path):
			final_dir_list.append(full_path) 
			find_dir_by_pattern(full_path) 
		#endif
		
	#return final_dir_list
#endDef

#-------------------------------------------------------------------------
# Clear the Business Space help cache directory under <profileRoot>/temp
#-------------------------------------------------------------------------
def clearBSpaceHelpCache(profileRoot):
        print "Entering clearBSpaceHelpCache"
	if existsPath(profileRoot + '/temp'):
		find_dir_by_pattern(profileRoot + '/temp')
	#endif
	
	print final_dir_list
	
	for refile in final_dir_list:
		if File(refile).getName().startswith('BSpaceHelp_'):
			delete_file_folder(refile)
		#endif
	
	print "Exiting clearBSpaceHelpCache"
#endDef


#-----------------------------------------------------------------
# Copy Business Space v80 system files
#-----------------------------------------------------------------
def copyBusinessSpaceSystemFiles(profileRoot):
        print "Entering copyBusinessSpaceSystemFiles"
	if existsPath(profileRoot + '/BusinessSpace'):
		if existsPath(profileRoot + '/BusinessSpace.system'):
		        delete_file_folder(profileRoot + '/BusinessSpace.system')
		#endif
		copyDirectory(wasHome + '/BusinessSpace/mm.runtime/public/files', profileRoot + '/BusinessSpace.system/files')
		copyDirectory(wasHome + '/BusinessSpace/themes', profileRoot + '/BusinessSpace.system/themes')
	#endif
	print "Exiting copyBusinessSpaceSystemFiles"
#endDef

#-----------------------------------------------------------------
# Utility -- copy directory
#-----------------------------------------------------------------
def copyDirectory(sourceDir, targetDir):
	if not os.path.exists(targetDir):   
                os.makedirs(targetDir)
	
	#copyFileCounts = 0
    
        for f in os.listdir(sourceDir):   
                sourceF = os.path.join(sourceDir, f)   
                targetF = os.path.join(targetDir, f)   
                 
                if os.path.isfile(sourceF):                        
                        #copyFileCounts += 1
                        shutil.copy(sourceF, targetF)
                elif os.path.isdir(sourceF):   
                        copyDirectory(sourceF, targetF)
                #endif	
#endDef

#-----------------------------------------------------------------
# Utility -- delete file / directory
#-----------------------------------------------------------------
def delete_file_folder(src):
	if os.path.isfile(src):
		try:
			os.remove(src)
		except:
			pass
	elif os.path.isdir(src):
		for item in os.listdir(src):
			itemsrc=os.path.join(src,item)
			delete_file_folder(itemsrc)
		try:
			os.rmdir(src)
		except:
			pass
#endDef

#-----------------------------------------------------------------
# Path utilities: 
# exists: return 1;
# not exist: return 0;
#-----------------------------------------------------------------
def existsPath(pathName):
	newPath=File(pathName);
	return newPath.exists();
#endDef

#*******************************************************************************************************
#This function returns the version string of the deployed BSpace ear
#*******************************************************************************************************
def getDeployedVersionId(appSubfixName):
  print "Enter getDeployedVersionId()... "

  #Find and return the BSpace version number of the deployed BSpace app 
  deployedVersion = "";
  versionArray = AdminApp.view('BSpaceEAR' + appSubfixName,'[-buildVersion]').split()
  #print "versionArray:  " + repr(len(versionArray))
  if ((versionArray is not None) and (len(versionArray) >= 4) and (versionArray[3] is not None)):
    print "versionArray:  " + versionArray[3]
    deployedVersion = stripVersion(versionArray[3])
    print "Deployed version of the Business Space EAR:  " + deployedVersion
  #endif
  
  print "Exit getDeployedVersionId()... "
  return deployedVersion
#enddef

#*******************************************************************************************************
#This function returns the version string of the newly installed BSpace ear
#*******************************************************************************************************
def getInstalledVersionId():
  print "Enter getInstalledVersionId()... "
  
  #Find the version of the BSpace EAR in installableApps for comparison
  #This is stored in the MANIFEST.MF file
  upgradeVersion = ""
  
  zf = ZipFile(wasHome + '//installableApps/BusinessSpace/BSpaceEAR.ear')
  ze = ZipEntry('META-INF/MANIFEST.MF')
  br = BufferedReader(InputStreamReader(zf.getInputStream(ze)))
  
  line = br.readLine()
  while line:
    if (line.startswith('Implementation-Version')):
      print "line: " + line
      versionLine = line.split(':')
      if ((versionLine is not None) and (len(versionLine) == 2)):
        upgradeVersion = stripVersion(versionLine[1].lstrip())
        print "Installed version of the Business Space EAR: " + upgradeVersion
      #endif
    #endif
    line = br.readLine()
  #endwhile
  
  br.close()
  zf.close()
  
  print "Exit getInstalledVersionId()... "
  return upgradeVersion
#enddef

#*******************************************************************************************************
#This function takes a full BSpace version string [product number]-[YYYYMMDD]_[HHMM] and returns, just the
#date part of the version id
#Note from 7004 onwards the version string looks like this: [product number]-[YYYYMMDD]-[HHMM]
#*******************************************************************************************************
def stripVersion(fullVersionId):
  print "Enter stripVersion()... "
  print "Full version Id: " + fullVersionId;

  version = ""
  lindex=0
  rIndex=0
  if (fullVersionId.find("_") > -1):
  	lIndex = fullVersionId.rfind("-")
  	rIndex = fullVersionId.find("_")
  	version = fullVersionId[lIndex+1:rIndex]
  else:
  	#lIndex = fullVersionId.find("-")
  	#rIndex = fullVersionId.rfind("-")
  	versionLine = fullVersionId.split('-')
  	if ((versionLine is not None) and (len(versionLine) == 3)):
        	version = versionLine[1].lstrip()
        elif ((versionLine is not None) and (len(versionLine) == 4)):
        	version = versionLine[2].lstrip()
      	#endif  	
  
  if (version == ""):
    print "Warning: Can not determine version"
  #endif
  
  print "Exit stripVersion()... "
  return version
#enddef

#*******************************************************************************************************
#This function converts path to unix style
#*******************************************************************************************************
def convertPathToUnixStyle(pathName):
  newPath = File(pathName);
  newPath = newPath.getAbsolutePath()
  return newPath.replace('\\',"/");
#endDef

#--------------------------------------------------------------
# Helper Class to normalize the string for comparision
#--------------------------------------------------------------
def wsadminToList(inStr):
	print "Enter wsadminToList()... "
	outList=[]
	if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
		tmpList = inStr[1:-1].split() #splits space-separated lists,
	else:
		tmpList = inStr.split("\n")   #splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip();         #removes any Windows "\r"
		if (len(item)>0):
			outList.append(item)
	
	print "Exit wsadminToList()... "
	return outList;
#endDef

#------------------------------------------------------------------------------
# get the scope path according to cellname, nodename, servername
#------------------------------------------------------------------------------
def getScopePath(cellName,nodeName,serverName):
	print "Enter getScopePath()... "
	
	scopePath="";
	if (cellName ==''):
		scopePath='';
	elif (nodeName ==''):
		scopePath='/Cell:' + cellName + '/';
	elif (serverName == ''):		
		scopePath='/Cell:' + cellName + '/Node:' + nodeName + '/';
	else:
		scopePath='/Cell:' + cellName + '/Node:' +nodeName + '/Server:' + serverName + '/';
	#endIf
	
	print "scopePath -- " + scopePath
	print "Exit getScopePath()... "
	return scopePath;
#endDef

#------------------------------------------------------------------------------
# Get application by application prefix in specific scope
#------------------------------------------------------------------------------
def getApplication(appPrefix, scope):
	print "Enter getApplication()... "
	
	result = ""
	applications = AdminApp.list(scope).split()
	for application in applications:
		if (application.startswith(appPrefix)):
			result = application
			return result
	
	print "Exit getApplication()... "
	return result
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed 
#------------------------------------------------------------------------------
def checkApplicationExist(appName, scope):
	methodName = "checkApplicationExist";
	print "Entering " + methodName + "; Application -- " + appName + "; Scope -- " + scope
	result = False
	applications = AdminApp.list(scope).split()
	for application in applications:
		if (application == appName):
			result = True
			print "Exiting " + methodName + " with " + str(result)
			return result
	print "Exiting " + methodName + " with " + str(result)
	return result
#endDef


#Main

print "Enter upgradeBSpaceProfile.py... "

#initialize the variables
wasHome = None
clusterName = None
appSubfixName = None
nodeName = None
serverName = None
cellName = System.getProperty("local.cell")
print "Cell name -- " + cellName
#profileRoot = System.getProperty("user.install.root")
#profileRoot = convertPathToUnixStyle(profileRoot);
#print "Profile root -- " + profileRoot
appPrefix = 'BSpaceEAR';
final_file_list = []
final_dir_list = []

#nodeType will be in the System properties if this script is called via the upgrade framework
nodeType = System.getProperty("bpm.upgrade.node.type")
#print "Node type -- " + nodeType

props = System.getProperties()
print props
print System.getProperty("local.node")

if (nodeType is None):  #script called from command line
  if ((len(sys.argv) == 2)):
    wasHome = sys.argv[0]
    clusterName = sys.argv[1]
    scope = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName
  elif ((len(sys.argv) == 3)):
    wasHome = sys.argv[0]
    nodeName = sys.argv[1]
    serverName= sys.argv[2]
    scope = 'WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName
  else:
    print "Incorrect Parameters"
    sys.exit(-1)
  #endif
  
  print "WAS home -- " + wasHome
  print "Scope -- " + scope
  
  appName = getApplication(appPrefix, scope)
  if (appName == ""):
    print "Business Space is not configured in target, skipping Business space profile upgrade for " + scope
    sys.exit(-1)
  upgradeBSpaceProfile(appName, nodeName, serverName, clusterName)
  
  if not clusterName:
    updateProfileConfiguration()
  
  #copyBusinessSpaceSystemFiles()
  #updateImportingFlags()

else:	#script called from upgrade framework
  wasHome = System.getProperty("was.install.root")
  print "WAS home -- " + wasHome
  
  #copyBusinessSpaceSystemFiles()
  #updateImportingFlags()
  
  clusterName = System.getProperty("bpm.upgrade.cluster.name")
  if not clusterName:
    updateProfileConfiguration()
    
  if nodeType == "dmgr": # updating the deployment manager in an ND environment
    dmgrProfileRoot = System.getProperty("user.install.root")    
    appServers = AdminUtilities.convertToList(AdminTask.listServers(["-serverType", "APPLICATION_SERVER"]))
    for appServer in appServers:
        if not AdminConfig.showAttribute(appServer, "clusterName"): # not a cluster member, i.e. a single managed server
            sname = appServer[0:appServer.find("(")]
            print "Server name="+sname
            nname = appServer[appServer.find("nodes/")+6:appServer.find("servers/")-1]
            print "Node name="+nname
            
            appName = appPrefix + '_' + nname + '_' + sname
            scopeAtt = 'WebSphere:cell=' + cellName + ',node=' + nname + ',server=' + sname
            
            if checkApplicationExist(appName, scopeAtt):
            	upgradeBSpaceProfile(appName, nname, sname, None)
            #endif
        #endif
    #endfor
  #endif
  
  if not (nodeType == "managed"): # skip the server upgrade for managed nodes
    #Use the framework's "updateDeploymentTargetsOf" function to invoke
    #the above callable (function) for each deployment target where an
    #application whose name starts with "BSpaceEAR" is deployed.
    appNames = [appName for appName in wsadminToList(AdminApp.list()) if appName.startswith(appPrefix) ]
    updateDeploymentTargetsOf(appNames, upgradeBSpaceProfile)
  #endif
#endif

print "Exit upgradeBSpaceProfile.py... "

#end Main