/** Licensed Materials - Property of IBM, 5724-U69, (C) Copyright IBM Corp. 2008, 2010 - All Rights reserved.  **/
dojo.provide("com.ibm.mm.enabler.portal.legacy");
if(!dojo._hasResource["com.ibm.mashups.enabler.EmbeddingGenerator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.EmbeddingGenerator"] = true;
dojo.provide( "com.ibm.mashups.enabler.EmbeddingGenerator" );  






/**
 * This class allows to generate single line script include for the embedding feature
 * @ibm-spi
 * @ibm-module Embedding
 */
dojo.declare("com.ibm.mashups.enabler.EmbeddingGenerator", null, {
    widgetDefinition: "",
    pid: "",
    width: "",
    height: "",
    scrollbar: false,
    sandboxed: false,
    framework: "light",
    preferences: null,
    
    /**
     * This is the constructor for EmbeddingGenerator, which allows to instantiate the EmbeddingGenerator with the default settings for widget / page embedding.
     * @param {String} widgetDef the path to the widget definition (either absolute or relative to the embedding server). In case of page embedding widgetDef points to the layout widget to use for page embedding
     * @param {String} pid the page ID of the page to include. If only widget embedding is needed, this parameter can be left empty.
     */
    constructor: function(widgetDef, pid) {
        this.widgetDefinition = widgetDef;
        this.pid = pid;
        this.hostAlias = null;
    },
    /**
     * This method will get a hostname, in sandboxed mode it will be a magic host name. 
     * NOTE: the host name is the same for the lifetime of this object.
     */
    _getSecureHostAlias: function() {
        if (this.hostAlias === null) {
            this.hostAlias = window.location.host;
        }
        
        return this.hostAlias;
    },

    /**
     * Creates the single script include with the current attribute settings of this class.
     */
    toString: function() {
        var url = window.location.protocol;
        url += "//" + this._getSecureHostAlias();
        url += com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.CONTEXT_ROOT);
        
        // add the contenthandler to use
        url += com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.EMBEDDING_PUBLIC);
        
        url += "?";
        
        // specify the uri schema
        url += "uri=widget:js";
        
        // add the url parameter, only applies to widget embedding
        if (null !== this.widgetDefinition) {
            var resolvedUrl = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.widgetDefinition);
            url += "&url=" + escape(resolvedUrl);
        }
        
        if (this.pid) {
            url += "&pid=" + escape(this.pid);
            
            this.height = this.height ? this.height:"850";
            this.width  = this.width  ? this.width:"750";
        } else {
            this.height = this.height ? this.height:"500";
            this.width  = this.width  ? this.width:"400";
        }
            
        url += "&h=" + escape(this.height);
        url += "&w=" + escape(this.width);
        
        if (this.scrollbar) {
            url += "&sb=yes";
        }
        else {
            url += "&sb=no";
        }
        if (this.framework && this.framework != "light") {
            url += "&fw=" + this.framework;
        }

        if (this.sandboxed) {
            url += "&sandbox=yes";
        }

        if (this.preferences && this.preferences.getAllNames()) {
            var names = this.preferences.getAllNames();
            
            for (var i=0, l = names.length; i<l; ++i) {
                url += "&p_" + encodeURIComponent(names[i]) + "=" + encodeURIComponent(this.preferences.getItemValue(names[i]));
            }
        }
        
        return url;
    },

    /**
     * gets the current widget definition path for this embedding generator.
     * @type String
     */
    getWidgetDefinition: function() {
        return this.widgetDefinition;
    },
    /**
     * sets the current widget definition path for this embedding generator
     * @param {String} value the widget definition url
     */
    setWidgetDefinition: function(value) {
        this.widgetDefinition = value;
    },
    
    /**
     * gets the current page id for this embedding generator.
     * @type String
     */
    getPid: function() {
        return this.pid;
    },
    
    /**
     * sets the pid for this embedding generator
     * @param {String} value the pid
     */
    setPid: function(value) {
        this.pid = value;
    },

    /**
     * gets the current width for this embedding generator.
     * @type int
     */
    getWidth: function() {
        return this.width;
    },
    
    /**
     * sets the current width for this embedding generator
     * @param {int} value the width
     */
    setWidth: function(value) {
        this.width = value;
    },
    
    /**
     * gets the current height for this embedding generator.
     * @type int
     */
    getHeight: function() {
        return this.height;
    },
    /**
     * sets the current height for this embedding generator
     * @param {int} value the height
     */
     setHeight: function(value) {
        this.height = value;
    },

    /**
     * gets the current  state of scrollbar for this embedding generator.
     * @type boolean
     */
    getScrollbar: function() {
        return this.scrollbar;
    },
    /**
     * sets whether the scrollbar should be enabled or not
     * @param {boolean} value true when the scrollbar should be enabled, default: false
     */
    setScrollbar: function(value) {
        this.scrollbar = value;
    },

    /**
     * gets whether the widget should be sandboxed or not
     * @type boolean
     */
    getSandboxed: function() {
        return this.sandboxed;
    },
    /**
     * sets whether the widget should be sandboxed
     * @param {boolean} value true when the widget should be sandboxed, default: false
     */
    setSandboxed: function(value) {
        this.sandboxed = value;
    },

    /**
     * gets the current framework typefor this embedding generator.
     * @type String
     */
    getFramework: function() {
        return this.framework;
    },
    
    /**
     * sets the current framework for this embedding generator
     * @param {String} value the framework to use for the widget (not used in page mode), default: to false
     */
    setFramework: function(value) {
        this.framework = value;
    },
    
    /**
     * gets the current framework typefor this embedding generator.
     * @type ItemSet
     */
    getPreferences: function() {
        return this.preferences;
    },
    
    /**
     * sets the preferences for this embedding generator
     * @param {ItemSet} value the preferences
     */
    setPreferences: function(value) {
        this.preferences = value;
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogNode"] = true;
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogNode");





/**
 * Interface that represents a catalog node
 * 
 * @ibm-api
 * @ibm-module CatalogModel
 */
dojo.declare("com.ibm.mashups.enabler.catalog.CatalogNode", [
		com.ibm.mashups.enabler.Identifiable,
		com.ibm.mashups.enabler.ModifiableLocalized, com.ibm.mashups.enabler.DefaultLocalized ], {});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogCategoryNode_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogCategoryNode_API"] = true;
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogCategoryNode_API");
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogCategoryNode");

/**
 * Interface that represents a catalog category; used in the
 * CatalogCategoryModel
 * 
 * @ibm-api
 * @ibm-module CatalogModel
 */
dojo.declare("com.ibm.mashups.enabler.catalog.CatalogCategoryNode",
		[ com.ibm.mashups.enabler.catalog.CatalogNode ], {});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl"] = true;
dojo.provide("com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl");












dojo.declare("com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl", [com.ibm.mashups.enabler.catalog.CatalogCategoryNode, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.DefaultLocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.EditLinkProviderXmlImpl, com.ibm.mm.enabler.TransformableImpl], {
    constructor: function(node) {
        this.xmlData = node;
        this.parentModel = null;
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_CATALOG);
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_XSI, nsf.NS_XML, nsf.NS_OPENSEARCH]);
        for (var prefix in this.serviceJson.namespaces) {
            if (Object.prototype.hasOwnProperty.call(this.serviceJson.namespaces,prefix)) {
                this.ns[prefix] = this.serviceJson.namespaces[prefix];
            }
        }
    },
    
    getParentModel: function() {
        return this.parentModel;
    },
    
    setParentModel: function(parentModel) {
        this.parentModel = parentModel;
    },
    
    toString: function() {
        return com.ibm.mm.enabler.utils.Dom.innerXML(this.xmlData);
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogCategoryNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogCategoryNode"] = true;
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogCategoryNode");





}

if(!dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogEntryNode_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogEntryNode_API"] = true;
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogEntryNode_API");
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogEntryNode");

/**
 * Interface that represents a catalog entry; used in the CatalogEntryModel
 * 
 * @ibm-api
 * @ibm-module CatalogModel
 */
dojo.declare("com.ibm.mashups.enabler.catalog.CatalogEntryNode", [
		com.ibm.mashups.enabler.catalog.CatalogNode,
		com.ibm.mashups.enabler.ModifiableMetaData ], {
	/**
	 * Returns the definition URL of this catalog entry
	 * @param {Boolean} resolveEndpoints optional parameter to resolve endpoint:// urls.  Optional, default is true.
	 * @return {String} the definition url, never <code>null</code>
	 */
	getDefinitionURL : function(resolveEndpoints) {
	},

	/**
	 * Returns the icon URL of this catalog entry
	 * @param {Boolean} resolveEndpoints optional parameter to resolve endpoint:// urls.  Optional, default is true.
	 * @return {String} the icon url, never <code>null</code>
	 */
	getIconURL : function(resolveEndpoints) {
	},

	/**
	 * Returns the preview thumbnail URL for this catalog entry
	 * @param {Boolean} resolveEndpoints optional parameter to resolve endpoint:// urls.  Optional, default is true.
	 * @return {String} the preview thumbnail url, may be <code>null</code>
	 */
	getPreviewThumbnailURL : function(resolveEndpoints) {
	},

	/**
	 * Returns the preview URL for this catalog entry
	 * @param {Boolean} resolveEndpoints optional parameter to resolve endpoint:// urls.  Optional, default is true.
	 * @return {String} the preview url, may be <code>null</code>
	 */
	getPreviewURL : function(resolveEndpoints) {
	},

	/**
	 * Returns the help URL for this catalog entry
	 * @param {Boolean} resolveEndpoints optional parameter to resolve endpoint:// urls.  Optional, default is true.
	 * @return {String} the help url, may be <code>null</code>
	 */
	getHelpURL : function(resolveEndpoints) {
	},

	/**
	 * Sets the definition URL of this catalog entry
	 * @param {String} url definition url to set; must not be <code>null</code>
	 * @type void
	 */
	setDefinitionURL : function(url) {
	},

	/**
	 * Confirms whether setting the url is possible.
	 * @param {String} url definition url to set; must not be <code>null</code>
	 * @return {Boolean} true if the value can be set, otherwise false.
	 */
	confirmSetDefinitionURL : function(url) {
	},

	/**
	 * Sets the icon URL of this catalog entry
	 * @param {String} url icon url to set; must not be <code>null</code>
	 * @type void
	 */
	setIconURL : function(url) {
	},

	/**
	 * Confirms whether setting the url is possible.
	 * @param {String} url icon url to set; must not be <code>null</code>
	 * @return {Boolean} true if the value can be set, otherwise false.
	 */
	confirmSetIconURL : function(url) {
	},

	/**
	 * Sets the preview thumbnail URL of this catalog entry
	 * @param {String} url preview thumbnail url to set; may be <code>null</code>
	 * @type void
	 */
	setPreviewThumbnailURL : function(url) {
	},

	/**
	 * Confirms whether setting the url is possible.
	 * @param {String} url preview thumbnail url to set; may be <code>null</code>
	 * @return {Boolean} true if the value can be set, otherwise false.
	 */
	confirmSetPreviewThumbnailURL : function(url) {
	},

	/**
	 * Sets the preview URL of this catalog entry
	 * @param {String} url preview url to set; may be <code>null</code>
	 * @type void
	 */
	setPreviewURL : function(url) {
	},

	/**
	 * Confirms whether setting the url is possible.
	 * @param {String} url preview url to set; may be <code>null</code>
	 * @return {Boolean} true if the value can be set, otherwise false.
	 */
	confirmSetPreviewURL : function(url) {
	},

	/**
	 * Sets the help URL of this catalog entry
	 * @param {String} url help url to set; may be <code>null</code>
	 * @type void
	 */
	setHelpURL : function(url) {
	},

	/**
	 * Confirms whether setting the url is possible.
	 * @param {String} url help url to set; may be <code>null</code>
	 * @return {Boolean} true if the value can be set, otherwise false.
	 */
	confirmSetHelpURL : function(url) {
	},

	/**
	 * Returns the short description of this object in the given locale.
	 * @param {String} locale   the locale for which to retrieve the
	 * short description, must not be <code>null</code>.
	 * @return {String} the short description of this node in the given locale. If a
	 * short description is not available in the given locale, this method
	 * will return <code>null</code>. It is up to the invoker of the
	 * method to implement an appropriate fallback mechanism.
	 */
	getShortDescription : function(locale) {
	},

	/**
	 * Sets the short description for the given locale.
	 * @param {String} desc     short description to set; must not be <code>null</code>
	 * @param {String} locale   locale to set the short description for; must not be <code>null</code>
	 * @return {String} the former short description; if none existed, <code>null</code> is returned
	 */
	setShortDescription : function(desc, locale) {
	}
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl"] = true;
dojo.provide("com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl");
















dojo.declare("com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl", [com.ibm.mashups.enabler.catalog.CatalogEntryNode, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.DefaultLocalizedXmlImpl, com.ibm.mm.enabler.MetaDataImpl2, com.ibm.mm.enabler.persistence.xml.EditLinkProviderXmlImpl, com.ibm.mm.enabler.ac.EffectiveAccessImpl, com.ibm.mm.enabler.TransformableImpl], {
    constructor: function(node) {
        this.xmlData = node;
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_CATALOG);
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ceni_ns = dojo.delegate(this.serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_XSI, nsf.NS_XML, nsf.NS_OPENSEARCH]));
        
        this.NLS_SHORT_DESCRIPTION = "atom:content/*/model:shortDescription/base:nls-string";
        this.NLS_SHORT_DESCRIPTION_TOP = "atom:content/*/model:shortDescription";
        this.BASE_SHORT_DESCRIPTION = "base:nls-string";
        
    },
    
    getDefinitionURL: function(resolveEndpoints) {
        return this._getURL("definition", resolveEndpoints);
    },
    setDefinitionURL: function(aUrl) {
        this._setURL("definition", aUrl);
    },
    confirmSetDefinitionURL: function(aUrl) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    getIconURL: function(resolveEndpoints) {
        var iconUrl = this._getURL("icon", resolveEndpoints);
        return iconUrl;
    },
    setIconURL: function(aUrl) {
        this._setURL("icon", aUrl);
    },
    confirmSetIconURL: function(aUrl) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    getPreviewThumbnailURL: function(resolveEndpoints) {
        return this._getURL("previewThumbnail", resolveEndpoints);
    },
    setPreviewThumbnailURL: function(aUrl) {
        this._setURL("previewThumbnail", aUrl);
    },
    confirmSetPreviewThumbnailURL: function(aUrl) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    getPreviewURL: function(resolveEndpoints) {
        var pUrl = this._getURL("preview", resolveEndpoints);
        return pUrl;
    },
    
    setPreviewURL: function(aUrl) {
        this._setURL("preview", aUrl);
    },
    confirmSetPreviewURL: function(aUrl) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    getHelpURL: function(resolveEndpoints) {
        var hUrl = this._getURL("help", resolveEndpoints);
        return hUrl;
    },
    
    setHelpURL: function(aUrl) {
        this._setURL("help", aUrl);
    },
    confirmSetHelpURL: function(aUrl) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    getShortDescription: function(locale) {
        var result = null;
        var nloc = locale.replace(/_/g, "-").toLowerCase();
        var descriptions = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(this.NLS_SHORT_DESCRIPTION, this.xmlData, this.li_ns);
        if (descriptions) {
            for (var i = 0; i < descriptions.length; i++) {
                var lang = com.ibm.mm.enabler.utils.Dom.getAttributeWithNS(descriptions[i], "xml:lang", "lang", com.ibm.mm.enabler.model.NameSpaceFactory.getNameSpaceUri("xml"));
                if (lang !== null && lang.replace(/_/g, "-").toLowerCase() == nloc) {
                    result = com.ibm.mm.enabler.utils.Dom.textContent(descriptions[i]);
                    break;
                }
            }
        }
        return result;
    },
    setShortDescription: function(descr, locale) {
        locale.replace(/-/g, "_");
        var descriptions = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(this.NLS_SHORT_DESCRIPTION, this.xmlData, this.li_ns);
        // iterate the localized titles (selecting the correct title via xpath doesnt seem to work)
        var localeExists = false;
        var i = descriptions.length;
        while (--i >= 0) {
            var lang = com.ibm.mm.enabler.utils.Dom.getAttributeWithNS(descriptions[i], "xml:lang", "lang", com.ibm.mm.enabler.model.NameSpaceFactory.getNameSpaceUri("xml"));
            if (lang !== null && lang.replace(/-/g, "_") == locale) {
                com.ibm.mm.enabler.utils.Dom.textContent(descriptions[i], descr);
                localeExists = true;
                break;
            }
        }
        
        // create locale
        if (!localeExists) {
            var descriptionBase = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(this.NLS_SHORT_DESCRIPTION_TOP, this.xmlData, this.li_ns);
            if (descriptionBase) {
                var xmlDom = this.xmlData.ownerDocument;
                var node = com.ibm.mm.enabler.utils.Dom.createElement(xmlDom, this.BASE_SHORT_DESCRIPTION, this.BASE_NS);
                node.setAttribute("xml:lang", locale);
                com.ibm.mm.enabler.utils.Dom.textContent(node, descr);
                descriptionBase.appendChild(node);
            }
            else {
                throw new Error(dojo.string.substitute(this.modelMessages.E_ELEMENT_NOT_FOUND_2, [this.NLS_SHORT_DESCRIPTION_TOP, this.toString()]));
            }
        }
        this._setDirty();
    },
    confirmSetShortDescription: function(descr, locale) {
        return this.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR);
    },
    
    _getURL: function(/*String*/rel, /*boolean*/ resolveEndpoints) {
        var result = null;
        var expr = "atom:content/model:catalog-entry/atom:link[@ext:rel='" + rel + "']";
        var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ceni_ns);
        if (nodes) {
            result = nodes.getAttribute("href");
            var newURL = new com.ibm.mm.enabler.utils.HttpUrl(result, resolveEndpoints);
            result = (newURL._absoluteURL ? newURL.toString(true) : result) || result;
        }
        var ret = result;
        if (resolveEndpoints !== false) {
            var ret2 = com.ibm.mm.enabler.EndpointUtils.checkForEndpoints(result);
            ret2 = ret2 || null;
            if (ret2 !== null) {
                ret = ret2;
            }
        }
        return ret;
    },
    
    _setURL: function(/*String*/rel, /*String*/ href) {
        var expr = "atom:content/model:catalog-entry/atom:link[@ext:rel='" + rel + "']";
        var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ceni_ns);
        if (nodes) {
            var newURL = new com.ibm.mm.enabler.utils.HttpUrl(href, false);
            var url = (newURL._absoluteURL ? newURL.toString() : newURL.toServerRelativeString(true)) || href;
            nodes.setAttribute("href", url);
            this._setDirty();
        }
    },
    
    toString: function() {
        return com.ibm.mm.enabler.utils.Dom.innerXML(this.xmlData);
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogEntryNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.catalog.CatalogEntryNode"] = true;
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogEntryNode");







}

if(!dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplate_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplate_API"] = true;
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplate_API");
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplate");






/**
 * Interface representing a layout template node.
 *
 * @ibm-api
 * @ibm-module LayoutTemplateModel
 */
dojo.declare("com.ibm.mashups.enabler.layouttemplate.LayoutTemplate", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.ModifiableLocalized, com.ibm.mashups.enabler.ModifiableMetaData, com.ibm.mashups.enabler.TimeStamped], {

    /**
     * Returns the URL of the layout template.
     * @return {String} the base URL
     */
    getURL: function(){
    
    },
    
    /**
     * Returns the URL of the head file of this layout template.
     * @return {String} the layout template URL
     */
    getHeadURL: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.layouttemplate.LayoutTemplate"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.layouttemplate.LayoutTemplate"] = true;
dojo.provide("com.ibm.mm.enabler.layouttemplate.LayoutTemplate");



// summary: The PropertiesFileFacade is in charge of mediating the input into and out of
//		the standard Java-style properties file format.
com.ibm.mm.enabler.layouttemplate.PropertiesFileFacade = {
    serialize: function(objMap) {
        // summary: Takes a map of keys and values and serializes it to a Java-style properties file format.
        // description: The output includes a simple header with a comment containing the date of serialization, 
        //		followed by each key/value pair on a new line.
        // objMap: an associative array containing key-value pairs
        // returns: the string in standard Java-style properties file format (including new line characters)
        var ret = ["#\n"];
        ret.push("# ", (new Date()).toUTCString(), "\n");
        for (var obj in objMap) {
            if (Object.prototype.hasOwnProperty.call(objMap, obj)) {
                ret.push(obj, "=", objMap[obj], "\n");
            }
        }
        
        return ret.join("");
    },
    deserialize: function(propertiesString) {
        // summary: Parses a Java-style properties file specified by contents or URL.
        // propertiesString: the string value associated with that property will be treated as the properties file 
        // 		contents.
        // returns: a dojo.Deferred object where callbacks will be triggered with an associative array representing the
        //		properties file contents
        var dfd = new dojo.Deferred();
        this._parse(propertiesString, dfd);
        return dfd;
    },
    _parse: function(contents, deferred) {
        // summary: Executes the parsing.
        //TODO: replace with regexps
        var lines = contents.split("\n");
        var l = lines.length;
        var objMap = {};
        for (var i = 0; i < l; i++) {
            var line = lines[i];
            if (line.indexOf("#") !== 0) {
                var key;
                var value;
                var equals = line.indexOf("=");
                if (equals > 0) {
                    key = line.substring(0, equals);
                    value = this._filter(line.substring(equals + 1));
                    if (key !== "") {
                        objMap[key] = value;
                    }
                }
            }
        }
        
        deferred.callback(objMap);
    },
    _filter: function(str) {
        // summary: Filters a property value for usage in javascript code. 
        // str: the string to filter
        // returns: the filtered string, ready for display and usage in javascript code
        
        //currently, the only activity is processing unicode characters. this method exists in case any further
        //processing needs to be performed on these strings. 
        return this._processUnicodeCharacters(str);
    },
    _processUnicodeCharacters: function(str) {
        // summary: Processes a string containing expanded unicode references ("\u0639"), replacing 
        //		the expanded unicode sequence with the unicode character based on the same sequence.
        // description: An alternative method would be to simply eval() the string surrounded by quotes. We are using
        //		this method to avoid the security risks imposed by that approach. In this solution, we are only calling
        //		eval() on character sequences that match our unicode regular expression and only one "character" at a 
        //		time. Otherwise, anyone with edit access to the localized files could enter script code in the
        //		properties file and have it executed when the properties file was read. 
        // str: the string to process
        // returns: a string with all expanded unicode references reduced to the expected unicode character
        var unicodeRegexp = /\\u[0-9a-fA-F]{4}/g;
        var matches = str.match(unicodeRegexp);
        
        if (matches) {
            for (var i = 0, l = matches.length; i < l; i++) {
                var unicodeStr = matches[i];
                var unicodeChar = eval("'" + unicodeStr + "'"); // JSLINT-IGNORE: no other way known so far to do this
                str = str.replace(unicodeStr, unicodeChar);
            }
        }
        
        return str;
    }
};

// summary: Resolves a deferred.
// description: Since the contract for resolving an enabler Deferred object is not defined, this object handles this
//		for com.ibm.mm.enabler.DeferredImpl objects. It essentially triggers the callback function associated with the
//		specified Deferred with the resource and status code.
dojo.declare("com.ibm.mm.enabler.layouttemplate.DeferredResolver", null, {
    setDelegate: function( /*Deferred*/d) {
        this.deferred = d;
    },
    resolve: function( /*Object*/resource, /*int*/ status) {
        if (this.deferred) {
            var cb = this.deferred.getFinishedCallback();
            var cbParams = this.deferred.getFinishedCallbackParameters();
            if (cb) {
                cb(resource, status, cbParams);
            }
        }
    }
});

// summary: Manages multiple enabler Deferred objects.
// description: There is no contract or specification for chaining multiple enabler Deferred objects together. This 
//		object hides the complexities and ugliness of executing this ability. It only currently works for the default 
//		implementation of Deferred (com.ibm.mm.enabler.DeferredImpl) and the WebDAVDeferredImpl. The limitation is one
//		of not knowing how to access the Deferred's callback function as this is not defined in the Deferred's contract.
com.ibm.mm.enabler.layouttemplate.DeferredManager = {
    counter: 0,
    extractCallbackInfo: function(enablerDfd) {
        // summary: Extracts the specified callback information for a given deferred.
        // enablerDfd: an enabler style deferred object
        // returns: an object of the form { callback: function, params: [] }; the values MAY be undefined
        var callback;
        var callbackParams;
        if (enablerDfd.getFinishedCallback) {
            callback = enablerDfd.getFinishedCallback();
        }
        else if (enablerDfd._callback) {
            callback = enablerDfd._callback;
        }
        
        if (enablerDfd.getFinishedCallbackParameters) {
            callbackParams = enablerDfd.getFinishedCallbackParameters();
        }
        else if (enablerDfd._parameters) {
            callbackParams = enablerDfd._parameters;
        }
        
        return {
            callback: callback,
            params: callbackParams
        };
    },
    combine: function(d1, d2) {
        // summary: Combines two Deferred objects into a new Deferred. Callbacks added to the result of this operation 
        //		are not called until both of the combined Deferred objects have completed. Callbacks already established
        //		on the individual deferreds are still called when each individual deferred completes. Deferreds can be 
        //		combined multiple times. For example, d1, d2, and d3 need to execute in this order: d1, d2, d3. d1 and 
        //		d2 would be combined. Then, the result of the combination of d1 and d2 would be combined with d3.
        // d1: the deferred to execute first.
        // d2: the deferred to execute second.
        // returns: a com.ibm.mm.enabler.DeferredImpl object that represents the combination of d1 and d2.
        var cb1 = this.extractCallbackInfo(d1);
        
        //This info is useful for debugging the combination of Deferreds in Firebug/Web Inspector, but serves no 
        //functional purpose.
        if (ibmConfig.isDebug) {
            if (d1._uid === undefined) {
                d1._uid = this.counter++;
            }
            if (d2._uid === undefined) {
                d2._uid = this.counter++;
            }
        }
        
        //This is the new "wrapped" deferred that will resolve after both of the combined deferreds resolve.
        var d = new com.ibm.mm.enabler.DeferredImpl(d, function() {
            d1.start();
        });
        var me = this;
        d1.setFinishedCallback(function() {
            //Once the first deferred has completed, we need to:
            //		1. Trigger it's finished callback.
            //		2. Setup the second deferred's callback.
            //		3. Start the second deferred.
            if (cb1.callback) {
                cb1.callback(arguments[0], arguments[1], cb1.params);
            }
            
            var cb2 = me.extractCallbackInfo(d2);
            d2.setFinishedCallback(function() {
                //We need to take this over to make sure any callback added to the combined deferred gets called at the
                //appropriate time. 
                if (cb2.callback) {
                    cb2.callback(arguments[0], arguments[1], cb2.params);
                }
                
                var cb3 = me.extractCallbackInfo(d);
                
                if (cb3.callback) {
                    cb3.callback(arguments[0], arguments[1], cb3.params);
                }
            });
            d2.start();
        });
        
        //This info is useful for debugging the combination of Deferreds in Firebug/Web Inspector, but serves no 
        //functional purpose.
        if (ibmConfig.isDebug) {
            d._first = d1;
            d._second = d2;
            d._uid = this.counter++;
        }
        
        return d;
    }
};

// summary: Abstraction layer to insulate the model code from changes in the Enabler WebDAV APIs. Can easily be 
//		customized by mixing in any overridden functions.
com.ibm.mm.enabler.layouttemplate.WebDAVInterface = {
    init: function( /*RemoteModel*/rModel) {
        // summary: Initialize the interface with the Remote (WebDAV) model.
        this.remoteModel = rModel;
        this._commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
    },
    getFileInfo: function( /*String*/path) {
        // summary: Retrieves the requested file information (PROPFIND). Note that the file could be a directory.
        // path: the path of the requested file
        // returns: a Deferred object that will eventually resolve to a RemoteFile object
        return this.remoteModel.find(path);
    },
    getFilename: function( /*RemoteFile*/file) {
        // summary: Returns the name (including any extension) of the file represented by the file
        //		object
        // file: the RemoteFile object
        // returns: a Deferred object that will eventually resolve to the file name
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            r.resolve(file.getName(), 200);
        });
        r.setDelegate(d);
        return d;
    },
    isDirectory: function( /*RemoteFile*/file) {
        // summary: Returns the name (including any extension) of the file represented by the file
        //		object
        // file: the RemoteFile object
        // returns: a Deferred object that will eventually resolve to a boolean value
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            r.resolve(file.isDirectory(), 200);
        });
        r.setDelegate(d);
        return d;
    },
    getDirectoryContentsURLs: function( /*RemoteFile*/file) {
        // summary: Returns an array of URLs that represent files or directories contained within
        //		the RemoteFile object provided
        // file: the RemoteFile object - must point to a directory
        // returns: a Deferred object that will eventually resolve to an array of URLs in string form that list
        //		the files and directories contained in the provided directory
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            r.resolve(file._getChildHrefs(), 200);
        });
        r.setDelegate(d);
        return d;
    },
    getURL: function( /*RemoteFile*/file) {
        // summary: Returns the full URL to access the provided file
        // file: the RemoteFile object
        // returns: a Deferred object that will eventually resolve to a String value
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            r.resolve(file.getURL(), 200);
        });
        r.setDelegate(d);
        return d;
    },
    getFileContents: function( /*String*/path) {
        // summary: Retrieves the file contents at the given path.
        // path: the path (relative to WebDAV root) to retrieve the file from
        // contents: the contents of the new file
        // returns a Deferred object that will eventually resolve to the contents of the file.
        var cmd = this._commandFactory.getDownloadCommand(path);
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            var dfd = cmd.execute();
            dfd.addCallback(function(response, ioArgs) {
                r.resolve(response, ioArgs);
            });
        });
        r.setDelegate(d);
        
        return d;
    },
    createFile: function( /*String*/path, /*String*/ contents) {
        // summary: Creates a file at the given path with the given contents.
        // path: the path (relative to WebDAV root) where the file should be created (including the filename)
        // contents: the contents of the new file
        // returns a Deferred object that will eventually resolve to a RemoteFile object representing the new
        //		file.
        var cmd = this._commandFactory.getCreateCommand(path, null, contents);
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            var dfd = cmd.execute();
            dfd.addCallback(function(response, ioArgs) {
                r.resolve(response);
            });
        });
        r.setDelegate(d);
        
        return d;
    },
    updateFile: function( /*String*/path, /*String*/ contents) {
        // summary: Updates an existing file at the given path with the given contents.
        // path: the path (relative to WebDAV root) where the file should be created (including the filename)
        // contents: the contents of the updated file - note that these contents REPLACE the existing contents
        // returns: a Deferred object that will eventually resolve to a RemoteFile object representing the new
        //		file.
        var cmd = this._commandFactory.getUpdateCommand(path, null, contents);
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            var dfd = cmd.execute();
            dfd.addCallback(function(response, ioArgs) {
                r.resolve(response);
            });
        });
        r.setDelegate(d);
        
        return d;
    },
    createDirectory: function( /*String*/path) {
        // summary: Creates a directory for the given path. The path should include the new directory name.
        // path: the path to the new directory relative to the WebDAV root
        // returns: a Deferred object that will resolve when the create operation has completed
        var cmd = this._commandFactory.getMakeCollectionCommand(path);
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            var dfd = cmd.execute();
            dfd.addCallback(function(response, ioArgs) {
                r.resolve(response);
            });
        });
        r.setDelegate(d);
        
        return d;
    },
    remove: function( /*String*/path) {
        // summary: Removes the resource identified by the provided path.
        // path: the path to the resource to remove - relative to the WebDAV root
        // returns: a Deferred object that will resolve when the delete operation has completed
        var cmd = this._commandFactory.getDeleteCommand(path);
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
            var dfd = cmd.execute();
            dfd.addCallback(function(response, ioArgs) {
                r.resolve(response);
            });
        });
        r.setDelegate(d);
        
        return d;
    }
};

// summary: Generates the URLs to the various files that provide the backend for the localized and metadata interfaces.
com.ibm.mm.enabler.layouttemplate.TemplateURLHelper = {
    //baseLayoutTemplatesDirectory ends with a slash
    baseDirectory: ibmConfig && ibmConfig.baseLayoutTemplatesDirectory ? ibmConfig.baseLayoutTemplatesDirectory : "layout-templates/",
    getLocalizedFileURL: function(layoutDirectoryName, locale) {
        // summary: Returns the URL (relative to the WebDAV root) to the properties file containing the
        //		localized data for the given layout template and locale.
        return this.baseDirectory + layoutDirectoryName + "/metadata/localized_" + locale + ".properties";
    },
    getMetaDataFileURL: function(layoutDirectoryName) {
        // summary: Returns the URL (relative to the WebDAV root) to the properties file containing the
        //		metadata for the given layout template.
        return this.baseDirectory + layoutDirectoryName + "/metadata.properties";
    }
};

// summary: Interface for a resource that is backed by an object in WebDAV instead of a feed or database. Manages
//		references to various interfaces required to read and modify the backend data.
// remoteModel: the model for accessing files in WebDAV
// remoteFile: the remote file object (usually a directory) containing the backend data
// webDavInterface: interface for interacting with WebDAV
// urlHelper: helper object for creating URLs to the various backend files
// propertiesFileFacade: interface for reading and writing properties files
// exists: indicates whether or not the object already exists or potentially will be created
dojo.declare("com.ibm.mm.enabler.layouttemplate.WebDAVBackedObject", null, {
    constructor: function(rModel, rFile, wdInt, uHelper, pFacade, doesExist) {
        this.remoteModel = rModel;
        this.remoteFile = rFile;
        this.webDavInterface = wdInt;
        this.urlHelper = uHelper;
        this.propertiesFileFacade = pFacade;
        this.exists = doesExist;
        
        this.webDavInterface.init(this.remoteModel);
    },
    setRemoteNls: function(rNlsFile) {
        this.remoteNlsFile = rNlsFile;
    }
});


// summary: Describes a modifiable localized resource that is backed by a file in WebDAV instead of feed data or 
//		a database. 
dojo.declare("com.ibm.mm.enabler.layouttemplate.WebDAVBackedModifiableLocalized", [com.ibm.mm.enabler.layouttemplate.WebDAVBackedObject, com.ibm.mashups.enabler.ModifiableLocalized], {
    constructor: function() {
        //Separate bundle information set on the client versus loaded remotely. This becomes important when
        //submitting changes as we have to know whether or not the remote data has been loaded before
        //saving. If, for example, only a title is set for a particular locale, but a description was set
        //previously, we need to have the pre-existing description to avoid omitting the description in 
        //our update.
        this._localBundles = {};
        this._remoteBundles = {};
        //manages the state of the object so we know when a change has been made
        this._housekeeper = {
            isClean: true,
            dirtyLocales: {},
            makeLocaleDirty: function(locale) {
                this.dirtyLocales[locale] = true;
                this.isClean = false;
            },
            reset: function() {
                this.dirtyLocales = {};
                this.isClean = true;
            }
        };
    },
    _getBundle: function(locale) {
        // summary: Retrieve the bundle for the specified locale.
        // locale: the locale to retrieve the bundle for
        // returns: an enabler Deferred that will be triggered once the bundle
        //		has been loaded into the remote bundles object.
        // IMPORTANT: There's currently a bug in Enabler that keeps any callback from
        //		being triggered when an unsuccessful request is sent.
        var wdi = this.webDavInterface;
        var tuh = this.urlHelper;
        var p = this.propertiesFileFacade;
        var me = this;
        var d;
        if (this.exists) {
            d = wdi.getFileContents(tuh.getLocalizedFileURL(this.remoteFile.getName(), locale));
            d.setFinishedCallback(function(contents) {
                var dfd = p.deserialize(contents);
                dfd.addCallback(function(props) {
                    me._remoteBundles[locale] = props;
                });
            });
        }
        else {
            var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
            d = new com.ibm.mm.enabler.DeferredImpl(window, function() {
                r.resolve();
            });
            r.setDelegate(d);
        }
        return d;
    },
    _mergeRemoteAndLocalBundle: function(locale) {
        // summary: Merges the remote and local bundles for a locale to provide a single bundle containing
        //		the most up to date localized information for the given object. Local bundle takes 
        //		preference over remote.
        var bundle = this._remoteBundles[locale];
        if (bundle === undefined) {
            bundle = {};
        }
        var localBundle = this._localBundles[locale];
        for (var obj in localBundle) {
            if (Object.prototype.hasOwnProperty.call(localBundle, obj)) {
                bundle[obj] = localBundle[obj];
            }
        }
        return bundle;
    },
    getTitle: function(locale) {
        // summary: Retrieves the title for the given locale.
        // locale: the locale to retrieve the title for
        // returns: an enabler Deferred object that will be triggered with the title string
        // IMPORTANT: The official interface for Localized specifies a String returned here but we don't
        //		follow that for two reasons. One is that the RemoteModel currently has NO support for 
        //		synchronous operation (we have to request the contents of the localized file to retrieve 
        //		this information). Two is that this really should be a Deferred object returned to account
        //		for cases like this; where retrieving the title requires a remote operation. The current
        //		interface is too limiting.
        return this._deferredInvocation(locale, "title");
    },
    getDescription: function(locale) {
        // summary: Retrieves the description for the given locale.
        // locale: the locale to retrieve the description for
        // returns: an enabler Deferred object that will be triggered with the description string
        // IMPORTANT: The official interface for Localized specifies a String returned here but we don't
        //		follow that for two reasons. One is that the RemoteModel currently has NO support for 
        //		synchronous operation (we have to request the contents of the localized file to retrieve 
        //		this information). Two is that this really should be a Deferred object returned to account
        //		for cases like this; where retrieving the description requires a remote operation. The current
        //		interface is too limiting.
        return this._deferredInvocation(locale, "description");
    },
    _deferredInvocation: function(locale, property) {
        // summary: Creates a Deferred object that will access the specified property in the specified
        //		locale's bundle.
        // locale: the requested locale
        // property: the property inside the bundle to retrieve
        // returns: a Deferred that will be triggered with the given property in the specified bundle

        // we need to fix the passed in locale in that we make sure it is of format lowercase_UPPERCASE
        locale = com.ibm.mm.enabler.utils.LocaleHelper._getLocaleObj(locale).toString();

        var locales = this._getExistingLocalesAsMap();
        var localeDataExists = locales[locale];
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            if (localeDataExists) {
                r.resolve(this._mergeRemoteAndLocalBundle(locale)[property]);
            }
            else {
                r.resolve(undefined, 404);
            }
        });
        if (this._remoteBundles[locale] === undefined && this.exists && localeDataExists) {
            var d = this._getBundle(locale);
            var d2 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d, newD);
            r.setDelegate(d2);
            return d2;
        }
        else {
            r.setDelegate(newD);
            return newD;
        }
    },
    getLocales: function() {
        // summary: Returns an array of the available locales.
        if (this._locales === undefined) {
            this._locales = [];
            if (this.exists) {
                this._locales = this._readLocales();
            }
        }
        
        return this._locales;
    },
    _getExistingLocalesAsMap: function() {
        // summary: Converts the array of existing locales into an associative array where the locale is
        //		the key in the object and the value is true. An undefined locale equates to a non-existant
        //		bundle.
        var obj = {};
        
        if (this.exists) {
            var ls = this._readLocales();
            var len = ls.length;
            for (var i = 0; i < len; i++) {
                obj[ls[i]] = true;
            }
        }
        
        return obj;
    },
    _readLocales: function() {
        // summary: Reads the locales out of the RemoteFile object using the child hrefs stored in that
        //		object. This technically is not part of the public API, but seems worth it to save a 
        //		request to the server just to retrieve this information. 
        if (!this.remoteNlsFile) {
            return ["en"];
        }
        var childHrefs = this.remoteNlsFile._getChildHrefs();
        var l = childHrefs.length;
        var locales = [];
        for (var i = 1; i < l; i++) {
            var child = childHrefs[i];
            var name = child.substring(child.lastIndexOf("/") + 1);
            if (name.indexOf("localized_") === 0) {
                //slice off the extension
                name = name.split(".")[0];
                locales.push(name.substring(10));  	// name is "localized_xx" or "localized_xx_XX", 
													//so substring(10) chops off the "localized_" part, 
													//leaving the locale string behind
            }
        }
        return locales;
    },
    setTitle: function(locale, newTitle) {
        // summary: Sets the title for the given locale. Note that no changes are saved until commit is
        //		called on the model.
        // locale: the locale
        // newTitle: the new title
        var bundle = this._localBundles[locale];
        if (bundle === undefined || bundle === null) {
            //bundle = {};
            this._localBundles[locale] = {};
            this.getLocales().push(locale);
        }
        
        this._localBundles[locale].title = newTitle;
        this._housekeeper.makeLocaleDirty(locale);
    },
    confirmSetTitle: function(locale, newTitle) {
        // summary: Confirms whether or not the given title for the given locale CAN be set.
        //TODO: what permissions to check here?
        return true;
    },
    removeTitle: function(locale) {
        // summary: Removes the title for the given locale. Note that no changes are saved until the model
        //		is committed.
        // locale: the locale to remote the title for
        var bundle = this._mergeRemoteAndLocalBundle(locale);
        if (!(bundle === undefined || bundle === null)) {
            delete bundle.title;
            this._housekeeper.makeLocaleDirty(locale);
        }
    },
    confirmRemoveTitle: function(locale) {
        // summary: Confirms whether or not the given title for the given locale CAN be removed.
        //TODO: what permissions to check here?
        var bundle = this._mergeRemoteAndLocalBundle(locale);
        return !(bundle === undefined || bundle === null);
    },
    removeTitles: function() {
        // summary: Removes all titles for this object. Note that no changes are saved until the model is
        //		committed.
        for (var locale in this._locales) {
            if (Object.prototype.hasOwnProperty.call(this._locales, locale)) {
                delete this._bundles[locale].title;
                this._housekeeper.makeLocaleDirty(locale);
            }
        }
    },
    setDescription: function(locale, newDescription) {
        // summary: Sets the description for the given locale. Note that no changes are saved until commit 
        //		is called on the model.
        // locale: the locale
        // newTitle: the new description
        var bundle = this._mergeRemoteAndLocalBundle(locale);
        if (bundle === undefined || bundle === null) {
            bundle = {};
            this._localBundles[locale] = bundle;
            this.getLocales().push(locale);
        }
        
        bundle.description = newDescription;
        this._housekeeper.makeLocaleDirty(locale);
    },
    confirmSetDescription: function(locale, newDescription) {
        // summary: Confirms whether or not the given description for the given locale CAN be set.
        //TODO: what permissions to check here?
        return true;
    },
    removeDescription: function(locale) {
        // summary: Removes the description for the given locale. Note that no changes are saved until the model
        //		is committed.
        // locale: the locale to remote the description for
        var bundle = this._bundles[locale];
        if (!(bundle === undefined || bundle === null)) {
            delete bundle.description;
            this._housekeeper.makeLocaleDirty(locale);
        }
    },
    confirmRemoveDescription: function(locale) {
        // summary: Confirms whether or not the given description for the given locale CAN be removed.
        //TODO: what permissions to check here?
        var bundle = this._remoteBundles[locale];
        return !(bundle === undefined || bundle === null);
    },
    removeDescriptions: function() {
        // summary: Removes all descriptions for this object. Note that no changes are saved until the model is
        //		committed.
        for (var locale in this._bundles) {
            if (Object.prototype.hasOwnProperty.call(this._bundles, locale)) {
                delete this._bundles[locale].description;
                this._housekeeper.makeLocaleDirty(locale);
            }
        }
    },
    _setLocalizedData: function() {
        // summary: Persists the localization data.
        // returns: a Deferred that will be triggered once all the save operations complete.
        // IMPORTANT: Modifying a large number of locales at once can generate a LARGE number of requests.
        //		For each bundle modified, we have to request the current contents of the bundle (if not 
        //		already requested), modify them, and then save the contents back to the server. Essentially,
        //		for each bundle modified, 2 requests will be generated.
        var wdi = this.webDavInterface;
        var tuh = this.urlHelper;
        var p = this.propertiesFileFacade;
        
        var locales = this._housekeeper.dirtyLocales;
        //All callbacks get rolled up into the retVal
        var retVal;
        //d stores the callback for the current iteration of the loop
        var d;
        
        for (var l in locales) {
            if (Object.prototype.hasOwnProperty.call(locales, l)) {
                var path = tuh.getLocalizedFileURL(this.remoteFile.getName(), l);
                //If this evaluates to true, the file already exists remotely.
                if (this._getExistingLocalesAsMap()[l] === true) {
                    //If the remote data has not been loaded, we need to load it FIRST.
                    if (this._remoteBundles[l] === undefined) {
                        d = this._getBundle(l);
                    }
                    if (d) {
                        //This needs to be in a separate function to give the locale a new closure. Otherwise,
                        //l will always refer to the last locale once the loop has completed.
                        d = this._newUpdateLocaleFn(d, l, wdi, path, p);
                    }
                    else {
                        //We already had the remote data.
                        d = wdi.updateFile(path, p.serialize(this._mergeRemoteAndLocalBundle(l)));
                    }
                }
                else {
                    //The file did not exist and we have to create it. We obviously can avoid retrieving
                    //the remote data in this case.
                    d = wdi.createFile(path, p.serialize(this._mergeRemoteAndLocalBundle(l)));
                }
                if (retVal === undefined) {
                    //First time through; no need to build on existing callbacks.
                    retVal = d;
                }
                else {
                    //Combine the existing callback with the latest one. 
                    retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(retVal, d);
                }
            }
        }
        
        var resetResolver = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var resetHousekeeper = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            this._housekeeper.reset();
            resetResolver.resolve();
        });
        resetResolver.setDelegate(resetHousekeeper);
        
        retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(retVal, resetHousekeeper);
        
        return retVal;
    },
    _newUpdateLocaleFn: function(getBundleDfd, locale, wdi, path, p) {
        // summary: Creates a new Deferred that contains the retrieval of the existing bundle, followed by 
        //		the request to update the file contents with the modified properties.
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            //The remote data has been retrieved, now merge it with our changes and update the file.
            var d2 = wdi.updateFile(path, p.serialize(this._mergeRemoteAndLocalBundle(locale)));
            var me = this;
            d2.setFinishedCallback(function() {
                //The remote data IS the local data now.
                me._remoteBundles[locale] = me._mergeRemoteAndLocalBundle(locale);
                me._localBundles[locale] = {};
                r.resolve();
            });
            d2.start();
        });
        var d3 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(getBundleDfd, newD);
        r.setDelegate(d3);
        return d3;
    },
    _commitLocalizedChanges: function() {
        // summary: Commits the changes to the localized data.
        var d;
        if (this._isLocalizedDataDirty()) {
            d = this._setLocalizedData();
        }
        return d;
    },
    _isLocalizedDataDirty: function() {
        // summary: Indicates whether or not the localized data has been modified or not since the last
        //		time it was saved.
        return !this._housekeeper.isClean;
    },
    _revertLocalizedChanges: function() {
        // summary: Discards any localized changes made locally.
        this._housekeeper.reset();
        this._localBundles = {};
    }
});

// summary: Describes a modifiable metadata provider that is backed by a file in WebDAV instead of feed data or 
//		a database.
dojo.declare("com.ibm.mm.enabler.layouttemplate.WebDAVBackedModifiableMetaData", [com.ibm.mm.enabler.layouttemplate.WebDAVBackedObject, com.ibm.mashups.enabler.ModifiableMetaData], {
    constructor: function() {
        //Separate remote data from local data. Allows us to track what's changed and what's been saved.
        this._remoteMd = undefined;
        this._localMd = {};
        this._mdNames = undefined;
        this._isMdDirty = false;
    },
    getMetaData: function(name) {
        // summary: Retrieves the metadata value associated with the given name.
        // name: the name of the metadata property
        // returns: a Deferred object that is triggered with the property value (if it exists)
        // IMPORTANT: The official interface for MetaData specifies a String returned here but we don't
        //		follow that for two reasons. One is that the RemoteModel currently has NO support for 
        //		synchronous operation (we have to request the contents of the metadata file to retrieve 
        //		this information). Two is that this really should be a Deferred object returned to account
        //		for cases like this; where retrieving the metadata requires a remote operation. The current
        //		interface is too limiting.
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            //We need to merge the remote and local data in case the value has been set on the client.
            var md = this._mergeRWMd();
            var result = md[name];
            r.resolve(result);
        });
        if (this._remoteMd === undefined && this.exists) {
            //No remote data has been retrieved. Do this first.
            //TODO: If a value has been set on the client, isn't that sufficient? Can't we avoid the request
            //in that case?
            var d = this._loadMetaData();
            var d2 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d, newD);
            r.setDelegate(d2);
            return d2;
        }
        else {
            //Otherwise, the only thing to do is retrieve the property and resolve the deferred immediately.
            r.setDelegate(newD);
            return newD;
        }
    },
    _mergeRWMd: function() {
        // summary: Merges the remote data with the local data (local takes precedence).
        // returns: an associative array where the key is the metadata name and the value is the metadata value.
        var md = this._remoteMd;
        if (typeof md == "undefined") {
            md = {};
        }
        for (var obj in this._localMd) {
            if (Object.prototype.hasOwnProperty.call(this._localMd, obj)) {
                md[obj] = this._localMd[obj];
            }
        }
        return md;
    },
    getMetaDataNames: function() {
        // summary: Retrieves the names of all the metadata properties set on a particular resource.
        // returns: a Deferred that will resolve to an array containing the names of the available properties.
        // IMPORTANT: The official interface for MetaData specifies a String [] returned here but we don't
        //		follow that for two reasons. One is that the RemoteModel currently has NO support for 
        //		synchronous operation (we have to request the contents of the localized file to retrieve 
        //		this information). Two is that this really should be a Deferred object returned to account
        //		for cases like this; where retrieving the metadata requires a remote operation. The current
        //		interface is too limiting.
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            var n = [];
            var md = this._mergeRWMd();
            for (var obj in md) {
                if (Object.prototype.hasOwnProperty.call(md, obj)) {
                    n.push(obj);
                }
            }
            this._mdNames = n;
            r.resolve(this._mdNames);
        });
        var d2 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(this.getMetaData(), newD);
        r.setDelegate(d2);
        return d2;
    },
    _loadMetaData: function() {
        // summary: Loads the remote data that describes the metadata for this resource. 
        // returns: a Deferred that resolves when the _remoteMd property has been assigned.
        var wdi = this.webDavInterface;
        var tuh = this.urlHelper;
        var p = this.propertiesFileFacade;
        var d = wdi.getFileContents(tuh.getMetaDataFileURL(this.remoteFile.getName()));
        var me = this;
        d.setFinishedCallback(function(contents) {
            var dfd = p.deserialize(contents);
            dfd.addCallback(function(props) {
                me._remoteMd = props;
            });
        });
        //d.start();
        return d;
    },
    setMetaData: function(name, value) {
        // summary: Sets the meta data value for the given name.
        this._localMd[name] = value;
        this._mdNames = undefined;
        this._isMdDirty = true;
    },
    confirmSetMetaData: function(name, value) {
        // summary: Confirms that the metadata name/value pair CAN be set.
        // returns: true if the operation is allowed, false otherwise
        return true;
    },
    removeMetaData: function(name) {
        // summary: Removes a metadata property.
        // name: the name of the metadata property to delete
        // returns: the String value of the property if it existed; NULL otherwise
        var dirty = false;
        var retVal = null;
        if (this._localMd[name] !== undefined) {
            retVal = this._localMd[name];
            delete this._localMd[name];
            dirty = true;
        }
        if (this._remoteMd[name] !== undefined) {
            retVal = this._remoteMd[name];
            delete this._remoteMd[name];
            dirty = true;
        }
        
        if (dirty) {
            this._mdNames = undefined;
            this._isMdDirty = true;
        }
        return retVal;
    },
    confirmRemoveMetaData: function(name) {
        // summary: Confirms that the metadata name/value pair CAN be removed.
        // returns: true if the operation is allowed, false otherwise
        return true;
    },
    _updateMetaData: function() {
        // summary: Updates the metadata properties file with the contents of the 
        //		new metadata.
        // returns: a Deferred object that resolves once the save operation is completed.
        var wdi = this.webDavInterface;
        var tuh = this.urlHelper;
        var p = this.propertiesFileFacade;
        
        var retVal;
        
        //If the remote data has not been previously retrieved, we need to do this FIRST.
        if (this._remoteMd === undefined && this.exists) {
            retVal = this._loadMetaData();
        }
        
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            //Save the new contents of the file.
            //TODO: Once it's possible to tell when the file doesn't yet exist, we need to handle the create
            //case here.
            var d = wdi.updateFile(tuh.getMetaDataFileURL(this.remoteFile.getName()), p.serialize(this._mergeRWMd()));
            d.setFinishedCallback(function() {
                r.resolve();
            });
            d.start();
        });
        
        if (retVal !== undefined) {
            retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(retVal, newD);
        }
        else {
            retVal = newD;
        }
        
        r.setDelegate(retVal);
        
        var resetResolver = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var resetDirtyFlag = new com.ibm.mm.enabler.DeferredImpl(this, function() {
            this._isMdDirty = false;
            resetResolver.resolve();
        });
        resetResolver.setDelegate(resetDirtyFlag);
        
        retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(retVal, resetDirtyFlag);
        
        return retVal;
    },
    _commitMetaDataChanges: function() {
        // summary: Commit the metadata changes that currently exist.
        var d;
        
        if (this._isMdDirty) {
            d = this._updateMetaData();
        }
        return d;
    },
    _isMetaDataDirty: function() {
        // summary: Indicates whether or not local changes have been made to the metadata for this resource.
        // returns: true if changes have been made, false if not.
        return this._isMdDirty;
    },
    _revertMetaDataChanges: function() {
        // summary: Discards any changes made to the metadata locally.
        this._localMd = {};
        this._isMdDirty = false;
    }
});

dojo.declare("com.ibm.mm.enabler.layouttemplate.LayoutTemplate", [com.ibm.mm.enabler.layouttemplate.WebDAVBackedModifiableMetaData, com.ibm.mm.enabler.layouttemplate.WebDAVBackedModifiableLocalized, com.ibm.mashups.enabler.layouttemplate.LayoutTemplate], {
    LAYOUT_HTML: "layout.html",
    HEAD_HTML: "head.html",
    getURL: function() {
        // summary: Get the URL to the layout.html file.
        // returns: the URL to the layout.html file inside the layout template directory
        var remoteFileURL = this.remoteFile.getURL();
        if (remoteFileURL && remoteFileURL[remoteFileURL.length - 1] !== "/") {
            remoteFileURL += "/";
        }
        return remoteFileURL + this.LAYOUT_HTML;
    },
    getHeadURL: function() {
        // summary: Get the URL to the head.html file.
        // returns: the URL to the head.html file inside the layout template directory
        // IMPORTANT: This method should be removed from the interface.
        return this.remoteFile.getURL() + this.HEAD_HTML;
    },
    getID: function() {
        // summary: Gets a unique identifier for this layout template.
        // returns: a String representing a unique identifier for this template
        return this.remoteFile.getID();
    },
    /**
     * Returns the creation date of the resource.
     * @return {Date} the creation date, never <code>null</code>.
     */
    getCreated: function() {
        if (this.remoteFile && !this.createdTime) {
            var resp = new com.ibm.mm.enabler.remote.WebDavResponseImpl();
            var created = resp.getCreationDate(this.remoteFile._properties, this.getURL());
            if (created) {
                //the created date comes back in a different format than the browser will parse
                //simplistic regexp here... could be more robust
                var dateCheck = /(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)(\.\d*)Z/;
                var match = created.match(dateCheck);
                if (match) {
                    //skip the first match - it will just be the full string
                    //second match will be the 4 digit year
                    var year = parseInt(match[1], 10);
                    //third match will be the 2 digit month
                    var month = parseInt(match[2], 10);
                    //fourth match will be the 2 digit day
                    var day = parseInt(match[3], 10);
                    //fifth match will be the 2 digit hour
                    var hour = parseInt(match[4], 10);
                    //sixth match will be the 2 digit minute
                    var minute = parseInt(match[5], 10);
                    //seventh match will be the 2 digit second
                    var seconds = parseInt(match[6], 10);
                    
                    var millis = 0;
                    if (match.length > 7) {
                        //this match will start with a period so cut that off and throw it away
                        millis = parseInt(match[7].substring(1), 10);
                    }
                    //the corresponding numbers for the months start at 0 in the Date constructor.
                    created = Date.UTC(year, month - 1, day, hour, minute, seconds, millis);
                }
                
                this.createdTime = new Date(created);
            }
        }
        return this.createdTime;
    },
    /**
     * Returns the modification date of the resource.
     * @return {Date} the modification date, never <code>null</code>.
     */
    getLastModified: function() {
        if (this.remoteFile && !this.lastModifiedTime) {
            var resp = new com.ibm.mm.enabler.remote.WebDavResponseImpl();
            var lastModified = resp.getLastModified(this.remoteFile._properties, this.getURL());
            this.lastModifiedTime = new Date(lastModified);
        }
        return this.lastModifiedTime;
    },
    _isDirty: function() {
        // summary: Indicates if this LayoutTemplate has been changed since the last save.
        // description: Typically only called by the model to query the state of the templates that it has
        //		loaded.
        // returns: true if the layout template has changed; false if it has not
        return this._isMetaDataDirty() || this._isLocalizedDataDirty();
    },
    _commit: function() {
        // summary: Commits the changes for this layout template.
        // returns: a Deferred object that resolves when the save for this template is completed.
        var d1 = this._commitLocalizedChanges();
        var d2 = this._commitMetaDataChanges();
        var retVal;
        
        if (d1 && d2) {
            //There are changes to both the localized data AND the metadata.
            retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d1, d2);
        }
        else if (d1 || d2) {
            //There are changes to either the localized data OR the metadata.
            retVal = d1 ? d1 : d2;
        }
        
        return retVal;
    },
    _create: function() {
        var wdi = this.webDavInterface;
        var urls = this.urlHelper;
        var d1 = wdi.createDirectory(urls.baseDirectory + this.remoteFile.getName());
        var d2 = this._commit();
        var retVal = d1;
        if (d2 !== undefined) {
            retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d1, d2);
        }
        return retVal;
    },
    _delete: function() {
        var wdi = this.webDavInterface;
        var urls = this.urlHelper;
        var d1 = wdi.remove(urls.baseDirectory + this.remoteFile.getName());
        return d1;
    },
    _revert: function() {
        this._revertLocalizedChanges();
        this._revertMetaDataChanges();
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplate"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplate"] = true;
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplate");





}

if(!dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel_API");
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel");






/**
 * This list model controller exposes layout templates. It allows to add and
 * remove layout templates.
 * 
 * @since 2.4
 * 
 * @ibm-api
 * @ibm-module LayoutTemplateModel
 */
dojo.declare("com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel", [
		com.ibm.mashups.enabler.ListModelController,
		com.ibm.mashups.enabler.ResourceLocator,
		com.ibm.mashups.enabler.Commitable ], {
	/**
	 * Creates a layout template. Note that it will not appear in the model unless
	 * it is inserted using the <code>insert</code> method.
	 *
	 * @param {JSON} context array of predefined information used for the creation of
	 * the layout template. Must not be <code>null</code>. You must specify all of following:<br>
	 * &nbsp;&nbsp;<b><code>name</code></b> - name as string; mandatory <br>
	 * @return {com.ibm.mashups.enabler.layouttemplate.LayoutTemplate} the created layout template, or <code>null</code> in case
	 * it cannot be created.
	 */
	create : function(context) {
	}
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.layouttemplate.LayoutTemplateModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.layouttemplate.LayoutTemplateModel"] = true;
dojo.provide("com.ibm.mm.enabler.layouttemplate.LayoutTemplateModel");







// summary: Wraps the RemoteFileIterator to convert the remote file objects into LayoutTemplate objects.
// IMPORTANT: Synchronous iteration is broken due to the fact that the RemoteFileIterator does not provide
//		support for this type of iteration. Strangely, it also does not implement the setForEachCallback
//		option either. See the LayoutTemplateTests for an example of how to use this iterator until those
//		shortcomings are fixed. The RemoteFileIterator also has a bug where the size returned is too large
//		by one. This implementation works around that bug, but this workaround will need to be removed 
//		when this bug is fixed in the remote iterator.
dojo.declare("com.ibm.mm.enabler.layouttemplate.DeferredIteratorImpl", com.ibm.mashups.enabler.DeferredIterator, {
    constructor: function(baseDirFar, ltModel, remoteModel){
        // important to omit the leading slash!
        //Setup the deferred that will load the iterator.
        var me = this;
        var baseURL = com.ibm.mm.enabler.layouttemplate.TemplateURLHelper.baseDirectory;
        baseDirFar.setFinishedCallback(function(template){
            var hrefs = template._getChildHrefs();
            var l = hrefs.length;
            
            for (var i = 1; i < l; i++) {
                var indexOf = hrefs[i].indexOf(baseURL);
                if (indexOf >= 0) {
                    hrefs[i] = hrefs[i].substring(indexOf);
                }
            }
            
            me._hrefs = hrefs;
        });
        this._deferred = baseDirFar;
        this._ltModel = ltModel;
        this._remoteModel = remoteModel;
        this._position = 1;
        this._hrefs = undefined;
    },
    start: function(sync){
        // summary: Starts the request to retrieve the iterator if it is undefined.
        // sync: indicates if this should be a synchronous request
        // IMPORTANT: sync currently does nothing
        if (sync) {
            console.error("Layout template iterator does not support synchronous iteration currently.");
        }
        if (this._hrefs === undefined) {
            this._deferred.start();
        }
    },
    setForEachCallback: function(callback, parameters){
        // summary: Sets the callback function that will be called for each item in the collection.
        // callback: the callback function
        // parameters: additional parameters to pass to the callback function
        // IMPORTANT: this method does nothing currently
        console.warn("setForEachCallback is not currently enabled in the Remote iterator! This callback will never get called!");
        //this._iterator.setForEachCallback( callback, parameters );
    },
    setFinishedCallback: function(callback, parameters){
        // summary: Sets the callback that is called when the iterator is loaded and initialized.
        // callback: the callback function
        // parameters: additional parameters to pass to the callback function
        var d = new com.ibm.mm.enabler.DeferredImpl(this, function(){
            callback(parameters);
        });
        this._deferred = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(this._deferred, d);
    },
    //Iterator methods
    hasNext: function(){
        // summary: Indicates if the iterator has any elements remaining.
        // returns: true if another element remains; false if none remain
        if (this._hrefs === undefined) {
            console.error("Iterator has not been loaded yet. Please call start() and call this method in or after the setFinishedCallback() callback is called.");
        }
        else {
            return this._position < this._hrefs.length;
        }
    },
    next: function(){
        // summary: Retrieves the next item in the iteration. Note that calling next() before the iterator 
        //		is defined will automatically wait until the iterator exists before resolving the callback.
        // returns: a Deferred that resolves to the next item in the iteration
        var rv;
        
        var me = this;
        var hrefs = this._hrefs;
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var next = hrefs === undefined ? undefined : this._hrefs[this._position++];
        var d2 = new com.ibm.mm.enabler.DeferredImpl(this, function(){
			
            var nextItem;
			if (next) {
				if (next.indexOf("access-control.xml") == -1) {
					nextItem = next;
				}
				else {
					nextItem = me._hrefs[me._position++];
				}
			} 
            
			if (nextItem) {
                var nextFar = me._ltModel.find(nextItem);
                nextFar.setFinishedCallback(function(object){
                    if (object) {
                        r.resolve(object);
                    }
                    else {
                        me.next();
                    }
                });
                nextFar.start();
            }
            else {
                r.resolve(undefined);
            }
        });
        r.setDelegate(d2);
        
        rv = this._hrefs === undefined ? com.ibm.mm.enabler.layouttemplate.DeferredManager(this._deferred, d2) : d2;
        
        return rv;
    },
    size: function(){
        // summary: Retrieves the number of elements in this iteration.
        // returns: a Deferred that resolves with the size of the iteration.
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var size = this._hrefs.length;
        var d = new com.ibm.mm.enabler.DeferredImpl(this, function(){
            //TODO: Fix once the Remote iterator is fixed.
            r.resolve(size - 1);
        });
        r.setDelegate(d);
        
        return d;
    },
    setCursorPosition: function(newPos){
        // summary: Sets the cursor position.
        // IMPORTANT: Only works as much as the remote iterator has implemented it
        //this.start( true );
        this._position = newPos;
    },
    getCursorPosition: function(){
        // summary: Gets the cursor position.
        // returns: the current cursor position
        // IMPORTANT: Only works as much as the remote iterator has implemented it
        //this.start( true );
        return this._position;
    }
});


dojo.declare("com.ibm.mm.enabler.layouttemplate.LayoutTemplateModel", com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel, {
    constructor: function(){
        this.remoteModel = com.ibm.mashups.enabler.remote.Factory.getRemoteModel();
        this._localCache = {};
        this._baseLayoutLocation = undefined;
    },
    iterator: function(){
        // summary: Retrieves a DeferredIterator that will iterate over the layout templates.
        // returns: a DeferredIterator.
        // IMPORTANT: The DeferredIterator does NOT support the synchronous operation mode. This is due
        //		to the fact that the RemoteModel currently has NO support for synchronous operations. 
        
        var d;
        if (this._baseLayoutLocation === undefined) {
            var baseURL = com.ibm.mm.enabler.layouttemplate.TemplateURLHelper.baseDirectory;
            d = this.remoteModel.find(baseURL);
            var me = this;
            d.setFinishedCallback(function(template){
                me._baseLayoutLocation = template;
            });
        }
        
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var d2 = new com.ibm.mm.enabler.DeferredImpl(this, function(){
            r.resolve(this._baseLayoutLocation);
        });
        r.setDelegate(d2);
        
        var baseLayoutFar;
        if (d) {
            baseLayoutFar = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d, d2);
        }
        else {
            baseLayoutFar = d2;
        }
        
        return new com.ibm.mm.enabler.layouttemplate.DeferredIteratorImpl(baseLayoutFar, this, this.remoteModel);
    },
    find: function(id){
        // summary: Finds the layout template with the given ID. Currently, this ID should be the path
        //		(relative to the WebDAV root) to the layout template directory.
        // returns: a Deferred object that resolves to the LayoutTemplate object requested, if it exists
        return this._deferredFind(id);
    },
    isDirty: function(){
        // summary: Indicates if any of the retrieved layout templates have been modified.
        // returns: true if any templates have changed since the last save; false if not.
        var retVal = false;
        var c = this._localCache;
        for (var node in c) {
            if (Object.prototype.hasOwnProperty.call(c, node) && c[node]._isDirty) {
                retVal = retVal || c[node]._isDirty();
            }
        }
        
        return retVal;
    },
    commit: function(){
        // summary: Commits any changed layout templates.
        // returns: a Deferred that resolves when all save operations are complete
        var c = this._localCache;
        var retVal;
        for (var node in c) {
        
            if (Object.prototype.hasOwnProperty.call(c, node) && c[node]._commit && c[node]._isDirty && c[node]._isDirty()) {
                var d = c[node]._commit();
                if (d) {
                    if (!retVal) {
                        retVal = d;
                    }
                    else {
                        retVal = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(retVal, d);
                    }
                }
            }
        }
        
        return retVal;
    },
    _deferredFind: function(id){
        // summary: Handles retrieving a template either remotely or locally.
        // returns: a Deferred that resolves when the template is retrieved
        
        //Resolves once the item has been loaded in to the local cache.
        var r = new com.ibm.mm.enabler.layouttemplate.DeferredResolver();
        var newD = new com.ibm.mm.enabler.DeferredImpl(this, function(item){
            r.resolve(this._localCache[id]);
        });
        if (this._localCache[id] === undefined) {
            //The item does not currently exist in the local cache. Request it and resolve the find before
            //resolving the retrieval from local cache.
            var me = this;
            var d = this.remoteModel.find(id);
            d.setFinishedCallback(function(item){
                var n = null;
                if (item.isDirectory()) {
                    n = new com.ibm.mm.enabler.layouttemplate.LayoutTemplate(me.remoteModel, item, com.ibm.mm.enabler.layouttemplate.WebDAVInterface, com.ibm.mm.enabler.layouttemplate.TemplateURLHelper, com.ibm.mm.enabler.layouttemplate.PropertiesFileFacade, true);
                }
                me._localCache[id] = n;
            });
            var suffix = "/";
            if (id.indexOf("access-control.xml") == -1) {
                suffix = "metadata" + suffix;
            }
            var dNLS = this.remoteModel.find(id + suffix);
            dNLS.setFinishedCallback(function(nlsItem){
                me._localCache[id].setRemoteNls(nlsItem);
            });
            
            var d2 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d, dNLS);
            var d3 = com.ibm.mm.enabler.layouttemplate.DeferredManager.combine(d2, newD);
            r.setDelegate(d3);
            return d3;
        }
        else {
            //Item already exists in local cache - this results in an immediate resolution of the Deferred.
            r.setDelegate(newD);
            return newD;
        }
    },
    //create methods
    create: function(context){
        var remoteFile = new com.ibm.mm.enabler.remote.RemoteFileImpl({
            name: context.name,
            isDirectory: true
        });
        var n = new com.ibm.mm.enabler.layouttemplate.LayoutTemplate(this.remoteModel, remoteFile, com.ibm.mm.enabler.layouttemplate.WebDAVInterface, com.ibm.mm.enabler.layouttemplate.TemplateURLHelper, com.ibm.mm.enabler.layouttemplate.PropertiesFileFacade, false);
        return n;
    },
    confirmCreate: function(){
        return true;
    },
    insert: function(layoutTemplate){
        var d = layoutTemplate._create();
        d.start();
    },
    confirmInsert: function(){
        return true;
    },
    remove: function(layoutTemplate){
        var d = layoutTemplate._delete();
        d.start();
    },
    confirmRemove: function(){
        return true;
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel"] = true;
dojo.provide("com.ibm.mashups.enabler.layouttemplate.LayoutTemplateModel");





}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.CatalogEntryModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.CatalogEntryModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.CatalogEntryModel_API");
dojo.provide("com.ibm.mashups.enabler.model.CatalogEntryModel");

/**
 * Interface for a catalog model containing entry nodes. This list model
 * describes a list of concrete catalog entries for a given category.
 * @ibm-api
 * @ibm-module CatalogModel
 */
dojo.declare("com.ibm.mashups.enabler.model.CatalogEntryModel", com.ibm.mashups.enabler.ListModelController, {
    /**
     * creates a new CatalogEnryNode. The created node can be inserted into the
     * model using an appropriate <code>insert</code> method. The node will not
     * appear in the model unless it is inserted.
     * @param {JSON} context array of predefined information
     * used for the creation of the node. May be <code>null</code>. Accepted
     * names are:<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>cid</code> - the client id for the entry
     * to be created (optional)<br>
     * @return {com.ibm.mashups.enabler.catalog.CatalogEntryNode} the created node
     */
    create: function(context){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.ModelIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.ModelIteratorImpl"] = true;
dojo.provide("com.ibm.mm.enabler.ModelIteratorImpl");











dojo.declare("com.ibm.mm.enabler.ModelIteratorImpl", [com.ibm.mm.enabler.DeferredIteratorImpl], {
    constructor: function(url, model, parentId) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        // parent id
        this.parentId = parentId;
        
        // the iterator's model
        this.model = model;
        
        // entries in feed order; undefined entries denote potential 
        // nodes to load; null entry marks end of node collection 
        this.entries = [];
        
        // cursor position
        this.cursor = 0;
        
        // number of nodes
        this.sizeValue = null;
        
        // atom feed
        this.ATOM_FEED = "atom:feed";
        
        // atom entry
        this.ATOM_ENTRY = this.ATOM_FEED + "/atom:entry";
        
        // next link
        this.ATOM_NEXT = this.ATOM_FEED + "/atom:link[@rel='next']";
        
        // size indicator
        this.OPENSEARCH_TOTALRESULTS = this.ATOM_FEED + "/opensearch:totalResults";
        
        if (url) {
            // collection url
            this.collectionUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(url, model);
            
            if (model.strategy) {
                for (var x in model.strategy) {
                    if (com.ibm.mm.enabler.utils.Misc.isInstanceOf(model.strategy[x], com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy)) {
                        // max number of entries 
                        this.collectionUrl.setParameter("num", model.strategy[x].getInterval());
                        // start element (only set with paging) 
                        this.collectionUrl.setParameter("start", this.getCursorPosition());
                        break;
                    }
                }
            }
            
            // let iterator implementations customize the url
            this._customizeUrl(this.collectionUrl);
        }
        
        // number of created nodes to include in this iterator
        this.numCreatedNodes = 0;
        
        // compile key-value pairs for each created node included in this 
        // iterator: (createdId, nextId)
        this._createdNodes = {};
        com.ibm.mm.enabler.utils.Misc.forIn(this.model.createdNodes, function(value, createdId) {
            var createdParent = this.model.getParent(createdId).start();
            if (createdParent && this.parentId == createdParent.getID()) {
                var created = this.model.find(createdId).start();
                // next link of created node
                var next = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:link[@rel='next']", created.getXml(), this.model.ns);
                this._createdNodes[createdId] = next ? com.ibm.mm.enabler.utils.Utils.getIdFromExtUri(this.model.prefix, next) : null;
                this.numCreatedNodes++;
            }
        }, this);
        
        // number of removed nodes not to include in this iterator
        this.numRemovedNodes = 0;
        
        // if the url is not present, removed nodes do not count towards size anymore
        if (url) {
            com.ibm.mm.enabler.utils.Misc.forIn(this.model.deletedNodes, function(value, removedId) {
                var parentLink = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("thr:in-reply-to[@ext:uri]", this.model.deletedNodes[removedId].getXml(), this.model.ns);
                if (parentLink) {
                    if (this.parentId == com.ibm.mm.enabler.utils.Utils.getIdFromExtUri(this.model.prefix, parentLink)) {
                        this.numRemovedNodes++;
                    }
                }
            }, this);
        }
    },
    
    // This abstract function must be implemented for each iterator implementation.
    // It is invoked for each atom entry in the collection, where the atom entry 
    // is passed in as xml data. This function must return an instance of a 
    // resource based on the xml data, or null, in case no node is to be created.
    _createNode: function(xmlData) {
        return null;
    },
    
    // This abstract function allows to modify the collection url, e.g. by adding 
    // url parameters. This method is applied on the url created in the constructor 
    // of the base iterator implementation, as well as on all 'next'-link urls, 
    // which are obtained from collection chunks in order to obtain the next chunk. 
    _customizeUrl: function(url) {
    },
    
    hasNext: function(dfr, sync) {
        if (typeof this.entries[this.cursor] == "undefined") {
            var insertionIndex = 0;
            if (this.collectionUrl) {
                // load nodes from server
                this._loadAhead(dfr || null, true);
                insertionIndex = this.cursor;
            }
            else {
                // do not load entries
                this.entries[0] = null;
            }
            // inject nodes, which are created at runtime
            this._insertCreated(this.entries, insertionIndex);
            
            // reflect number of created/removed nodes in size value
            this.sizeValue = this.sizeValue + this.numCreatedNodes - this.numRemovedNodes;
        }
        
        var candidate = this.entries[this.cursor];
        var hasNext = candidate ? true : false;
        if (hasNext && this.model.deletedNodes && this.model.deletedNodes[candidate]) {
            // do not expose nodes, which are deleted at runtime
            this.cursor++;
            return this.hasNext(dfr, sync);
        }
        return hasNext;
    },
    
    next: function(dfr, sync) {
        return this.hasNext(dfr || null, typeof sync == "undefined" ? true : sync) ? this.model.loadedNodes[this.entries[this.cursor++]] : null;
    },
    
    _insertCreated: function(entries, cursor) {
        while (entries[cursor]) {
            var currentId = entries[cursor];
            for (var createdId in this._createdNodes) {
                if (currentId == this._createdNodes[createdId]) {
                    // insert created node if not done already
                    if (cursor > 0 && entries[cursor - 1] != createdId) {
                        entries.splice(cursor, 0, createdId);
                    }
                }
            }
            cursor++;
        }
        // handle nodes appended to the end of the list
        if (entries[cursor] === null) {
            for (var createdLastId in this._createdNodes) {
                if (null === this._createdNodes[createdLastId]) {
                    entries.splice(cursor, 0, createdLastId);
                    cursor++;
                }
            }
        }
    },
    
    setCursorPosition: function(position) {
        this.cursor = position;
        this.collectionUrl.setParameter("start", position);
    },
    
    getCursorPosition: function() {
        return this.cursor;
    },
    
    size: function() {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._size);
    },
    
    _size: function(dfr, sync) {
        if (!this.sizeValue) {
            this.hasNext(null, true);
        }
        
        // finished callback 
        if (dfr) {
            dfr.finish(this.sizeValue, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
        }
        // convenience
        return this.sizeValue;
    },
    
    start: function(sync) {
        this._loadAhead(this, sync, true);
    },
    
    _loadAhead: function(deferred, sync, iterate) {
        // logging support
        var LOG_METHOD = "_loadAhead";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        
        // handle empty iterator
        if (!this.collectionUrl) {
            this.entries[this.cursor] = null;
            if (deferred) {
                deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
            return;
        }
        
        // service request
        var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(this.collectionUrl, null, null, false, sync);
        serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
            
                // fetch size
                this.sizeValue = com.ibm.mashups.enabler.xml.XPath.evaluateNumber(this.OPENSEARCH_TOTALRESULTS, data, this.model.ns);
                
                // iterate atom entries
                var c = 0;
                var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(this.ATOM_ENTRY, data, this.model.ns);
                if (nodes) {
                    for (var i = 0, l = nodes.length; i < l; i++) {
                        // instantiate node
                        var node = this._createNode(nodes[i]);
                        if (node) {
                            // put id in entry list
                            this.entries[this.cursor + c++] = node.getID();
                            
                            // callback handling
                            if (deferred) {
                                deferred.nextFinish(node);
                            }
                        }
                        else {
                            // an entry for which no node is instantiated  
                            // does not count towards collection size
                            this.sizeValue--;
                        }
                    }
                }
                // check for 'next'-link on feed level
                var next = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(this.ATOM_NEXT, data, this.model.ns);
                if (next) {
                    var url = next.getAttribute("href");
                    
                    // set collection url from 'next'-link
                    this.collectionUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(url, this.model);
                    
                    // let iterator implementations customize the url
                    this._customizeUrl(this.collectionUrl);
                    if (nodes && iterate) {
                        // was last next element
                        this._loadAhead(deferred, sync, iterate);
                    }
                }
                else {
                    // mark end of collection
                    this.entries[this.cursor + c] = null;
                    
                    // finished callback (end of collection)
                    if (deferred) {
                        deferred.finish(null, xhr.status);
                    }
                }
            }
            else {
                // finished callback (error)
                if (deferred) {
                    deferred.finish(null, xhr.status);
                }
            }
        }));
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryIteratorImpl"] = true;
// APIs



// Implementations





dojo.provide("com.ibm.mm.enabler.catalog.CatalogEntryIteratorImpl");

// navigation model iterator
dojo.declare("com.ibm.mm.enabler.catalog.CatalogEntryIteratorImpl", [com.ibm.mm.enabler.ModelIteratorImpl], {    
    _createNode: function(xmlData) {
		// instantiate node
		var node = new com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl(xmlData);

		// cache node in model
        node = this.model._cacheNode(node);

        return node;
    },
    
    _customizeUrl: function(url) {
		// access control info
        url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
		// full representation
        url.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);

		// hidden meta data
		this.model._applyHiddenMetaDataLoadingStrategyToModelURL(url);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogEntryModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.catalog.CatalogEntryModelImpl");

































// catalog entry model
dojo.declare("com.ibm.mm.enabler.catalog.CatalogEntryModelImpl", [com.ibm.mashups.enabler.model.CatalogEntryModel, com.ibm.mashups.enabler.ListModel, com.ibm.mm.enabler.DeferredIteratorImpl, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.ConfirmableModelImpl, com.ibm.mm.enabler.strategy.HiddenMetaDataLoadingStrategyHelperImpl], {
    constructor: function(category, parentModel) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        this.category = category;
        this.parentModel = parentModel;
        this.strategy = parentModel.strategy;
        this.loadedNodes = {};
        this.createdNodes = {};
        this.deletedNodes = {};
        this.initialContents = {};
        this.nodeIds = [];
        this.cid = 0;
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_CATALOG);
        this.prefix = this.serviceJson.idprefix;
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_OPENSEARCH, nsf.NS_XSI]));
        
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:link[@ext:rel='entries']", this.category.xmlData, this.ns);
        this.collectionUrl = node.getAttribute("href");
    },
    
    find: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, uri);
    },
    
    _find: function(deferred, sync, uri) {
        // logging support
        var LOG_METHOD = "_find";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync, uri]);
        }
        
        if (this.category.getID() == "CATROOT") {
            return null;
        }
        if (uri in this.deletedNodes) {
            if(deferred) { deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND); }
            return null;
        }
        if (uri in this.loadedNodes) {
            // fetch node from cache
            if (deferred && deferred.getFinishedCallback()) {
                // callback handling
                if(deferred) { deferred.finish(this.loadedNodes[uri], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK); }
            } else {
                return this.loadedNodes[uri];
            }
        }
        else {
            // prepare xhr
            var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.CATALOG_URL, this);
            myUrl.setNodes([{
                value: com.ibm.mm.enabler.utils.Misc.encodeModelID4Uri(uri),
                isID: true
            }, {
                value: com.ibm.mm.enabler.utils.Misc.encodeModelID4Uri(this.category.getID()),
                isID: true
            }]);
            myUrl.setParameter("rep", "full");
            myUrl.addParameter("aspect", "ac");
            var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
            var entry = null;
            var result = {};
            serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
                // obtain node
                var expr = "atom:feed/atom:entry";
                var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, data, this.ns);
                if (node) {
                    this._cacheNode(new com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl(node));
                }
                // callback handling
                if(deferred) { deferred.finish(this.loadedNodes[uri] || null, xhr.status); }
                // handle error in sync call
                if (sync && type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                    // do not expose HTTP_NOT_FOUND; simply return null in this case
                    if (xhr.status != com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND) {
                        result.status = xhr.status;
                    }
                }
            }));
            
            if (result.status) {
                throw new Error(dojo.string.substitute(this.modelMessages.E_CATNODE_NOTFOUND_1, [uri, result.status]));
            }
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
        return this.loadedNodes[uri];
    },
    
    _cacheNode: function(node) {
        var id = node.getID();
        if (!(id in this.loadedNodes)) {
            this.loadedNodes[id] = node;
            this.initialContents[id] = node.toString();
        }
        return this.loadedNodes[id];
    },
    
    iterator: function() {        
    	if (!this.entries) {
    		this.entries = new com.ibm.mm.enabler.catalog.CatalogEntryIteratorImpl(this.collectionUrl, this);
    	} else {			   
    		this.entries.setCursorPosition(0);
    	}
        return this.entries;
    },
    
    create: function(context) {
        // logging support
        var LOG_METHOD = "create";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [context]);
        }
        
        var clientId;
        if (context && context.cid) {
            // use cid specified with context
            clientId = context.cid;
        }
        else {
            // use generated cid
            clientId = "" + this.cid++;
        }
        var entry = com.ibm.mm.enabler.utils.Utils.createNode("atom:entry", this.ns.atom);
        var idElement = com.ibm.mm.enabler.utils.Utils.createNode("atom:id", this.ns.atom);
        com.ibm.mm.enabler.utils.Dom.textContent(idElement, "cid:" + clientId + "@" + this.prefix + ":" + this.category.getID());
        entry.appendChild(idElement);
        var contentElement = com.ibm.mm.enabler.utils.Utils.createNode("atom:content", this.ns.atom);
        entry.appendChild(contentElement);
        var catalogEntryElement = com.ibm.mm.enabler.utils.Utils.createNode("model:catalog-entry", this.ns.model);
        contentElement.appendChild(catalogEntryElement);
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("model:title", this.ns.model));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("model:description", this.ns.model));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createExtLinkNode("", null, "definition"));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createExtLinkNode("", null, "icon"));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createExtLinkNode("", null, "preview"));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createExtLinkNode("", null, "previewThumbnail"));
        catalogEntryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createExtLinkNode("", null, "help"));
        var ret = new com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl(entry);
        
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, [ret]);
        }
        
        return ret;
    },
    confirmCreate: function(context) {
        return this.getConfirmCreate();
    },
    
    insert: function(node, nextNode) {
        // logging support
        var LOG_METHOD = "insert";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [node, nextNode]);
        }
        var id = node.getID();
        // check if the node is inserted or moved
        if (!(id in this.loadedNodes)) {
            // cache node
            this.loadedNodes[id] = node;
            // mark as created
            this.createdNodes[id] = null;
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    confirmInsert: function(node, nextNode) {
        return this.getConfirmInsert();
    },
    
    remove: function(node) {
        // logging support
        var LOG_METHOD = "remove";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [node]);
        }
        
        var id = (com.ibm.mm.enabler.utils.Misc.isInstanceOf(node, com.ibm.mm.enabler.catalog.CatalogEntryNodeImpl)) ? node.getID() : node;
        // mark node as deleted
        if (!(id in this.deletedNodes)) {
            this.deletedNodes[id] = this.loadedNodes[id];
        }
        // remove cache entry
        if (id in this.loadedNodes) {
            delete this.loadedNodes[id];
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    confirmRemove: function(node) {
        return this.getConfirmRemove(node);
    },
    
    // internal commit
    _commit: function(deferred, sync, mpHandler) {
        // logging support
        var LOG_METHOD = "_commit";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync, mpHandler]);
        }
        
        var serviceReq;
        var myUrl, urlId;
        
        for (var id in this.loadedNodes) {
			if (Object.prototype.hasOwnProperty.call(this.loadedNodes,id)) {
				var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
				var entry = com.ibm.mm.enabler.utils.Utils.createFeed("catalog:" + id, "IBM Lotus Mashups Catalog Feed", this.loadedNodes[id], this.ns);
				if (id in this.createdNodes) {
					// handle created nodes
					myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.collectionUrl, this);
					
					// HTTP POST created nodes
					this.parentModel._acquire();
					serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
					serviceReq.create(entry, dojo.hitch(this, function(type, data, xhr, args){
						if (deferred) {
							// public callback handling
							if (deferred.operationCallback) {
								deferred.operationCallback(this.loadedNodes[args], com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE, xhr.status, deferred.operationCallbackParameters);
							}
							// internal callback handling
							if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
								if (deferred.errorCallback) {
									deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
								}
							}
							else {
								this.length++;
							}
						}
						this.parentModel._release(deferred, xhr.status);
					}), id); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
				}
				else {
					// skip  unmodified nodes
					if (this.initialContents[id] === this.loadedNodes[id].toString()) {
						continue;
					}
					// handle modified nodes
                    var _editLink = this.loadedNodes[id].getEditLink(true);
                    if (!_editLink) {
                        if (deferred) {
                            // public callback handling
                            if (deferred.operationCallback) {
                                deferred.operationCallback(this.loadedNodes[id], com.ibm.mashups.enabler.DeferredOperation.OPERATION_MODIFY, 403, deferred.operationCallbackParameters);
                            }
                        }
                    }
                    else {
                        myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(_editLink, this);
                        
                        // HTTP PUT modified nodes
                        myUrl.setParameter("update", "replace");
                        this.parentModel._acquire();
                        serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                        serviceReq.update(entry, dojo.hitch(this, function(type, data, xhr, args){
                            if (deferred) {
                                // public callback handling
                                if (deferred.operationCallback) {
                                    deferred.operationCallback(this.loadedNodes[args], com.ibm.mashups.enabler.DeferredOperation.OPERATION_MODIFY, xhr.status, deferred.operationCallbackParameters);
                                }
                                // internal callback handling
                                if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                                    if (deferred.errorCallback) {
                                        deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
                                    }
                                }
                            }
                            this.parentModel._release(deferred, xhr.status);
                        }), id); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                        // mark new contents
                        this.initialContents[id] = this.loadedNodes[id].toString();
                    }
				}
				// unmark created nodes
				this.createdNodes = {};
			}
		}
        
        // HTTP DELETE deleted nodes
        for (var id2 in this.deletedNodes) {
			if (Object.prototype.hasOwnProperty.call(this.deletedNodes,id2)) {
				myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.deletedNodes[id2].getEditLink(), this);
				
				this.parentModel._acquire();
				serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
				serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args){
					if (deferred) {
						// public callback handling
						if (deferred.operationCallback) {
							deferred.operationCallback(args, com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE, xhr.status, deferred.operationCallbackParameters);
						}
						if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
							if (deferred.errorCallback) {
								deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
							}
						}
					}
					this.parentModel._release(deferred, xhr.status);
				}), id2); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
			}
		}
        
        // unmark deleted nodes
        this.deletedNodes = {};
        
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.CatalogEntryModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.CatalogEntryModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.CatalogEntryModel");







}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.CatalogCategoryModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.CatalogCategoryModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.CatalogCategoryModel_API");
dojo.provide("com.ibm.mashups.enabler.model.CatalogCategoryModel");

/**
 * Interface for a catalog model containing category nodes. This tree model
 * describes the topology in which the categories are structured.
 * As of now only one level below the root is supported; no child categories.
 * @ibm-api
 * @ibm-module CatalogModel
 */
dojo.declare("com.ibm.mashups.enabler.model.CatalogCategoryModel", com.ibm.mashups.enabler.TreeModelController, {
    /**
     * Returns the catalog entry model for the given category.
     * @param {com.ibm.mashups.enabler.catalog.CatalogCategoryNode} category the CatalogCategoryNode for which
     * to return the CatalogEntryModel; must not be <code>null</code>
     * @type com.ibm.mashups.enabler.catalog.CatalogEntryModel
     * @return catalog entry model for given category, Never <code>null</code>.
     */
    getCatalogEntryModel: function(category){
    },
    
    /**
     * creates a new CatalogCategoryNode. The created node can be inserted into
     * the model using an appropriate <code>insert</code> method. The node will
     * not appear in the model unless it is inserted.
     * @param {JSON} context array of predefined information
     * used for the creation of the node. May be <code>null</code>. Accepted names are:<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>category</code> - the name of the category
     * to be created (mandatory)<br>
     * @type com.ibm.mashups.enabler.catalog.CatalogCategoryNode
     * @return the created node
     */
    create: function(context){
    },
    
    /**
     * Commits the modifications applied to this model and all dependent models.<br>
     * @return {DeferredOperation} a deferred object used to start this operation. 
     * The return value when executed through the deferred object is <code>null</null>
     */
    commit: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryIteratorImpl"] = true;
// APIs



// Implementations





dojo.provide("com.ibm.mm.enabler.catalog.CatalogCategoryIteratorImpl");

// navigation model iterator
dojo.declare("com.ibm.mm.enabler.catalog.CatalogCategoryIteratorImpl", [com.ibm.mm.enabler.ModelIteratorImpl], {    
    _createNode: function(xmlData) {
		// instantiate node
		var node = new com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl(xmlData);

		// cache node in model
        node = this.model._cacheNode(node);

        return node;
    },
    
    _customizeUrl: function(url) {
		// access control info
        url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
		// full representation
        url.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);

		// hidden meta data
		this.model._applyHiddenMetaDataLoadingStrategyToModelURL(url);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl");



































// catalog category model
dojo.declare("com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl", [com.ibm.mashups.enabler.model.CatalogCategoryModel, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.ConfirmableModelImpl, com.ibm.mm.enabler.strategy.HiddenMetaDataLoadingStrategyHelperImpl], {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        this.root = null;
        this.loadedNodes = {};
        this.createdNodes = {};
        this.deletedNodes = {};
        this.initialContents = {};
        this.entryModels = {};
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_CATALOG);
        this.entryUrl = this.serviceJson.url;
        this.prefix = this.serviceJson.idprefix;
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_OPENSEARCH]));
    },
    
    // singleton
    getInstance: function() {
        var instance = com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl._instance = new com.ibm.mm.enabler.catalog.CatalogCategoryModelImpl());
    },
    
    getCatalogEntryModel: function(category) {
        if (!(category in this.entryModels)) {
            this.entryModels[category] = new com.ibm.mm.enabler.catalog.CatalogEntryModelImpl(category, this);
        }
        return this.entryModels[category];
    },
    
    _cacheNode: function(node) {
        var id = node.getID();
        if (!(id in this.loadedNodes)) {
            this.loadedNodes[id] = node;
            this.initialContents[id] = node.toString();
        }
        return this.loadedNodes[id];
    },
    
    find: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, uri);
    },
    
    _find: function(deferred, sync, uri) {
        // logging support
        var LOG_METHOD = "_find";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync, uri]);
        }
        
        // check if node is deleted in controller scope
        if (uri in this.deletedNodes) {
            // callback handling
            if (deferred) {
                deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
            }
            return null;
        }
        
        // fetch node from cache
        if (uri in this.loadedNodes) {
            // public callback handling
            if (deferred) {
                deferred.finish(this.loadedNodes[uri], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
        }
        else {
            // prepare xhr
            var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.CATALOG_URL, this);
            myUrl.setNodes([{
                value: com.ibm.mm.enabler.utils.Misc.encodeModelID4Uri(uri),
                isID: true
            }]);
            myUrl.setParameter("rep", "full");
            myUrl.addParameter("aspect", "ac");
            var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
            var entry = null;
            var result = {};
            serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
                // obtain node
                var expr = "atom:feed/atom:entry";
                var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, data, this.ns);
                if (node) {
                    var entry = new com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl(node);
                    if (entry !== null) {
                        entry.setParentModel(this);
                        if (!(uri in this.loadedNodes)) {
                            this.loadedNodes[uri] = entry;
                            this.initialContents[uri] = entry.toString();
                        }
                    }
                }
                // handle callback
                if (deferred) {
                    deferred.finish(this.loadedNodes[uri] || null, xhr.status);
                }
                // handle error in sync call
                if (sync && type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                    // do not expose HTTP_NOT_FOUND; simply return null in this case
                    if (xhr.status != com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND) {
                        result.status = xhr.status;
                    }
                }
            }));
            if (result.status) {
                throw new Error(dojo.string.substitute(this.modelMessages.E_CCATNODE_NOTFOUND_1, [uri, result.status]));
            }
            
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
        return this.loadedNodes[uri];
    },
    
    create: function(context) {
        // logging support
        var LOG_METHOD = "create";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [context]);
        }
        var ret = null;
        if (context && context.category) {
            var entry = com.ibm.mm.enabler.utils.Utils.createNode("atom:entry", this.ns.atom);
            var idElement = com.ibm.mm.enabler.utils.Utils.createNode("atom:id", this.ns.atom);
            com.ibm.mm.enabler.utils.Dom.textContent(idElement, "catalog:" + this.prefix + ":" + context.category);
            entry.appendChild(idElement);
            entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:title", this.ns.atom));
            entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:summary", this.ns.atom));
            entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:published", this.ns.atom));
            entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:updated", this.ns.atom));
            var contentElement = com.ibm.mm.enabler.utils.Utils.createNode("atom:content", this.ns.atom);
            entry.appendChild(contentElement);
            var catalogCategoryElement = com.ibm.mm.enabler.utils.Utils.createNode("model:catalog-category", this.ns.model);
            contentElement.appendChild(catalogCategoryElement);
            catalogCategoryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("model:title", this.ns.model));
            catalogCategoryElement.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("model:description", this.ns.model));
            ret = new com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl(entry);
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, [ret]);
        }
        return ret;
    },
    confirmCreate: function(context) {
        return this.getConfirmCreate();
    },
    
    commit: function() {
        return new com.ibm.mm.enabler.DeferredOperationImpl(this, this._commit);
    },
    
    _commit: function(deferred, sync) {
        // logging support
        var LOG_METHOD = "_commit";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync]);
        }
        // initialize callback handling
        this.statusCode = com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK;
        this.requestCount = 0;
        this._acquire();
        var serviceReq;
        // commit category model changes
        for (var id in this.loadedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.loadedNodes,id)) {
                var time = new Date();
                var entry = '<?xml version="1.0" encoding="UTF-8"?>\n' +
                this.loadedNodes[id];
                var createUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.entryUrl, this);
                if (id in this.createdNodes) {
                    // HTTP POST created nodes
                    this._acquire();
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(createUrl, null, null, false, sync);
                    serviceReq.create(entry, dojo.hitch(this, function(type, data, xhr, args) {
                        if (deferred) {
                            // public callback handling
                            if (deferred.operationCallback) {
                                deferred.operationCallback(this.loadedNodes[args], com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE, xhr.status, deferred.operationCallbackParameters);
                            }
                            // internal callback handling
                            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                                if (deferred.errorCallback) {
                                    deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
                                }
                            }
                        }
                        this._release(deferred, xhr.status);
                    }), id); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
                else {
                    // skip  unmodified nodes
                    if (this.initialContents[id] === this.loadedNodes[id].toString()) {
                        continue;
                    }
                    // HTTP PUT modified nodes
                    var updateUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.loadedNodes[id].getEditLink(), this);
                    updateUrl.setParameter("update", "replace");
                    this._acquire();
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(updateUrl, null, null, false, sync);
                    serviceReq.update(entry, dojo.hitch(this, function(type, data, xhr, args) {
                        if (deferred) {
                            // public callback handling
                            if (deferred.operationCallback) {
                                deferred.operationCallback(this.loadedNodes[args], com.ibm.mashups.enabler.DeferredOperation.OPERATION_MODIFY, xhr.status, deferred.operationCallbackParameters);
                            }
                            // internal callback handling
                            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                                if (deferred.errorCallback) {
                                    deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
                                }
                            }
                        }
                        this._release(deferred, xhr.status);
                    }), id); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
        }
        
        // HTTP DELETE deleted nodes
        for (var deleted_id in this.deletedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.deletedNodes,deleted_id)) {
                var deleteUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.deletedNodes[deleted_id].getEditLink(), this);
                this._acquire();
                serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(deleteUrl, null, null, false, sync);
                serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args) {
                    if (deferred) {
                        // public callback handling
                        if (deferred.operationCallback) {
                            deferred.operationCallback(args, com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE, xhr.status, deferred.operationCallbackParameters);
                        }
                        // internal callback handling
                        if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                            if (deferred.errorCallback) {
                                deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
                            }
                        }
                    }
                    this._release(deferred, xhr.status);
                }), deleted_id); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
            }
        }
        
        // commit the entry model changes
        for (var entry_id in this.entryModels) {
            if (Object.prototype.hasOwnProperty.call(this.entryModels,entry_id)) {
                this.entryModels[entry_id]._commit(deferred, sync);
            }
        }
        
        // the finishedCallback is called in _release
        this._release(deferred, 0);
        
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    
    // acquire semaphore
    _acquire: function() {
        this.requestCount++;
    },
    
    // release semaphore
    _release: function(deferred, statusCode) {
        // set overall status Code
        this.statusCode = Math.max(parseInt(statusCode, 10), this.statusCode);
        // check for full release
        this.requestCount--;
        if (this.requestCount <= 0) {
            if (deferred) {
                deferred.finish(null, this.statusCode);
            }
        }
    },
    
    getRoot: function() {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._getRoot);
    },
    
    _getRoot: function(deferred, sync) {
        if (this.root === null) {
            var node = com.ibm.mm.enabler.utils.Utils.createNode("atom:entry", this.ns.atom);
            node.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:id", this.ns.atom));
            this.root = new com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl(node);
            this.root.setParentModel(this);
            this.root.setID("CATROOT");
        }
        return this.root;
    },
    
    hasChildren: function(node) {
        return (node.getID() == "CATROOT") ? true : false;
    },
    
    getChildren: function(node) {
        // fix collection url
        if (!this.children) {
            this.children = new com.ibm.mm.enabler.catalog.CatalogCategoryIteratorImpl(this.serviceJson.url, this);
        }
        else {
            this.children.setCursorPosition(0);
        }
        return this.children;
    },
    
    getParent: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._getParent, uri);
    },
    
    _getParent: function(deferred, sync, node) {
        return (node.getID() == "CATROOT") ? null : this.getRoot();
    },
    
    insert: function(node) {
        // cache and mark as created
        this.loadedNodes[node.getID()] = node;
        this.createdNodes[node.getID()] = null;
    },
    confirmInsert: function(node) {
        return this.getConfirmCreate();
    },
    
    remove: function(node) {
        // logging support
        var LOG_METHOD = "remove";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [node]);
        }
        
        var id = (com.ibm.mm.enabler.utils.Misc.isInstanceOf(node, com.ibm.mm.enabler.catalog.CatalogCategoryNodeImpl)) ? node.getID() : node;
        // mark node as deleted
        if (!(id in this.deletedNodes)) {
            this.deletedNodes[id] = this.loadedNodes[id];
        }
        // remove cache entry
        if (id in this.loadedNodes) {
            delete this.loadedNodes[id];
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    confirmRemove: function(node) {
        return this.getConfirmRemove(node);
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.CatalogCategoryModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.CatalogCategoryModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.CatalogCategoryModel");









}

if(!dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteFile_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteFile_API"] = true;
dojo.provide("com.ibm.mashups.enabler.remote.RemoteFile_API");
dojo.provide("com.ibm.mashups.enabler.remote.RemoteFile");

/**
 * Interface for a remote File.
 * @ibm-api
 * @ibm-module RemoteModel
 */
dojo.declare("com.ibm.mashups.enabler.remote.RemoteFile", [com.ibm.mashups.enabler.MetaData, com.ibm.mashups.enabler.Identifiable], {
    /**
     * Returns the name of this File
     * @return {String} The name
     */
    getName: function(){
    },
    
    /**
     * Returns true if this is a directory otherwise returns false.
     * @return {boolean} True if this is a directory
     */
    isDirectory: function(){
    },
    
    /**
     * Returns the absolute URL of this file.
     * @return {String} The URL
     * @deprecated Use getURL instead.
     */
    getUrl: function(){
    },
    
    /**
     * Returns the absolute URL of this file.
     * @return {String} The URL
     */
    getURL: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WebDavUrlFactoryImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WebDavUrlFactoryImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WebDavUrlFactoryImpl");




/*
 * A private object that supplies the urls used to access the webdav server
 * Since this is a private object, this does not implement a public interface but that could easily be done.
 */
dojo.declare("com.ibm.mm.enabler.remote.WebDavUrlFactoryImpl", com.ibm.mm.enabler.ServiceDocConsumer, {

    _initServiceDoc: function() {
    	this.inherited("_initServiceDoc", arguments);
        // Get the absolute url
        var collectionData = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData([com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_WEBDAV, com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_FILESTORE]);
        
        if (collectionData) {
            this._fullUrl = collectionData.url;
            
            var idx = this._fullUrl.indexOf("//");
            var tempString = this._fullUrl.substring(idx + 2);
            var idx2 = tempString.indexOf("/");
            var domainLength = idx + idx2 + 2;
            
            // This is the protocol, domain and port
            // example: http://localhost:8080
            this._domainUrl = this._fullUrl.substring(0, domainLength);
            
            // This is the path to the filestore root
            // example: /mum/mycontenthandler/mm/dav/mmdav/
            this._context = this._fullUrl.substring(domainLength);
        }
    },
    
    /*
     * Returns an absolute path to a webdav resource
     */
    createUrl: function(path) {
    	this._initServiceDoc();
        var uri = null;
        if (!path) {
            uri = this._fullUrl;
        }
        else if (path == this._fullUrl) {
            // they're asking for root but they're missing the elusive double slash
            uri = path + "/";
        }
        else if (path.indexOf(this._fullUrl) === 0) {
            // this is already an absolute path
            uri = path;
        }
        else if (path.indexOf(this._context) === 0) {
            // this has the context but not the domain etc
            uri = this._domainUrl + path;
        }
        else {
            // this is just the relative path, make it absolute
            uri = this._fullUrl + path;
        }
        return uri;
    },
    
    getUserHome: function() {
        var userHome = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData([com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_WEBDAV, com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_USER]);
        
        return userHome.url;
    }
});

// create a static var to make it easier to call
com.ibm.mm.enabler.remote.WebDavUrlFactory = new com.ibm.mm.enabler.remote.WebDavUrlFactoryImpl();

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WebDavCommandImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WebDavCommandImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WebDavCommandImpl");




/*
 * A private object that hides xhr requests to a Webdav server
 * Since this is a private object, this does not implement a public interface but that could easily be done.
 */
dojo.declare("com.ibm.mm.enabler.remote.WebDavCommandImpl", null, {

    statics: {
        timeout: 1000 * 60 * 3 // 3 mins       
    },
    
    /*
     * action: the http method to use
     * path: the path of the resource
     * form: (optional) the id of a form node that contains data to send with the request
     * data: (optional) the data to send within the body of the action
     * headers: (optional) a map of headers & values to send with the request
     * handleAs: should be "xml" or "text"
     */
    constructor: function(action, path, form, data, headers, handleAs) {
        this._action = action;
        if (dojo.isIE) {
            // Internet Explorer sucks, it does not encode the url for ajax when it should
            // without this, urls with unicode chars break (ie filenames with asian chars)        	
            this._path = encodeURI(path);
        }
        else {
            this._path = path;
        }
        this._handleAs = handleAs;
        this._form = form;
        this._data = data;
        this._headers = headers;
    },
    
    /*
     * Returns an absolute uri for the file
     */
    getURI: function() {
        var uri = com.ibm.mm.enabler.remote.WebDavUrlFactory.createUrl(this._path);
        return uri;
    },
    
    /*
     * Returns a dojo.deferred
     */
    execute: function(sync) {
    
        var actionArgs = {
            url: this.getURI(),
            sync: sync || false,
            handleAs: this._handleAs,
            timeout: this.statics.timeout,
			preventCache: dojo.isIE <7			
        };
        
        // check if we need to send a body with the request as well
        var hasBody = false;
        if (this._data) {
            actionArgs.postData = this._data;
            hasBody = true;
        }
        
        // add any custom headers if applicable
        if (this._headers) {
            actionArgs.headers = this._headers;
        }
        
        if (this._form) {
            // if we're given a form that contains a file input, we can't use ajax (because of security constraints) 
            // instead we need to use an iframe which unfortunately can only can do a POST.			
            actionArgs.form = this._form;
            actionArgs.method = "POST";
            
            // dojo.io.iframe expects a special html response wrapped in a <textarea>
            // add a special param so that the Portal resolver knows to wrap the response 
            if (actionArgs.url.indexOf('?') == -1) {
                actionArgs.url = actionArgs.url + "?mime-type=text/html";
            }
            else {
                actionArgs.url = actionArgs.url + "&mime-type=text/html";
            }
            
            return dojo.io.iframe.send(actionArgs);
        }
        else {
            return dojo.xhr(this._action, actionArgs, hasBody);
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WebDavResponseImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WebDavResponseImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WebDavResponseImpl");




/*
 * A private object that hides handling of an xml response to an xhr requests
 * Since this is a private object, this does not implement a public interface but that could easily be done.
 */
dojo.declare("com.ibm.mm.enabler.remote.WebDavResponseImpl", null, {

    // We have to pass the namespace to the XPath engine
    namespace: {
        "D": "DAV:",
        "a": "DAV:",
        "dav": "DAV:"
    },
    
    paths: {
        collection: "//D:multistatus/D:response/D:propstat/D:prop/D:resourcetype/D:collection",
        href: "//D:multistatus/D:response/D:href",
        lastmodified: "//D:multistatus/D:response[D:href='${path}']/D:propstat/D:prop/D:getlastmodified",
        creationdate: "//D:multistatus/D:response[D:href='${path}']/D:propstat/D:prop/D:creationdate"
    },
    
    constructor: function() {
    },
    
    /*
     * Returns an array of nodes
     * If this is an empty array, then the resource is a collection
     */
    isCollection: function(data) {
        return com.ibm.mashups.enabler.xml.XPath.evaluateBoolean(this.paths.collection, data, this.namespace);
    },
    
    /*
     * Returns an array of nodes
     * If this is a collection, there will be one node per child containing it's href
     * Otherwise, there will be just a single node that contains the href of the item.
     */
    getHrefs: function(data) {
        var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(this.paths.href, data, this.namespace);
        return this.convertNodesToStrings(nodes);
    },
    
    getID: function(data, fn) {
        if (!data) {
            return null;
        }
        var nodes = this.getHrefs(data);
        return nodes && nodes.length > 0 ? nodes[0] : null;
    },
    
    getLastModified: function(data, fileHref) {
        var expr = this.paths.lastmodified.replace(/\$\{path\}/, fileHref);
        var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(expr, data, this.namespace);
        var arr = this.convertNodesToStrings(nodes);
        return arr && arr.length > 0 ? arr[0] : null;
    },
    
    getCreationDate: function(data, fileHref) {
        var expr = this.paths.creationdate.replace(/\$\{path\}/, fileHref);
        var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(expr, data, this.namespace);
        var arr = this.convertNodesToStrings(nodes);
        return arr && arr.length > 0 ? arr[0] : null;
    },
    
    /*
     * Converts a node[] to a string[]
     */
    convertNodesToStrings: function(nodes) {
        var strings = null;
        if (nodes) {
            strings = [];
            for (var i = 0, l = nodes.length; i < l; i++) {
                // we need to use this dojo method for browser compatability 
                var s = dojox.xml.parser.textContent(nodes[i]);
                // the contents are url encoded so we need to decode this
                strings[i] = decodeURI(s);
            }
        }
        return strings;
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WebDavCommandFactoryImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WebDavCommandFactoryImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WebDavCommandFactoryImpl");




/*
 * A private factory object that creates WebDavCommandImpl objects.
 * Since this is a private object, this does not implement a public interface but that could easily be done/
 */
dojo.declare("com.ibm.mm.enabler.remote.WebDavCommandFactoryImpl", null, {

    REQUEST_METHOD_MKCOL: "MKCOL",
    REQUEST_METHOD_PROPFIND: "PROPFIND",
    REQUEST_METHOD_PUT: "PUT",
    REQUEST_METHOD_DELETE: "DELETE",
	POST_ACTION_HEADER: "X-Method-Override",

    constructor: function() {
    },
    defaultProperties: {
        "resourcetype": true
    },
    defaultPropfindDataString: '<?xml version="1.0" encoding="utf-8" ?><D:propfind xmlns:D="DAV:"><D:prop><D:resourcetype/></D:prop></D:propfind>',
    propfindParameterizedDataString: '<?xml version="1.0" encoding="utf-8" ?><D:propfind xmlns:D="DAV:"><D:prop ${xmlns}>${props}</D:prop></D:propfind>',
    webDAVNamespaceAlias: "D",
    requiredWebDAVProperties: {
        "resourcetype": true
    },
    getDownloadCommand: function(path) {
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("GET", path, null, null, null, "text");
    },
    
    getCreateCommand: function(path, node, content) {
        var headers = null;
        if ((!node) && (content)) {
            // we're uploading content and not a file so set the content type correctly
            // without this, it will default to form/urlencoded which the portal resolver 
            // does not handle consistently
            headers = {
                "content-type": "text/html;charset=utf-8"
            };
        }
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("POST", path, node, content, headers, "text");
    },
    
    getCopyCommand: function(path, dest) {
        var headers = {
            "destination": encodeURI(dest)
        };
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("COPY", path, null, null, headers, "text");
    },
    
    getMoveCommand: function(path, dest) {
        var headers = {
            "destination": encodeURI(dest),
            "overwrite": false
        };
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("MOVE", path, null, null, headers, "text");
    },
    
    getUpdateCommand: function(path, node, content) {
        if (com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.TUNNEL_MODE) === true) {
            var headers = {
                "depth": "1"
            };
            if ((!node) && (content)) {
                // we're uploading content and not a file so set the content type correctly
                // without this, it will default to form/urlencoded which the portal resolver
                // does not handle consistently
                headers["content-type"] = "text/html;charset=utf-8";
            }
            headers[this.POST_ACTION_HEADER] = this.REQUEST_METHOD_PUT;
            return new com.ibm.mm.enabler.remote.WebDavCommandImpl("POST", path, node, content, headers, "text");
        }
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("PUT", path, node, content, null, "text");
    },

    
    getDeleteCommand: function(path) {
        var headers = {
        	"depth": "1"
        };
        if (com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.TUNNEL_MODE) === true) {
        	headers[this.POST_ACTION_HEADER] = this.REQUEST_METHOD_DELETE;
        	return new com.ibm.mm.enabler.remote.WebDavCommandImpl("POST", path, null, null, headers, "text");
        }
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("DELETE", path, null, null, null, "text");
    },
    
    getMakeCollectionCommand: function(path) {
        var headers = {
            "depth": "1"
        };
        if (com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.TUNNEL_MODE) === true) {
            headers[this.POST_ACTION_HEADER] = this.REQUEST_METHOD_MKCOL;
            return new com.ibm.mm.enabler.remote.WebDavCommandImpl("POST", path, null, null, headers, "xml");
        }
								
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("MKCOL", path, null, null, headers, "xml");
    },
    
    getFindPropertiesCommand: function(path, props) {
        // the server will respond with whatever properties we ask for so send a small list.
        // including the resourcetype allows us to determine if it's a collection or not
        //<D:getlastmodified/><D:lockdiscovery/><D:displayname/><D:getcontentlength/><D:getcontenttype/>
        var data = this.defaultPropfindDataString;
        
        if (props) {
            data = this.propfindParameterizedDataString;
            props = dojo.mixin(props, this.defaultProperties);
            var propsString = "";
            var xmlnsString = "";
            //TODO: Devise a better way to get a unique namespace alias... Not sure if a number is a valid alias...
            //Skip "D" because it's already in use for the WebDAV namespace
            var aliases = ["A", "B", "C", "E", "F", "G", "H", "I"];
            var aliasIndex = 0;
            for (var prop in props) {
                if (Object.prototype.hasOwnProperty.call(props,prop)) {
                    var alias = this.webDAVNamespaceAlias;
                    var namespace = "";
                    var indexOfHash = prop.indexOf("#");
                    
                    if (indexOfHash >= 0) {
                        namespace = prop.substring(0, indexOfHash);
                        prop = prop.substring(indexOfHash + 1);
                        alias = aliases[aliasIndex++];
                        xmlnsString += alias + ":" + namespace + " ";
                    }
                    
                    propsString += "<" + alias + ":" + prop + "/>";
                }
            }
            
            data = data.replace(/\$\{props\}/, propsString);
            data = data.replace(/\$\{xmlns\}/, xmlnsString);
        }
        
        var headers = {
            "depth": "1"
        };
        if (com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.TUNNEL_MODE) === true) {
            headers[this.POST_ACTION_HEADER] = this.REQUEST_METHOD_PROPFIND;
            return new com.ibm.mm.enabler.remote.WebDavCommandImpl("POST", path, null, data, headers, "xml");
        }
				
        return new com.ibm.mm.enabler.remote.WebDavCommandImpl("PROPFIND", path, null, data, headers, "xml");
    }
});

// create a static var to make it easier to call
com.ibm.mm.enabler.remote.WebDavCommandFactory = new com.ibm.mm.enabler.remote.WebDavCommandFactoryImpl();

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WebDavDeferredImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WebDavDeferredImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WebDavDeferredImpl");



/*
 * A simple implementation of com.ibm.mashups.enabler.Deferred
 */
dojo.declare("com.ibm.mm.enabler.remote.WebDavDeferredImpl", com.ibm.mashups.enabler.Deferred, {

    constructor: function(context, startfn) {
        // context for start function
        this._context = context;
        // start function
        this._startfn = startfn;
    },
    
    setFinishedCallback: function(callback, parameters) {
        this._callback = callback;
        this._parameters = parameters;
    },
    
    start: function(sync) {
        // we only support asynchronous..
        if ((this._context) && (this._startfn)) {
            dojo.hitch(this._context, this._startfn)(sync);
        }
        return null;
    },
    
    doCallback: function(resource, status) {
        if (this._callback) {
            this._callback(resource, status, this._parameters);
        }
    }
    
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.RemoteFileImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.RemoteFileImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.RemoteFileImpl");









/*
 * Implementation of com.ibm.mashups.enabler.remote.RemoteFile
 */
dojo.declare("com.ibm.mm.enabler.remote.RemoteFileImpl", [com.ibm.mashups.enabler.remote.RemoteFile], {

    constructor: function(context) {
    
        if (context && context.name) {
            this._name = context.name;
        }
        else {
            this._name = null;
        }
        
        this._children = null;
        this._isDirectory = context.isDirectory;
        this._id = null;
        this._properties = null; // a Document containing relevant properties 
        this._context = context;
        this._deferred = null;
    },
    
    /**
     * @private
     * used by consumer to get Deferred object
     */
    init: function() {
        this._deferred = new com.ibm.mm.enabler.remote.WebDavDeferredImpl(this, this._init);
        return this._deferred;
    },
    
    /**
     * @private
     * called by WebDavDeferredImpl.start() and triggers the actual request
     */
    _init: function(sync) {
        if (this._context && this._context.isCreate) {
            if ((this._context.isDirectory) && (this._context.form)) {
                // we're uploading a zip 
                this._uploadDirectory(this._name, this._context.form);
            }
            else if (this._context.isDirectory) {
                // we're creating an empty directory
                this._createDirectory(this._name, sync);
            }
            else if (!this._context.isDelete) {
                // we're creating a new file
                this._createFile(this._name, this._context.form, this._context.content, sync);
            }
        }
        else if (this._context && this._context.isUpdate) {
            // we're updating an existing file
            this._updateFile(this._name, this._context.form, this._context.content);
        }
        else {
            // we're just loading info about the file
            this._load(this._name);
        }
    },
    
    /**
     * @private
     * Loads information from the server
     */
    _load: function(name) {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var propsToRequest = this._context && this._context.props ? this._context.props : null;
        var command = commandFactory.getFindPropertiesCommand(name, propsToRequest);
        var d = command.execute();
        
        // this callback will process the data
        d.addCallback(dojo.hitch(this, this._handleLoad));
        // this callback will notify the caller's callbacks
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
    },
    
    /**
     * @private
     * Creates a file
     */
    _createFile: function(name, form, content, sync) {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var command = commandFactory.getCreateCommand(name, form, content);
        var d = command.execute(sync);
        
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
    },
    
    /**
     * @private
     * Updates an existing file
     */
    _updateFile: function(name, form, content) {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var command = commandFactory.getUpdateCommand(name, form, content);
        var d = command.execute();
        
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
    },
    
    /**
     * @private
     * Creates a new directory
     */
    _createDirectory: function(name, sync) {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var command = commandFactory.getMakeCollectionCommand(name);
        var d = command.execute(sync);
        
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
    },
    
    /**
     * @private
     * Uploads a zip file and creates a new directory
     */
    _uploadDirectory: function(name, form) {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var command = commandFactory.getUpdateCommand(name, form, null);
        var d = command.execute();
        
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
    },
    
    /**
     * @private
     * a private callback
     */
    _handleLoad: function(loadData) {
        // this should be an xml document 
        // I'd love to do an error check on this but that breaks IE :( 
        this._properties = loadData;
    },
    
    /**
     * @private
     * A private success callback that triggers the caller's callback
     * Internally we use a dojo.deferred but the caller expects a com.ibm.mashups.enabler.Deferred so
     * this does the "conversion". It's also useful for debugging / logging
     */
    _handleSuccess: function(data) {
        // This callback does not get the http status info, just the returned object.
        // Return a 200 to indicate this was succesful
        this._deferred.doCallback(this, 200);
    },
    
    /**
     * @private
     * A private error handler that just calls the defferred's errback
     * Internally we use a dojo.deferred but the caller expects a com.ibm.mashups.enabler.Deferred so
     * this does the "conversion". It's also useful for debugging / logging
     */
    _handleError: function(error) {
        // this is a normal error scenario
        // unfortunately com.ibm.mashups.enabler.Deferred does not support a seperate error callback
        // which means the caller has to understand http status codes and the server has to send the correct one..
        // we don't have the actual status code but we know this is an error so return a 500
        this._deferred.doCallback(this, 500);
    },
    
    /**
     * @private
     * deletes this file
     */
    _delete: function() {
        var commandFactory = com.ibm.mm.enabler.remote.WebDavCommandFactory;
        var command = commandFactory.getDeleteCommand(this.getID());
        var d = command.execute();
        
        d.addCallback(dojo.hitch(this, this._handleSuccess));
        d.addErrback(dojo.hitch(this, this._handleError));
        
        return this._deferred;
    },
    
    getName: function() {
        // this should only return the name and not the full path even though we
        // may be storing the full name
        
        var ret = this._name;
        
        if ("/" == this._name) {
            // do nothing	
        }
        else {
            var parts = this._name.split("/");
            var c = this._name.charAt(this._name.length - 1);
            if (c == "/") {
                ret = parts[parts.length - 2];
            }
            else {
                ret = parts[parts.length - 1];
            }
        }
        
        return ret;
    },
    
    isDirectory: function() {
        if (!this._isDirectory) {
            var resp = new com.ibm.mm.enabler.remote.WebDavResponseImpl();
			if (!this._properties) {
				this._properties = document;
			}
            this._isDirectory = resp.isCollection(this._properties);
        }
        
        return this._isDirectory;
    },
    
    /**
     * @private
     */
    _getChildHrefs: function() {
        if (!this._children) {
            this._children = [];
            
            if (this.isDirectory()) {
                var resp = new com.ibm.mm.enabler.remote.WebDavResponseImpl();
                var nodes = resp.getHrefs(this._properties);
                
                var i = 0;
                for (var node in nodes) {
                    if (Object.prototype.hasOwnProperty.call(nodes,node)) {
                        if (i > 0) {
                            // the first file is the current file so we skip it
                            this._children[i] = nodes[node];
                        }
                        i++;
                    }
                }
            }
        }
        
        return this._children;
    },
    getID: function() {
        if (!this._id) {
            var resp = new com.ibm.mm.enabler.remote.WebDavResponseImpl();
            var id = resp.getID(this._properties);
            
            if (id) {
                this._id = id;
            }
            else {
                this._id = com.ibm.mm.enabler.remote.WebDavUrlFactory.createUrl(this._name);
            }
        }
        return this._id;
    },
    
    // @deprecated
    getUrl: function() {
        return this.getURL();
    },
    getURL: function() {
        return this.getID();
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteFile"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteFile"] = true;
dojo.provide("com.ibm.mashups.enabler.remote.RemoteFile");








}

if(!dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteLocator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.remote.RemoteLocator"] = true;
dojo.provide("com.ibm.mashups.enabler.remote.RemoteLocator");




/**
 * Interface for finding RemoteFile objects.
 * @ibm-api
 * @ibm-module RemoteModel
 */
dojo.declare("com.ibm.mashups.enabler.remote.RemoteLocator", null, {
    /**
     * Returns the current user's home Directory.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for 
     * which the callback will be passed a RemoteFile.
     * @see RemoteFile 
     */
    getUserRoot: function(){
    
    }
  
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.RemoteModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.RemoteModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.RemoteModel_API");
dojo.provide("com.ibm.mashups.enabler.model.RemoteModel");

/**
 * Interface for a remote File.
 *
 * @ibm-api
 * @ibm-module RemoteModel
 */
dojo.declare("com.ibm.mashups.enabler.model.RemoteModel", [com.ibm.mashups.enabler.TreeModel, com.ibm.mashups.enabler.remote.RemoteLocator], {});


}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.WidgetHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.WidgetHelper"] = true;
dojo.provide("com.ibm.mm.enabler.remote.WidgetHelper");







/**
 * A RemoteModel helper object for widgets.
 * This is a protected api.
 */
dojo.declare("com.ibm.mm.enabler.remote.WidgetHelper", null, {
    /**
     * @private
     */
    constructor: function(model) {
        this._model = model;
    },
    
    /**
     * Returns a url that will retrieve the specified Directory as a zip file.
     * @param {com.ibm.mashups.enabler.remote.RemoteFile} dir The directory
     * @return {String} The modified url
     */
    getDirectoryAsZipUrl: function(dir) {
        return dir.getID() + '?mime-type=application/zip';
    },
    
    /**
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a com.ibm.mashups.enabler.remote.RemoteFile.
     */
    addNewTemplate: function(name, form) {
        if (name && name.length > 0) {
            // this has to be a collection, if the user did not pass in a 
            // name ending in a slash, add the slash
            var lastChar = name.charAt(name.length - 1);
            if (lastChar != '/') {
                name = name + '/';
            }
        }
        var d = new com.ibm.mm.enabler.DeferredImpl(this, this._addNewTemplate, {
            "name": name,
            "form": form
        });
        return d;
    },
    
    _addNewTemplate: function(deferred, sync, params) {
    
        params.deferred = deferred;
        
        var d = this.getTemplateDir();
        d.setFinishedCallback(function(templateDir, status, params) {
            var context = {};
            context.name = templateDir.getUrl() + params.name + "?mode=replace";
            context.isDirectory = true;
            context.isCreate = true;
            context.form = params.form;
            
            var file = new com.ibm.mm.enabler.remote.RemoteFileImpl(context);
            var d2 = file.init();
            d2.setFinishedCallback(function(template, status, params) {
                var originalDeferred = params.deferred;
                if (originalDeferred) {
                    originalDeferred.finish(template, status);
                }
            }, params);
            d2.start();
            
        }, params);
        d.start();
    },
    
    /**
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a com.ibm.mashups.enabler.remote.RemoteFile.
     */
    uploadZip: function(path, name, form) {
        if (name && name.length > 0) {
            // this has to be a collection, if the user did not pass in a 
            // name ending in a slash, add the slash
            var lastChar = name.charAt(name.length - 1);
            if (lastChar != '/') {
                name = name + '/';
            }
        }
        var d = new com.ibm.mm.enabler.DeferredImpl(this, this._uploadZip, {
            "path": path,
            "name": name,
            "form": form
        });
        return d;
    },
    
    _uploadZip: function(deferred, sync, params) {
        params.deferred = deferred;
        
        var d = this.getZipDir(params.path);
        d.setFinishedCallback(function(zipDir, status, params) {
            var context = {};
            context.name = zipDir.getUrl() + params.name + "?mode=merge";
            context.isDirectory = true;
            context.isCreate = true;
            context.form = params.form;
            
            var file = new com.ibm.mm.enabler.remote.RemoteFileImpl(context);
            var d2 = file.init();
            d2.setFinishedCallback(function(template, status, params) {
                var originalDeferred = params.deferred;
                if (originalDeferred) {
                    originalDeferred.finish(template, status);
                }
            }, params);
            d2.start();
            
        }, params);
        d.start();
    },
    
    getZipDir: function(path) {
        return this._model.find(path);
    },
    
    /**
     * Returns the remote directory where Template objects are stored.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a com.ibm.mashups.enabler.remote.RemoteFile.
     */
    getTemplateDir: function() {
        return this._model.find("/public/templates/");
    },
    
    /**
     * Returns the remote directory where Project objects are stored.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a com.ibm.mashups.enabler.remote.RemoteFile.
     */
    getProjectDir: function() {
        var userhome = com.ibm.mm.enabler.remote.WebDavUrlFactory.getUserHome();
        return this._model.find(userhome + "/public/projects/");
    },
    
    /**
     * Returns a new File object created from the supplied context.
     * @param {Object} context The file context
     * 	context.name = the name of the file
     *  context.content = (optional) the contents of the file
     *  context.form = (optional) the id of a form that contains the contents of the file.
     *  If uploading a file, the following should be true:
     *  	The form action should be POST
     *  	The form enctype should be multipart/form-data
     *  	The input of type file should have both a valid id and name.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a RemoteFile.
     */
    createFile: function(context) {
        context.isDirectory = false;
        context.isCreate = true;
        var file = new com.ibm.mm.enabler.remote.RemoteFileImpl(context);
        return file.init();
    },
    
    /**
     * Returns a new File object created from the supplied context.
     * @param {String} path The full path of the folder to create
     */
    createFolder: function(path) {
        var context = {};
        context.isDirectory = true;
        context.isCreate = true;
        context.name = path;
        var file = new com.ibm.mm.enabler.remote.RemoteFileImpl(context);
        return file.init();
    },
    
    /**
     * Updates an existing file
     * @param {com.ibm.mashups.enabler.remote.RemoteFile} file The file to update
     *  context.content = (optional) the new contents of the file
     *  context.form = (optional) the id of a form that contains the new contents of the file
     *  If uploading a file, the following should be true:
     *  	The form action should be POST
     *  	The form enctype should be multipart/form-data
     *  	The input of type file should have both a valid id and name.
     * @param {Object} context The file context
     * @type void
     */
    updateFile: function(file, context) {
        context.isDirectory = false;
        context.isUpdate = true;
        
        var returnFile = new com.ibm.mm.enabler.remote.RemoteFileImpl(context);
        return returnFile.init();
    },
    
    /**
     * Copies an existing file or folder
     * @param {com.ibm.mashups.enabler.remote.RemoteFile} file The RemoteFile to copy
     * @param {String} newPath The destination file / folder name. This must be an absolute path.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a RemoteFile.
     */
    copy: function(file, newPath) {
        var copyCommand = com.ibm.mm.enabler.remote.WebDavCommandFactory.getCopyCommand(file.getUrl(), newPath);
        var ed = new com.ibm.mm.enabler.remote.WebDavDeferredImpl();
        var d = copyCommand.execute();
        d.addCallback(dojo.partial(this._copy, ed));
        
        return ed;
    },
    
    _copy: function(ed, result) {
        ed.doCallback(result, 200);
    },
    
    /**
     * Moves an existing file or folder
     * @param {com.ibm.mashups.enabler.remote.RemoteFile} file The RemoteFile to copy
     * @param {String} newPath The destination file / folder name. This must be an absolute path.
     * 	known bug: the full path is required but it won't be used. The file will always be moved to the same folder as the source.
     * 	IE you cannot move a file to a different folder.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a RemoteFile.
     */
    move: function(file, newPath) {
        var moveCommand = com.ibm.mm.enabler.remote.WebDavCommandFactory.getMoveCommand(file.getUrl(), newPath);
        var ed = new com.ibm.mm.enabler.remote.WebDavDeferredImpl();
        var d = moveCommand.execute();
        d.addCallback(dojo.partial(this._move, ed));
        
        return ed;
    },
    
    _move: function(ed, result) {
        ed.doCallback(result, 200);
    },
    
    /**
     * Deletes an existing file
     * @param {com.ibm.mashups.enabler.remote.RemoteFile} file The file to delete
     * @type void
     */
    deleteFile: function(file) {
    	//16293: adding isDelete flag to prevent deletes from executing create logic in com.ibm.mm.enabler.remote.RemoteFileImpl._init
    	file._context.isDelete=true;
        return file._delete();
    }
    
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.RemoteIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.RemoteIteratorImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.RemoteIteratorImpl");



/*
 * Implementation of com.ibm.mm.enabler.DeferredIteratorImpl that returns com.ibm.mm.enabler.remote.RemoteFileImpl
 */
dojo.declare("com.ibm.mm.enabler.remote.RemoteIteratorImpl", com.ibm.mm.enabler.DeferredIteratorImpl, {
    constructor: function(model, hrefs) {
        this._model = model;
        this._hrefs = hrefs;
        this._position = 1;
    },
    
    start: function(sync) {
    
    },
    
    hasNext: function(sync) {
        return this._position < this._hrefs.length;
    },
    
    next: function(sync) {
        var href = this._hrefs[this._position];
        var next = null;
        if (href) {
            this._position++;
            next = this._model.find(href);
        }
        
        return next;
    },
    
    setCursorPosition: function(position) {
        this._position = position;
    },
    
    getCursorPosition: function() {
        // we increment position after we fetch it in next() so we need to decrement before we return it here
        return (this._position - 1);
    },
    
    size: function() {
        return this._hrefs.length;
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.remote.RemoteModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.remote.RemoteModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.remote.RemoteModelImpl");







 
dojo.declare("com.ibm.mm.enabler.remote.RemoteModelImpl", [com.ibm.mashups.enabler.model.RemoteModel, com.ibm.mm.enabler.model.ModelImpl], {

    constructor: function() {
        this._files = {};
    },
    
    getRoot: function() {
        return this.find('/');
    },
    
    find: function(path) {
        var file = new com.ibm.mm.enabler.remote.RemoteFileImpl({
            name: path,
            props: {
            	"getlastmodified": true,
            	"resourcetype": true,
            	"creationdate": true
            }
        });
        var d = file.init();
        return d;
    },
    
    /**
     * Returns the current user's home Directory.
     * @return {com.ibm.mashups.enabler.Deferred} A Deferred object for which the callback will be passed
     * a RemoteFile.
     */
    getUserRoot: function() {
        var userhome = com.ibm.mm.enabler.remote.WebDavUrlFactory.getUserHome();
        return this.find(userhome);
    },
    
    hasChildren: function(file) {
        var kids = file._getChildren();
        if (kids) {
            for (var child in kids) {
                if (Object.prototype.hasOwnProperty.call(kids,child)) {
                    return true;
                }
            }
        }
        return false;
    },
    
    getParent: function(file) {
        var id = file.getID();
        var index = id.lastIndexOf('/');
        var parentPath = id.substring(0, index);
        if (file.isDirectory()) {
            parentPath = parentPath + "/../";
        }
        
        return this.find(parentPath);
    },
    
    getChildren: function(file) {
        return new com.ibm.mm.enabler.remote.RemoteIteratorImpl(this, file._getChildHrefs());
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.RemoteModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.RemoteModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.RemoteModel");







}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SkinModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SkinModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SkinModel_API");
dojo.provide("com.ibm.mashups.enabler.model.SkinModel");

/**
 * Interface for a skin model. This list model describes a list of skin objects.<br>
 * <br>
 * The skin model supports the following strategies:
 * <ul>
 * <li>{@link com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy ListLoadAheadStrategy}</li>
 * <li>{@link com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy HiddenMetaDataLoadingStrategy}</li>
 * </ul>
 * @ibm-api
 * @ibm-module ThemeModel
 */
dojo.declare("com.ibm.mashups.enabler.model.SkinModel", [com.ibm.mashups.enabler.ListModel, com.ibm.mashups.enabler.ResourceLocator], {
    
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.theme.Skin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.theme.Skin"] = true;
dojo.provide("com.ibm.mashups.enabler.theme.Skin");






/**
 * Interface that represents a skin; used in the SkinModel
 * @ibm-api
 * @ibm-module ThemeModel
 */
dojo.declare("com.ibm.mashups.enabler.theme.Skin", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized, com.ibm.mashups.enabler.DefaultLocalized, com.ibm.mashups.enabler.MetaData], {
    /**
     * Returns the base url of the skin
     * @type String
     * @return base url of the skin. can never be <code>null</code>
     * @deprecated Since 2.0. Use ThemeModel/ResourceLocator findResourceURL instead
     */
    getBaseUrl: function(){
    },
    
    /**
     * Returns an image which can be used for preview urls
     * @type String
     * @return url to an image; may be <code>null</code>
     * @deprecated Use getPreviewURL instead.
     */
    getPreviewUrl: function(){
    },
    
    /**
     * Returns an image which can be used for preview urls
     * @return {String} url to an image; may be <code>null</code>
     */
    getPreviewURL: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.SkinImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.SkinImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.SkinImpl");













// skin
dojo.declare("com.ibm.mm.enabler.theme.SkinImpl", [com.ibm.mashups.enabler.theme.Skin, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.DefaultLocalizedXmlImpl, com.ibm.mm.enabler.MetaDataImpl2], {
    constructor: function(node) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        this.xmlData = node;
        // service document and initialization
        var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_THEME);
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM]));
    },
    
    getBaseUrl: function(internal) {
        if (typeof internal == "undefined") {
            var LOG_METHOD = "getBaseUrl";
            this.LOGGER.warning(LOG_METHOD, "Deprecated. Use com.ibm.mashups.enabler.model.ThemeModel.findResourceURL instead.");
        }
        return this._getBaseUrl();
    },
    _getBaseUrl: function() {
        var entry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:content/model:skin/model:link[@ext:rel='baseUrl']", this.xmlData, this.ns);
        return entry ? entry.getAttribute("href") : null;
    },
    
    // @deprecated
    getPreviewUrl: function() {
        return this.getPreviewURL();
    },
    
    getPreviewURL: function() {
        var entry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:content/model:skin/model:link[@ext:rel='previewUrl']", this.xmlData, this.ns);
        return entry ? entry.getAttribute("href") : null;
    },
    
    _getMediaURL: function() {
        var entry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry('atom:link[@ext:rel="media"]', this.xmlData, this.ns);
        return entry ? entry.getAttribute("href") : null;
    },
    
    toString: function() {
        return com.ibm.mm.enabler.utils.Dom.innerXML(this.xmlData);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.SkinModelIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.SkinModelIteratorImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.SkinModelIteratorImpl");








// skin model iterator
dojo.declare("com.ibm.mm.enabler.theme.SkinModelIteratorImpl", [com.ibm.mashups.enabler.Iterator, com.ibm.mm.enabler.DeferredIteratorImpl, com.ibm.mm.enabler.ModelIteratorImpl], {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
    },
    
    _createNode: function(xmlData) {
        var node = new com.ibm.mm.enabler.theme.SkinImpl(xmlData);
        this.model.loadedNodes[node.getID()] = node;
        return node;
    },
    
    _customizeUrl: function(url) {
        // include access control info
        url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
        // full representation
        url.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);
        
        // include hidden meta data
        this.model._applyHiddenMetaDataLoadingStrategyToModelURL(url);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.SkinModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.SkinModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.SkinModelImpl");















// skin model
dojo.declare("com.ibm.mm.enabler.theme.SkinModelImpl", [com.ibm.mashups.enabler.model.SkinModel, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.strategy.HiddenMetaDataLoadingStrategyHelperImpl], {
    constructor: function(url, themeId) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        // service document and initialization
        var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_THEME);
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_OPENSEARCH]));
        
        // loaded nodes
        this.loadedNodes = {};
        
        // skin collection url
        this.collectionUrl = url;
        
        // theme id
        this.themeId = themeId;
    },
    
    iterator: function() {
        return new com.ibm.mm.enabler.theme.SkinModelIteratorImpl(this.collectionUrl, this);
    },
    
    
    /**
     * Attention: This is also used by StaticSkinModelImpl in a static way, e.g. no themeId attached
     *
     * @param {Object} uri
     */
    find: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._load, uri);
    },
    
    /**
     * gets called from StaticSkinModelImpl
     */
    _setThemeId: function(themeId) {
        this.themeId = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(themeId);
    },
    
    /**
     * Attention: This is also used by StaticSkinModelImpl in a static way, e.g. no themeId attached
     *
     * @param {Object} deferred
     * @param {Object} sync
     * @param {Object} uri
     */
    _load: function(deferred, sync, uri) {
    
        uri = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(uri);
        
        // logging support
        var LOG_METHOD = "_load";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        var ret;
        if (uri in this.loadedNodes) {
            // finished callback
            ret = this.loadedNodes[uri];
            if (deferred) {
                deferred.finish(ret, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
        }
        else {
            // request node from server
            var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.THEME_URL, this);
            
            var nodeArray = [{
                value: uri,
                isID: true
            }];
            
            // check if themeId is there - may not be there when being used by StaticSkinModel
            if (this.themeId) {
                nodeArray.push({
                    value: this.themeId,
                    isID: true
                });
            }
            
            myUrl.setNodes(nodeArray);
            
            // include access control info
            myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
            
            // full representation
            myUrl.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);
            
            // apply hidden meta data strategy
            this._applyHiddenMetaDataLoadingStrategyToModelURL(myUrl);
            var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
            serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
                if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                    var entry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:feed/atom:entry", data, this.ns);
                    if (entry) {
                        this.loadedNodes[uri] = new com.ibm.mm.enabler.theme.SkinImpl(entry);
                        ret = this.loadedNodes[uri] || null;
                    }
                }
                // finished/error callback
                if (deferred) {
                    deferred.finish(ret, xhr.status);
                }
            }));
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
        return ret;
    },
    
    // @deprecated
    findResourceUrl: function(node, name) {
        return this.findResourceURL(node, name);
    },
    findResourceURL: function(node, name) {
        var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.THEME_MEDIA_URL, this);
        var nodeJSON = [{
            value: node.getID(),
            isID: true
        }, {
            value: this.themeId,
            isID: true
        }, {
            value: name,
            isID: false
        }];
        myUrl.setNodes(nodeJSON);
        return myUrl.getAbsoluteURL();
    },
    
    invalidate: function() {
        this.loadedNodes = {};
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SkinModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SkinModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SkinModel");







}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.space.SpaceExtensionResult"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.space.SpaceExtensionResult"] = true;
dojo.provide("com.ibm.mashups.enabler.model.space.SpaceExtensionResult");

/**
 * SpaceExtensionResult class.
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.model.space.SpaceExtensionResult", null, {
    STATE_OK: "OK",
    STATE_WARNING: "WARNING",
    STATE_ERROR: "ERROR",

    /**
     * Defining the constructor function of the class.
     * The instance of this class is used to carry the return value of SpaceExtension methods.
     *
     * result: String value, indicate the overall status of result. Must be one of following -
     *                SpaceExtensionResult.STATE_OK
     *                SpaceExtensionResult.STATE_WARNING
     *                SpaceExtensionResult.STATE_ERROR
     *
     * This class may also have following fields
     *     message: String, localized message to show what has happened.
     *     solution: String, localized solution message to suggest what actions can be taken to correct the problem.
     *     optionalInfo: this is a field reserved for future.
     */
    constructor: function(result){
        this.result = result;
    },
    
    /**
     * Returns a String value to indicate the overall state of the SpaceExtensionResult
     */
    getState: function(){
        return this.result;
    },
    
    /**
     * Returns localized message
     * return: String, localized message or empty String if mesage doesn't exist.
     */
    getLocaleMessage: function(){
        return this.message || "";
    },
    
    /**
     * Returns localized solution
     * return: String, localized solution message or empty String if solution doesn't exist.
     */
    getLocaleSolution: function(){
        return this.solution || "";
    },
    
    /**
     * Returns additional information from the result.
     * this is reserved for future.
     */
    getOptionalInfo: function(){
        return this.optionalInfo || "";
    },
    
    /**
     * Sets a localized message to the result.
     * localeMessage: String, localized message to show what has happened.
     */
    setLocaleMessage: function(localeMessage){
        this.message = localeMessage;
    },
    
    /**
     * set a localized solution to the result.
     * localeSolution: String, localized solution message to suggest what actions can be taken to correct the problem.
     */
    setLocaleSolution: function(localeSolution){
        this.solution = localeSolution;
    },
    
    /**
     * set additional information to the result.
     * this is reserved for future.
     */
    setOptionalInfo: function(info){
        if (optionalInfo) {
            this.optionalInfo = info;
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.space.SpaceExtension"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.space.SpaceExtension"] = true;
dojo.provide("com.ibm.mashups.enabler.model.space.SpaceExtension");




 

/**
 *
 * Interface SpaceExtension for space life cycle events.
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.model.space.SpaceExtension", null, {

    /**
     * This method is called before space is created in business space.
     *
     * spaceId: String, Id of the space
     * spaceName: String, name of the space
     * spaceDescription: String, description of the space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means "notification has been delivered successfully and accepted".
     */
    onSpaceCreate: function(spaceId, spaceName, spaceDescription, optionalInfo){
        // alert("SpaceExtension.onSpaceCreate");
    },
    
    /**
     * This method is called before space is copied in business space.
     *
     * sourceSpaceId: String, Id of the source space
     * targetSpaceId: String, Id of the target space
     * targetSpaceName: String, name of the target space
     * targetDescription: String, description of the target space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means "notification has been delivered successfully and accepted".
     */
    onSpaceCopy: function(sourceSpaceId, targetSpaceId, targetSpaceName, targetDescription, optionalInfo){
        // alert("SpaceExtension.onSpaceCopy");
    },
    
    /**
     * This method is called before space is updated in business space.
     *
     * spaceId: String, Id of the space
     * spaceData: JSON object, the set of space data that's going to be updated.
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means "notification has been delivered successfully and accepted".
     */
    onSpaceUpdate: function(spaceId, spaceData, optionalInfo){
        // alert("SpaceExtension.onSpaceUpdate");
    },
    
    /**
     * This method is called before space is deleted in business space.
     *
     * spaceId: String, Id of the space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means "notification has been delivered successfully and accepted".
     */
    onSpaceDelete: function(spaceId, optionalInfo){
        // alert("SpaceExtension.onSpaceDelete");
    },
    
    /**
     * This method is used to check if the specified space can be exported.
     *
     * spaceId: String, Id of the space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means that the space can be exported.
     */
    canExportSpace: function(spaceId, optionalInfo){
        // alert("SpaceExtension.canExportSpace");
    },
    
    /**
     * This method is used to check if the specified space can be copied.
     *
     * spaceId: String, Id of the space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means that the space can be copied.
     */
    canCopySpace: function(spaceId, optionalInfo){
        // alert("SpaceExtension.canCopySpace");
    },
    
    /**
     * This method is used to check if the specified space can be deleted.
     *
     * spaceId: String, Id of the space
     * optionalInfo: this is a field reserved for future.
     *
     * return: SpaceExtensionResult or no return.
     *         No return means that the space can be deleted.
     */
    canDeleteSpace: function(spaceId, optionalInfo){
        // alert("SpaceExtension.prototype.canDeleteSpace");
    }    
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.SpaceExtensionLocator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.SpaceExtensionLocator"] = true;
dojo.provide("com.ibm.mashups.enabler.space.SpaceExtensionLocator");




/**
 * This locator interface allows searching on the space extension model
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.space.SpaceExtensionLocator", [com.ibm.mashups.enabler.Locator],     {
    /**
     * Returns the corresponding space extension for the given space node, or null if none exists
     * @param {com.ibm.mashups.enabler.space.SpaceNode} spaceNode the SpaceNode object to find the extension for. Must not be <code>null</code>.
     * @return {com.ibm.mashups.enabler.Deferred} a deferred object used to start this operation. 
     * The return value when executed through the deferred object is
     * a space extension for the given space node or <code>null</code> if there no appropriate space extension available.
     * @see SpaceExtension
     */
    findExtensionBySpace: function (spaceNode) {
    }	
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SpaceExtensionModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SpaceExtensionModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SpaceExtensionModel_API");
dojo.provide("com.ibm.mashups.enabler.model.SpaceExtensionModel");

/**
 * Interface for a space extension model.
 * 
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.model.SpaceExtensionModel", [
		com.ibm.mashups.enabler.ListModelController,
		com.ibm.mashups.enabler.space.SpaceExtensionLocator ], {

	/**
	 * Creates a new Space extension. The created node can be inserted into the model 
	 * using an appropriate <code>insert</code> method. The node will not appear 
	 * in the model unless it is inserted.
	 * @param {JSON} context array of predefined information 
	 * used for the creation of the node. Should be <code>null</code>.
	 * @type com.ibm.mashups.enabler.model.space.SpaceExtension 
	 * @return created space extension
	 */
	create : function(context) {
		return new SpaceExtension();
	}
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.SpaceLocator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.SpaceLocator"] = true;
dojo.provide("com.ibm.mashups.enabler.space.SpaceLocator");




// we must not include NavigationNode here as it pulls in too many dependencies
//

/**
 * This locator interface allows searching on the space model
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.space.SpaceLocator", [com.ibm.mashups.enabler.Locator], {
    /**
     * Returns a Deferred object to obtain the space node, which contains the given navigation node
     * @param {com.ibm.mashups.enabler.navigation.NavigationNode} navigationNode the NavigationNode object to find the space for. Must not be <code>null</code>.
     * @return {com.ibm.mashups.enabler.Deferred} a deferred object used to start this operation, never <code>null</code>.
     * The return value when executed through the deferred object is
     * a space node for the given navigation node or <code>null</code> if no space contains this page
     * @see com.ibm.mashups.enabler.space.SpaceNode
     */
    findSpaceByNavigation: function(navigationNode){
    },
    
    /**
     * Returns an iterator containing all favorite spaces of the list.
     *
     * @return {com.ibm.mashups.enabler.DeferredIterator} a deferred iterator, never <code>null</code>
     */
    findFavorites: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.SpaceNode_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.SpaceNode_API"] = true;
dojo.provide("com.ibm.mashups.enabler.space.SpaceNode_API");
dojo.provide("com.ibm.mashups.enabler.space.SpaceNode");

/**
 * Interface representing a space node.
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.space.SpaceNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.ModifiableLocalized, com.ibm.mashups.enabler.DefaultLocalized, com.ibm.mashups.enabler.ModifiableMetaData, com.ibm.mashups.enabler.ThemeProvider], {
    /**
     * Returns the root navigation ID, or <code>null</code> if no navigation has been
     * associated with the node.
     * @type com.ibm.mashups.enabler.Identifiable
     * @return the root navigation ID, or <code>null</code> if it has not been associated
     */
    getRootNavigation: function(){
    },
    
    /**
     * Returns a URL to export the space in zip file format
     * @type String
     * @return link to export the space
     */
    getExportSpaceURL: function(){
    },
    
    /**
     * Returns a URL to export the space as a template
     * in zip file format
     * @type String
     * @return link to export the space as a template
     */
    getExportTemplateURL: function(){
    },
    
    /**
     * Save the space as template.
     * @type void
     * @return template object.
     */
    saveAsTemplate: function(){
    },
    
    /**
     * Returns whether this space node is flagged as hidden. It is the responsibility of the
     * UI to render the space nodes correctly and honor this flag.
     * @type Boolean
     * @return true if the space is flagged as hidden, otherwise false
     */
    isHidden: function(){
    },
    
    /**
     * Allows to flag the space node as hidden or visible.
     * @param {Boolean} hide true to hide the space, otherwise false
     * @type void
     */
    setHidden: function(hide){
    },
    /**
     * Confirms whether setting the hidden flag is possible.
     * @param {Boolean} hide true to hide the space, otherwise false
     * @return {Boolean} true if the value can be set, otherwise false.
     */
    confirmSetHidden: function(hide){
    },
    
    /**
     * Returns whether this space node is flagged as a favorite space.
     * @type Boolean
     * @return true if the space is flagged as favorite, otherwise false
     */
    isFavorite: function(){
    },
    
    /**
     * Allows to mark the space node as favorite ir non-favorite.
     * @param {Boolean} favorite true to mark this space as favorite, otherwise false
     * @type void
     */
    setFavorite: function(favorite){
    }
});


}

if(!dojo._hasResource["com.ibm.mm.enabler.ac.OwnerAspect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.ac.OwnerAspect"] = true;
dojo.provide("com.ibm.mm.enabler.ac.OwnerAspect");

dojo.declare("com.ibm.mm.enabler.ac.OwnerAspect", null, {
    /**
     * Returns a user object of the owner
     * @type {com.ibm.mashups.mode.user.User}
     * @return the owner
     */
    getOwner: function(){
    },
    /**
     * Returns a Deferred object to obtain the owner.
     * @type com.ibm.mashups.enabler.Deferred(Object)
     * @return a deferred object used to start this operation. The return value
     * when executed through the deferred object is the owner
     * @ibm-api
     */
    getOwnerDeferred: function(){
    },
    /**
     * Sets the new owner
     * @param {com.ibm.mashups.mode.user.User} newOwner the new owner
     */
    setOwner: function(newOwner){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.SpaceExtensionAspect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.SpaceExtensionAspect"] = true;
dojo.provide("com.ibm.mm.enabler.space.SpaceExtensionAspect");

dojo.declare("com.ibm.mm.enabler.space.SpaceExtensionAspect", null, {
    /**
     * Returns the space extension url belonging to this space
     * @type {String}
     * @return the extension url
     */
    getSpaceExtensionURL: function() {
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.SpaceNodeImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.SpaceNodeImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.SpaceNodeImpl");
















dojo.declare("com.ibm.mm.enabler.space.SpaceNodeImpl", [com.ibm.mashups.enabler.space.SpaceNode, com.ibm.mm.enabler.persistence.PersistenceHandlerImpl], {
    constructor: function(){
        // init delegation
        var e = com.ibm.mashups.enabler;
        this._initDelegators(e.Identifiable.prototype);
        this._initDelegators(e.Localized.prototype);
        this._initDelegators(e.DefaultLocalized.prototype);
        this._initDelegators(e.MetaData.prototype);
        this._initDelegators(e.ModifiableLocalized.prototype);
        this._initDelegators(e.ModifiableMetaData.prototype);
        this._initDelegators(e.Transformable.prototype);
        this._initDelegators(e.EditLinkProvider.prototype);
        this._initDelegators(e.DirtyFlagProvider.prototype);
        this._initDelegators(e.space.SpaceNode.prototype);
        this._initDelegators(e.ac.EffectiveAccess.prototype);
        
        // internal APIs
        var ie = com.ibm.mm.enabler;
        this._initDelegators(ie.ac.OwnerAspect.prototype);
        this._initDelegators(ie.space.SpaceExtensionAspect.prototype);
        
        // init transition
        var sm = com.ibm.mashups.enabler.space.Factory.getSpaceModel();
        this._initTransition(sm, sm.getModifiableNode);
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.SpaceNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.SpaceNode"] = true;
dojo.provide("com.ibm.mashups.enabler.space.SpaceNode");









}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SpaceModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SpaceModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SpaceModel_API");
dojo.provide("com.ibm.mashups.enabler.model.SpaceModel");

/**
 * Interface for a space model.<br>
 * <br>
 * The space model supports the following strategies:
 * <ul>
 * <li>{@link com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy ListLoadAheadStrategy}</li>
 * <li>{@link com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy HiddenMetaDataLoadingStrategy}</li>
 * </ul>
 * 
 * @ibm-api
 * @ibm-module SpaceModel
 */
dojo.declare("com.ibm.mashups.enabler.model.SpaceModel", [
		com.ibm.mashups.enabler.ListModelController,
        com.ibm.mashups.enabler.Invalidatable,
		com.ibm.mashups.enabler.SubmittableFormProvider,
		com.ibm.mashups.enabler.space.SpaceLocator,
        com.ibm.mashups.enabler.DirtyFlagProvider ], {
	/**
	 * creates a new Space node. The created node can be inserted into the model
	 * using an appropriate <code>insert</code> method. The node will not appear
	 * in the model unless it is inserted.
	 * @param {JSON} context array of predefined information
	 * used for the creation of the node. Should be <code>null</code>.
	 * @type com.ibm.mashups.enabler.space.SpaceNode
	 * @return created space node
	 */
	create : function(context) {
	},

    /**
    * Commits the modifications applied to this model and all dependant models<br>
    * @return {DeferredOperation} a deferred object used to start this operation. 
    * The return value when executed through the deferred object is <code>null</null>
    */
	commit : function() {
	},

	/**
	 * Confirms whether exporting the node is possible.
	 * @param{com.ibm.mashups.enabler.space.SpaceNode} space space or space uri (without any scope).
	 *                      Must not be <code>null</code>
	 * @return {Boolean} true if the node can be exported, otherwise false.
	 */
	confirmExport : function(space) {
	},

	/**
	 * Confirms whether copying the node is possible.
	 * @param{com.ibm.mashups.enabler.space.SpaceNode} space space or space uri (without any scope).
	 *                      Must not be <code>null</code>
	 * @return {Boolean} true if the node can be copied, otherwise false.
	 */
	confirmCopy : function(space) {
	},

	/**
	 * Returns a SubmittableForm object associated with the given id to import a space.
	 * Same as <code>getSubmittableForm</code>
	 * @param {String} id the id of the html form this object is associated with
	 * @return {com.ibm.mashups.enabler.SubmittableForm} SubmittableForm object, maybe <code>null</code>
	 */
	getSubmittableImportForm : function(id) {
	}
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl");

/*
 * This is injected on the fly and not fetched through an API. Therefore we need to define the @ibm-module so that the build process is picking it up
 * @ibm-module SpaceModel
 */










dojo.declare("com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl", [com.ibm.mashups.enabler.ListModel, com.ibm.mm.enabler.model.ModelImpl], {

    constructor: function() {
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.modelMessages = dojo.i18n.getLocalization("com.ibm.mm.enabler", "modelMessages");
    },
    
    // singleton
    _getInstance: function() {
        var instance = com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl._instance = new com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl());
    },
    find: function(identifiable) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, identifiable);
    },
    _find: function(deferred, sync, identifiable) {
        var LOG_METHOD = "_find(deferred, sync, id)";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync, identifiable]);
        }
        var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(identifiable);
        var jp = com.ibm.mm.enabler.persistence.JsonPersistenceProvider;
        var retVal = null;
        jp.findSpace(id, dojo.hitch(this, function(result, status, parameters) {
            if (result) {
                var f = com.ibm.mm.enabler.space.persistence.json.Factory;
                retVal = f.createSpaceNodePersistence(result);
            }
            deferred.finish(retVal, status);
        }), sync, this);
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
        return retVal;
    },
    
    invalidate: function(node){
        var jp = com.ibm.mm.enabler.persistence.JsonPersistenceProvider;
        
        if (node) {
            var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(node);
            jp.invalidateSpace(id);
        }
        else {
            jp.invalidateSpace();
        }
    },
    invalidateFavorites: function(node) {
        var jp = com.ibm.mm.enabler.persistence.JsonPersistenceProvider;
        jp.invalidateFavorites();
    },
    _getFavoriteIteratorPersistenceHandler: function() {
        var that = this;
        return {
            // result is a json object
            // nodes: the nodes preloaded
            // size: the size of the iterator
            // loadFurtherNodes: defines whether we are at the end of the iterator or not
            loadAhead: function(context) {
                return new com.ibm.mm.enabler.DeferredImpl(that, function(deferred, sync, node){
                    // logging support
                    var LOG_METHOD = "loadAhead";
                    var bIsLoggable = this.LOGGER.isLoggable(that.LOG_LEVEL);
                    if (bIsLoggable) {
                        this.LOGGER.entering(LOG_METHOD, arguments);
                    }
                    var retVal = null;
                    var jp = com.ibm.mm.enabler.persistence.JsonPersistenceProvider;
                    var f = com.ibm.mm.enabler.space.persistence.json.Factory;
                    jp.getFavoriteSpaces(dojo.hitch(this, function(result, status, parameters) {
                        if ((result) && (dojo.isArray(result))) {
                            retVal = {
                                nodes: [],
                                loadFurtherNodes: false
                            };
                            for (var i=0, l=result.length; i<l; ++i) {
                                retVal.nodes.push(f.createSpaceNodePersistence(result[i]));
                            }
                            retVal.size = retVal.nodes.length;
                            if (deferred) {
                                deferred.finish(retVal, status);
                            }
                        }
                        else {
                            if (deferred) {
                                deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_BAD_REQUEST);
                            }
                        }
                    }), sync, this);
                    // exit trace
                    if (bIsLoggable) {
                        this.LOGGER.exiting(LOG_METHOD, retVal);
                    }
                    return retVal;
                });
            },
            getLoadAheadContext: function(context, result) {
                return context;
            },
            setCursorPosition: function(context, iterator) {
                return context;
            }
        };
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SpaceModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SpaceModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SpaceModel");










}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.Factory_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.Factory_API"] = true;
dojo.provide("com.ibm.mashups.enabler.space.Factory_API");
dojo.provide("com.ibm.mashups.enabler.space.Factory");

/**
 * Space model factory.
 * 
 * Please note that the model factory creates only one instance of the model and
 * returns this instance every time the respective get-method is called.
 * 
 * @ibm-api
 * @ibm-module SpaceModel
 * 
 * @since 2.4
 */
dojo.declare("com.ibm.mashups.enabler.space.Factory", null, {
	/**
	 * Returns the space model.
	 *
	 * @return {com.ibm.mashups.enabler.model.SpaceModel} space model, never <code>null</code>
	 */
	getSpaceModel : function() {
	},

	/**
	 * Returns the space extension model.
	 *
	 * @return {com.ibm.mashups.enabler.model.SpaceExtensionModel} space extension model, never <code>null</code>
	 */
	getSpaceExtensionModel : function() {
	}
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.persistence.FactoryImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.persistence.FactoryImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.persistence.FactoryImpl");



dojo.declare("com.ibm.mm.enabler.space.persistence.FactoryImpl", null, {
    createSpaceNode: function(p){
        var w = new com.ibm.mm.enabler.space.SpaceNodeImpl();
        w._setPersistence(p);
        return w;
    }
});

// factory
com.ibm.mm.enabler.space.persistence.Factory = new com.ibm.mm.enabler.space.persistence.FactoryImpl();

}

if(!dojo._hasResource["com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl"] = true;
dojo.provide("com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl");






//dojo.require("com.ibm.mm.enabler.persistence.IteratorPersistenceProvider");
//dojo.require("com.ibm.mm.enabler.persistence.IteratorModelProvider");

// this is an implementation of an unordered iterator
dojo.declare("com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl", [com.ibm.mm.enabler.DeferredIteratorImpl], {
    constructor: function(persistenceProvider, modelProvider) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        // the persistence provider this iterator is connected to.
        // must implement IteratorPersistenceProvider
        this.persistenceProvider = persistenceProvider;
        
        // the iterator's model
        this.modelProvider = modelProvider;
        this.deletedNodes = this.modelProvider.getDeletedNodes();
        
        // entries in feed order; undefined entries denote potential 
        // nodes to load; null entry marks end of node collection 
        this.entries = [];
        // stores the loaded entries of the iterator so that they are not loaded again
        this.entryCache = {};
        // context being used for loadahead
        this.context = null;
        // cursor position
        this.cursor = 0;
        // indicates whether to load more nodes with load ahead
        this.loadFurtherNodes = true;
        // number of nodes
        this.sizeValue = null;
        // number of created nodes to include in this iterator
        this.numCreatedNodes = 0;
        
        // compile key-value pairs for each created node included in this 
        // iterator: (createdId, nextId)
        this._createdNodes = {};
        com.ibm.mm.enabler.utils.Misc.forIn(this.modelProvider.getCreatedNodes(), function(node, id) {
            this._createdNodes[id] = node;
            this.numCreatedNodes++;
        }, this);
        
        // number of removed nodes not to include in this iterator
        this.numRemovedNodes = 0;
        // if the url is not present, removed nodes do not count towards size anymore
        com.ibm.mm.enabler.utils.Misc.forIn(this.deletedNodes, function() {
            this.numRemovedNodes++;
        }, this);
    },
    
    hasNext: function(dfr, sync) {
        if (typeof this.entries[this.cursor] == "undefined") {
            var insertionIndex = 0;
            if (this.loadFurtherNodes) {
                // load nodes from server
                this._loadAheadInternal(dfr || null, true);
                insertionIndex = this.cursor;
            }
            else {
                // do not load entries
                this.entries[0] = null;
            }
        }
        
        var candidateId = this.entries[this.cursor];
        var hasNext = candidateId ? true : false;
        if (hasNext && this.modelProvider.lookupNodeInCacheByID(candidateId)===null) {
            // do not expose nodes, which are deleted at runtime
            this.cursor++;
            return this.hasNext(dfr, sync);
        }
        return hasNext;
    },
    
    next: function(dfr, sync) {
        if (this.hasNext(dfr || null, true)) {
            var entryId = this.entries[this.cursor++];
            var lookup = this.modelProvider.lookupNodeInCacheByID(entryId);
            return (typeof lookup == "undefined") ? this.entryCache[entryId] : lookup;
        }
        return null;
    },
    
    _insertCreated: function(entries, cursor) {
        for (var createdId in this._createdNodes) { // JSLINT-IGNORE: no prototype properties in here
            entries.splice(cursor, 0, createdId);
            cursor++;
        }
    },
    
    setCursorPosition: function(position) {
        if (position<0) {
            return;
        }
        if (!this.loadFurtherNodes) {
            if (position>=this.size().start()) {
                return;
            }
        }
        this.cursor = position;
        this.persistenceProvider.setCursorPosition(this.context, this);
    },
    
    getCursorPosition: function() {
        return this.cursor;
    },
    
    size: function() {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._size);
    },
    
    _size: function(dfr, sync) {
        if (!this.sizeValue) {
            this.hasNext(null, true);
        }
        
        var size = this.sizeValue + this.numCreatedNodes - this.numRemovedNodes;
        if (size<0) {
            size = 0;
        }
        // finished callback 
        if (dfr) {
            dfr.finish(size, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
        }
        // reflect number of created/removed nodes in size value
        return size;
    },
    
    start: function(sync) {
        this._loadAheadInternal(this, sync, true);
    },
    
    _loadAheadInternal: function(deferred, sync, iterate) {
        // logging support
        var LOG_METHOD = "_loadAhead";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        
        if (!this.context) {
            this.context = this.persistenceProvider.getLoadAheadContext(null, this);
        }
        
        var retVal;
        var dfr = this.persistenceProvider.loadAhead(this.context);
        dfr.setFinishedCallback(dojo.hitch(this, function(result, status){
            // result is a json object
            // nodes: the nodes preloaded
            // size: the size of the iterator
            // loadFurtherNodes
            if (result == null) {
                // error
                this.loadFurtherNodes = false;
                this.sizeValue = 0;
                this.entries[this.cursor] = null;
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_BAD_REQUEST);
                }
                return;
            }

            this.loadFurtherNodes = result.loadFurtherNodes;
            this.sizeValue = result.size;

            // handle empty iterator
            if (result.size == 0) {
                this.entries[this.cursor] = null;
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
                }
                return;
            }
            
            // iterate entries
            var c = 0;
            if (result.nodes) {
                for (var i = 0, l = result.nodes.length; i < l; i++) {
                    // instantiate node
                    var node = result.nodes[i];
                    var nodeid = node.getID();
                    
                    // see if any of the nodes are deleted, then we skip them
                    if (this.deletedNodes[nodeid]) {
                        continue;
                    }
                    
                    // put id in entry list
                    this.entries[this.cursor + c++] = node.getID();
                    this.entryCache[nodeid] = node;
                    
                    // contribute node to model cache
                    this.modelProvider.contributeNodeToCache(node);
                    
                    // callback handling
                    if (deferred) {
                        deferred.nextFinish(node);
                    }
                }
            }
            
            if (this.loadFurtherNodes) {
                if (result.nodes && iterate) {
                    this.context = this.persistenceProvider.getLoadAheadContext(this.context, this);
                    // was last next element
                    this._loadAheadInternal(deferred, sync, iterate);
                }
            }
            else {
                // mark end of collection
                this.entries[this.cursor + c] = null;

                // inject nodes, which are created at runtime
                this._insertCreated(this.entries, this.cursor + c);
                
                // finished callback (end of collection)
                if (deferred) {
                    deferred.finish(null, status);
                }
            }
        }));
        dfr.start(sync);
        
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.SpaceModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.SpaceModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.SpaceModelImpl");























dojo.declare("com.ibm.mm.enabler.space.SpaceModelImpl", [com.ibm.mashups.enabler.model.SpaceModel, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.ConfirmableModelImpl, com.ibm.mm.enabler.DirtyFlagProviderImpl, com.ibm.mm.enabler.SubmittableFormProviderImpl, com.ibm.mm.enabler.LazyLoadSupport], {

    constructor: function() {
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        var LOG_METHOD = "constructor()";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD);
        }
        this.modelMessages = dojo.i18n.getLocalization("com.ibm.mm.enabler", "modelMessages");
        this.enforcementHelper = com.ibm.mm.enabler.utils.APIEnforcementHelper;

        this.spaceExtensionModel = com.ibm.mashups.enabler.space.Factory.getSpaceExtensionModel();

        this.loadedNodes = {};
        this.createdNodes = {};
        this.deletedNodes = {};
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },

    // CANDIDATE FOR BASE CLASS
    _getPersistenceModel: function(){
        var m = this._getJsonModel();
        return m ? m : this._getXmlModel();
    },
    
    // CANDIDATE FOR BASE CLASS
    _hasXmlModel: function(){
        return !!this._xmlModel;
    },
    
    _getXmlModel: function(){
        if (!this._xmlModel) {
            this.lazyLoadModules(["SpaceModelXml"]);
            dojo["require"]("com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl"); // JSLINT-IGNORE: have dojo.require only at runtime
            this._xmlModel = com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl.prototype._getInstance();
            // transfer strategies to xml model
            this._xmlModel.setStrategy(this.getStrategies());
        }
        return this._xmlModel;
    },
    
    // CANDIDATE FOR BASE CLASS
    _hasJsonModel: function(){
        return !!this._jsonModel;
    },
    
    _getJsonModel: function(){
        if ((ibmConfig) && (ibmConfig.json) && (ibmConfig.json["resource-service"])) {
            if (!this._jsonModel) {
                dojo["require"]("com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl"); // JSLINT-IGNORE: have dojo.require only at runtime
                this._jsonModel = com.ibm.mm.enabler.space.persistence.json.SpaceModelJsonImpl.prototype._getInstance();
                // transfer strategies to json model
                this._jsonModel.setStrategy(this.getStrategies());
            }
            return this._jsonModel;
        }
        return null;
    },

    // singleton
    _getInstance: function() {
        var instance = com.ibm.mm.enabler.space.SpaceModelImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.space.SpaceModelImpl._instance = new com.ibm.mm.enabler.space.SpaceModelImpl());
    },
    
    // resolves any id or node to its model representation, i.e.
    // CANDIDATE FOR BASE CLASS
    // * returns a wrapper
    // * returns null if the node is deleted in model scope 
    _resolve: function(node){
        if (node == null) {
            return null;
        }
        var id = dojo.isString(node) ? node : node.getID();
        return (id in this.deletedNodes) ? null : this.find(id).start();
    },
    
    // CANDIDATE FOR BASE CLASS
    // wraps any input nodes if necessary, i.e.
    // * creates wrapper for persistence nodes
    // * replaces persistence for cached nodes
    // * caches resolved nodes
    _wrap: function(node){
        var result = null;
        if (node) {
            if (node.addDirtyCallback) {
                // we only add this on nodes that implement the method.
                // It should not be added to the JSON Nodes for instance
                node.removeDirtyCallback(this._setModelDirty);
                node.addDirtyCallback(this, this._setModelDirty);
            }
            var id = node.getID();
            // handle wrapper; make sure a wrapped object is returned
            var isWrapper = com.ibm.mm.enabler.utils.Misc.isInstanceOf(node, com.ibm.mm.enabler.persistence.PersistenceHandlerImpl);
            if (id in this.loadedNodes) {
                if (isWrapper) {
                    // wrapper is cached; input node is wrapper
                    if (this.loadedNodes[id] !== node) {
                        // if wrapper NOT identical -> replace persistence
                        this.loadedNodes[id]._setPersistence(node._getPersistence());
                    }
                }
                else {
                    // wrapper is cached; input node is persistence -> replace persistence
                    if (!this.isModifiableNode(this.loadedNodes[id])) {
                        this.loadedNodes[id]._setPersistence(node);
                    }
                }
            }
            else {
                if (isWrapper) {
                    // wrapper is NOT cached; input node is wrapper
                    this.loadedNodes[id] = node;
                }
                else {
                    // wrapper is NOT cached; input node is persistence
                    this.loadedNodes[id] = new com.ibm.mm.enabler.space.persistence.Factory.createSpaceNode(node);
                }
            }
            result = this.loadedNodes[id];
        }
        return result;
    },
    
    // CANDIDATE FOR BASE CLASS
    // returns persistence    
    _unwrap: function(node){
        return (node && dojo.isFunction(node._getPersistence)) ? node._getPersistence() : node;
    },
    
    _setModelDirty: function(){
        this.setDirty();
    },
    
    invalidate: function(nodeId) {
        if (nodeId) {
            delete this.loadedNodes[nodeId];
            delete this.createdNodes[nodeId];
            delete this.deletedNodes[nodeId];
        }
        else {
            // clear all
            for (var id in this.loadedNodes) { // JSLINT-IGNORE: no prototype properties in here
                delete this.loadedNodes[id];
            }
            for (var id in this.createdNodes) { // JSLINT-IGNORE: no prototype properties in here
                delete this.createdNodes[id];
            }
            for (var id in this.deletedNodes) { // JSLINT-IGNORE: no prototype properties in here
                delete this.deletedNodes[id];
            }
            this.loadedNodes = {};
            this.createdNodes = {};
            this.deletedNodes = {};
            
            this.setClean();
        }
        
        // remove node from persistence cache
        if (this._hasJsonModel()) {
            // no unwrap needed as we are going to provide this as "identifiable" only
            this._getJsonModel().invalidate(nodeId);
        }
    },

    find: function(uri){
        this.enforcementHelper.enforceDefinedNotNull(uri, this.modelMessages.E_PARAM_ISNULL_OR_UNDEFINED_3, ["id", this.declaredClass, "find(id)"]);
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, {
            "id": uri,
            "ignoreDeletedNodes": false
        });
    },
    
    _find:function(deferred, sync, params){
        // logging support
        var LOG_METHOD = "find";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        var uri = params.id;
        var ignoreDeletedNodes = params.ignoreDeletedNodes;
        
        if (!uri) {
            throw new Error(this.modelMessages.E_NODE_ISNULL_0);
        }
        var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(uri);
        
        var retVal = null;
        if (!ignoreDeletedNodes) {
            if (this.deletedNodes[id]) {
                // do not expose nodes that are deleted in the model scope
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
                }
                retVal = null;
                
                // exit trace
                if (bIsLoggable) {
                    this.LOGGER.exiting(LOG_METHOD, retVal);
                }
                // compatibility: only meaningful in sync mode
                return retVal;
            }
        }
        
        if (this.loadedNodes[id]) {
            // fetch node from cache
            if (deferred) {
                deferred.finish(this.loadedNodes[id], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
            retVal = this.loadedNodes[id];
            
            // exit trace
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD, retVal);
            }
            // compatibility: only meaningful in sync mode
            return retVal;
        }
        
        var dfr = this._getPersistenceModel().find(id);
        dfr.setFinishedCallback(dojo.hitch(this, function(node, status){
            retVal = this._wrap(node);
            deferred.finish(retVal, status);
        }));
        dfr.start(sync);
        
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, retVal);
        }
        // compatibility: only meaningful in sync mode
        return retVal;
    },
    
    isModifiableNode: function(node){
        return !!this._unwrap(node).xmlData;
    },
    
    getModifiableNode: function(node, ignoreDeleted){
        this.enforcementHelper.enforceDefinedNotNull(node, this.modelMessages.E_PARAM_ISNULL_OR_UNDEFINED_3, ["node", this.declaredClass, "getModifiableNode(node)"]);
        return new com.ibm.mm.enabler.DeferredImpl(this, function(deferred, sync, node){
            // logging support
            var LOG_METHOD = "getModifiable";
            var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
            if (bIsLoggable) {
                this.LOGGER.entering(LOG_METHOD, arguments);
            }
            
            var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(node);
            var retVal = null;
            if ((this.deletedNodes[id]) && (!ignoreDeleted)) {
                // do not expose nodes that are deleted in the model scope
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
                }
                retVal = null;
            }
            else {
                if (!this.loadedNodes[id] || !this.isModifiableNode(this.loadedNodes[id])) {
                    var dfr = this._getXmlModel().find(id);
                    dfr.setFinishedCallback(dojo.hitch(this, function(node, status){
                        if (node) {
                            retVal = this._wrap(node);
                        }
                        deferred.finish(retVal, status);
                    }));
                    dfr.start(sync);
                }
                else {
                    // serve from cache
                    if (deferred) {
                        deferred.finish(this.loadedNodes[id], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
                    }
                    retVal = this.loadedNodes[id];
                }
            }
            
            // exit trace
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD, retVal);
            }
            return retVal;
        }, node);
    },
    
    findSpaceByNavigation: function(navNode) {
        // we don't need to wrap the return node as the find call under the covers goes back to the _find method of this model
        // logging support
        var LOG_METHOD = "findSpaceByNavigation";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }

        if (typeof(navNode) == "string") {
            return new com.ibm.mm.enabler.DeferredImpl(this, function(deferred, sync) {
                // public callback handling
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
                }
            
                if (bIsLoggable) {
                    this.LOGGER.exiting(LOG_METHOD);
                }
                return null;
            });
        }
        var spaceIdentifier = navNode.getSpaceIdentifier();
        if (!spaceIdentifier) {
            return new com.ibm.mm.enabler.DeferredImpl(this, function(deferred, sync) {
                // public callback handling
                if (deferred) {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
                }
                
                if (bIsLoggable) {
                    this.LOGGER.exiting(LOG_METHOD);
                }
                return null;
            });
        }
        
        return this.find(spaceIdentifier);
    },
    
    findFavorites: function() {
        var that = this;
        var persistenceProvider = this._getPersistenceModel()._getFavoriteIteratorPersistenceHandler();
        return new com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl(
            // Persistence Provider
            {
                loadAhead: function(context) {
                    return new com.ibm.mm.enabler.DeferredImpl(that, function(dfr, sync){
                        var deferred = persistenceProvider.loadAhead(context);
                        deferred.setFinishedCallback(dojo.hitch(that, function(result, status){
                            if (result) {
                                var tempArray = [];
                                // no we wrap the nodes
                                for(var i=0; i<result.nodes.length; i++) {
                                    tempArray.push(this._wrap(result.nodes[i]));
                                }
                                result.nodes = tempArray;
                            }
                            dfr.finish(result, status);
                        }));
                        deferred.start(sync);
                    });
                },
                getLoadAheadContext: function(context, iterator) {
                    return persistenceProvider.getLoadAheadContext(context, iterator);
                },
                setCursorPosition: function(context, iterator){
                    return persistenceProvider.setCursorPosition(context, iterator);
                }
                
            },
            // Model Provider
            {
                getCreatedNodes: function() {
                    return that._getCreatedFavorites();
                },
                getDeletedNodes: function() {
                    return that._getDeletedFavorites();
                },
                lookupNodeInCacheByID: function(id) {
                    if (that.deletedNodes[id]) {
                        return null;
                    }
                    else if (that.isModifiableNode(that.loadedNodes[id])) {
                        return that.loadedNodes[id];
                    }
                    // else return undefined
                },
                contributeNodeToCache: function(node){
                    // only contribute node to cache if no other representation is cached
                    if (!that.loadedNodes[node.getID()]) {
                        that._wrap(node);
                    }
                }
            }
        );
    },
    
    iterator: function() {
        var that = this;
        var persistenceProvider = this._getXmlModel()._getIteratorPersistenceHandler();
        return new com.ibm.mm.enabler.ModelIteratorUnorderedBaseImpl(
            // Persistence Provider
            {
                loadAhead: function(context) {
                    return new com.ibm.mm.enabler.DeferredImpl(that, function(dfr, sync){
                        var deferred = persistenceProvider.loadAhead(context);
                        deferred.setFinishedCallback(dojo.hitch(that, function(result, status){
                            if (result) {
                                var tempArray = [];
                                // no we wrap the nodes
                                for(var i=0; i<result.nodes.length; i++) {
                                    tempArray.push(this._wrap(result.nodes[i]));
                                }
                                result.nodes = tempArray;
                            }
                            dfr.finish(result, status);
                        }));
                        deferred.start(sync);
                    });
                },
                getLoadAheadContext: function(context, iterator) {
                    return persistenceProvider.getLoadAheadContext(context, iterator);
                },
                setCursorPosition: function(context, iterator){
                    return persistenceProvider.setCursorPosition(context, iterator);
                }
            },
            // Model Provider
            {
                getCreatedNodes: function() {
                    return that.createdNodes;
                },
                getDeletedNodes: function() {
                    return that.deletedNodes;
                },
                lookupNodeInCacheByID: function(id) {
                    if (that.deletedNodes[id]) {
                        return null;
                    }
                    else if (that.isModifiableNode(that.loadedNodes[id])) {
                        return that.loadedNodes[id];
                    }
                    // else return undefined
                },
                contributeNodeToCache: function(node){
                    // only contribute node to cache if no other representation is cached
                    if (!that.loadedNodes[node.getID()]) {
                        that._wrap(node);
                    }
                }
            }
        );
    },
    
    create: function(context) {
        return new com.ibm.mm.enabler.space.persistence.Factory.createSpaceNode(this._getXmlModel().create(context));
    },
    confirmCreate: function(context) {
        return this._getXmlModel().confirmCreate(context);
    },
    
    insert: function(node, nextNode) {
        var id = node.getID();
        this.createdNodes[id] = node;
    },
    confirmInsert: function(node, nextNode) {
        return this.getConfirmInsert();
    },
    
    commit: function(){
        var invalidateFavorites = false;
        return new com.ibm.mm.enabler.DeferredOperationImpl(this, function(deferred, sync){
            // logging support
            var LOG_METHOD = "commit";
            var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
            if (bIsLoggable) {
                this.LOGGER.entering(LOG_METHOD, arguments);
            }
            
            var that = this;
            var createdFavorites = this._getCreatedFavorites();
            var deletedFavorites = this._getDeletedFavorites();
            var dfr = this._getXmlModel().commit({
                getNodes: function(){
                    // provide created nodes
                    var createdNodes = {};
                    com.ibm.mm.enabler.utils.Misc.forIn(that.createdNodes, function(node, id){
                        createdNodes[id] = that._unwrap(node);
                    });
                    
                    // provide deleted nodes
                    var deletedNodes = {};
                    com.ibm.mm.enabler.utils.Misc.forIn(that.deletedNodes, function(node, id){
                        deletedNodes[id] = that._unwrap(dojo.hitch(that, that.getModifiableNode)(node,true).start());
                    });
                    
                    // provide loaded nodes
                    var loadedNodes = {};
                    com.ibm.mm.enabler.utils.Misc.forIn(that.loadedNodes, function(node, id){
                        // nodes in xml persistence only
                        if (that.isModifiableNode(node)) {
                            loadedNodes[id] = that._unwrap(node);
                        }
                    });
    
                    // provide created favorite nodes
                    var createdFavoriteNodes = {};
                    com.ibm.mm.enabler.utils.Misc.forIn(createdFavorites, function(node, id){
                        createdFavoriteNodes[id] = that._unwrap(node);
                    });
    
                    // provide deleted favorite nodes
                    var deletedFavoriteNodes = {};
                    com.ibm.mm.enabler.utils.Misc.forIn(deletedFavorites, function(node, id){
                        deletedFavoriteNodes[id] = that._unwrap(node);
                    });
                    
                    return {
                        createdNodes: createdNodes,
                        deletedNodes: deletedNodes,
                        loadedNodes: loadedNodes,
                        createdFavoriteNodes: createdFavoriteNodes,
                        deletedFavoriteNodes: deletedFavoriteNodes
                    };
                },
                
                // return the wrapped node
                getModelNode: dojo.hitch(that, function(node){
                    return that._wrap(node); 
                }),
    
                // callback to clean up caches after creating a node
                nodeCreated: dojo.hitch(that, function(cid, node){
                    delete that.loadedNodes[cid];
                    delete that.createdNodes[cid];
                    that._wrap(node); // implicitly puts wrapper into cache with actual id
                }),
                
                nodeUpdated: dojo.hitch(that, function(node){
                    // make updated node clean
                    that._wrap(node).setClean();
                }),
                
                // callback to clean up caches after deleting a node
                nodeDeleted: dojo.hitch(that, function(id){
                    that.invalidate(id);
                    invalidateFavorites = true;
                })
            });
            dfr.setOperationCallback(dojo.hitch(this, function(node, mode, statusCode, params){
                if (deferred && deferred.operationCallback) {
                    deferred.operationCallback(node, mode, statusCode, params);
                }
            }));
            dfr.setFinishedCallback(dojo.hitch(this, function(result, status){
                // invalidate favorites
                if ((!com.ibm.mm.enabler.utils.Misc.isEmpty(createdFavorites)) || 
                   (!com.ibm.mm.enabler.utils.Misc.isEmpty(deletedFavorites)) ||
                   (invalidateFavorites)) {
                    if (this._hasJsonModel()) {
                        // no unwrap needed as we are going to provide this as "identifiable" only
                        this._getJsonModel().invalidateFavorites();
                    }
                }
                deferred.finish(null, status);
            }));
            dfr.start(sync);

            // exit trace
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD, retVal);
            }
        });
    },
    
    getSubmittableImportForm: function(id) {
        return this._getXmlModel().getSubmittableImportForm(id);
    },
    
    remove: function(node) {
        this.enforcementHelper.enforceDefinedNotNull(node, this.modelMessages.E_PARAM_ISNULL_OR_UNDEFINED_3, ["node", this.declaredClass, "remove(node)"]);
        
        // mark node as deleted
        var id = node.getID();
        this.deletedNodes[id] = this.loadedNodes[id];
        
        // potentially remove node from created nodes
        delete this.createdNodes[id];
        
        // keep node in cache; remove callback
        //this.loadedNodes[id]._removeDirtyCallback(this._getXmlModel()._setNavigationModelDirty);
        
        // this marks the model dirty
        this._setDirty();
        
        // do xml stuff
        return this._getXmlModel().remove(node ? this._unwrap(node) : null);
    },
    
    confirmRemove: function(node) {
        if (!node) {
            return false;
        }
        var allowedByExtension = true;
        var extension = this.spaceExtensionModel.findExtensionBySpace(node).start();
        if (extension) {
            var spaceID = typeof(node) == "string" ? node : node.getID();
            var result = extension.canDeleteSpace(spaceID, null);
            allowedByExtension = (!result || result.getState() == "OK");
        }
        node = node ? this._unwrap(node) : null;
        return this._getXmlModel().confirmRemove(node);
    },
    
    confirmExport: function(node) {
        if (com.ibm.mm.enabler.AnonymousHelper.getAnonymousMode() == com.ibm.mashups.enabler.user.AnonymousMode.ANONYMOUS) {
            return false;
        }
        if (!node) {
            return false;
        }
        var allowedByExtension = true;
        var extension = this.spaceExtensionModel.findExtensionBySpace(node).start();
        if (extension) {
            var spaceID = typeof(node) == "string" ? node : node.getID();
            var result = extension.canExportSpace(spaceID, null);
            allowedByExtension = (!result || result.getState() == "OK");
        }
        node = node ? this._unwrap(node) : null;
        return allowedByExtension;
    },
    
    confirmCopy: function(node) {
        if (com.ibm.mm.enabler.AnonymousHelper.getAnonymousMode() == com.ibm.mashups.enabler.user.AnonymousMode.ANONYMOUS) {
            return false;
        }
        if (!node) {
            return false;
        }
        var allowedByExtension = true;
        var extension = this.spaceExtensionModel.findExtensionBySpace(node).start();
        if (extension) {
            var spaceID = typeof(node) == "string" ? node : node.getID();
            var result = extension.canCopySpace(spaceID, null);
            allowedByExtension = (!result || result.getState() == "OK");
        }
        node = node ? this._unwrap(node) : null;
        return allowedByExtension;
    },
    
    _getCreatedFavorites: function() {
        var retVal = {};
        var that = this;
        com.ibm.mm.enabler.utils.Misc.forIn(this.loadedNodes, function(node, id) {
            // nodes in xml persistence only
            if (that.isModifiableNode(node)) {
                var persistenceNode = that._unwrap(node);
                if ((persistenceNode.isFavoriteChanged()) && (persistenceNode.isFavorite())) {
                    retVal[id] = node;
                }
            }
        });
        com.ibm.mm.enabler.utils.Misc.forIn(this.createdNodes, function(node, id) {
            if (!(id in retVal)) {
                // nodes in xml persistence only
                if (that.isModifiableNode(node)) {
                    var persistenceNode = that._unwrap(node);
                    if ((persistenceNode.isFavoriteChanged()) && (persistenceNode.isFavorite())) {
                        retVal[id] = node;
                    }
                }
            }
        });
        return retVal;
    },

    _getDeletedFavorites: function() {
        var retVal = {};
        var that = this;
        com.ibm.mm.enabler.utils.Misc.forIn(this.loadedNodes, function(node, id) {
            // nodes in xml persistence only
            if (that.isModifiableNode(node)) {
                var persistenceNode = that._unwrap(node);
                if ((persistenceNode.isFavoriteChanged()) && (!persistenceNode.isFavorite())) {
                    retVal[id] = node;
                }
            }
        });
        return retVal;
    },
    
    // TODO: generate the following functions generically?    
    setStrategy: function(s){
        if (this._hasXmlModel()) {
            this._getXmlModel().setStrategy(s);
        }
        if (this._hasJsonModel()) {
            this._getJsonModel().setStrategy(s);
        }
        return this.inherited(arguments);
    },
    
    addStrategy: function(s){
        if (this._hasXmlModel()) {
            this._getXmlModel().addStrategy(s);
        }
        if (this._hasJsonModel()) {
            this._getJsonModel().addStrategy(s);
        }
        return this.inherited(arguments);
    },
    
    removeStrategy: function(s){
        if (this._hasXmlModel()) {
            this._getXmlModel().removeStrategy(s);
        }
        if (this._hasJsonModel()) {
            this._getJsonModel().removeStrategy(s);
        }
        return this.inherited(arguments);
    }

});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.FactoryImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.FactoryImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.FactoryImpl");





dojo.declare( "com.ibm.mm.enabler.space.FactoryImpl", com.ibm.mashups.enabler.space.Factory,
{
    getSpaceModel: function() {
        return com.ibm.mm.enabler.space.SpaceModelImpl.prototype._getInstance();
    },

    getSpaceExtensionModel: function() {
        return new com.ibm.mm.enabler.space.SpaceExtensionModelImpl.prototype._getInstance();
    }
});

// public factory
com.ibm.mashups.enabler.space.Factory = new com.ibm.mm.enabler.space.FactoryImpl();

}

if(!dojo._hasResource["com.ibm.mashups.enabler.space.Factory"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.space.Factory"] = true;
dojo.provide("com.ibm.mashups.enabler.space.Factory");







}

if(!dojo._hasResource["com.ibm.mm.enabler.space.SpaceExtensionModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.SpaceExtensionModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.SpaceExtensionModelImpl");











/**
 * Locates and loads SpaceExtension class from end points.
 */
dojo.declare("com.ibm.mm.enabler.space.SpaceExtensionModelImpl", [com.ibm.mashups.enabler.model.SpaceExtensionModel], {

    constructor: function() {
        this.loadedNodes = {};
    },

    // singleton
    _getInstance: function() {
        var instance = com.ibm.mm.enabler.space.SpaceExtensionModelImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.space.SpaceExtensionModelImpl._instance = new com.ibm.mm.enabler.space.SpaceExtensionModelImpl());
    },
    
    confirmCreate: function(context) {
        if (context) {
            if (context.space) {
                var spaceExtensionURL = context.space.getSpaceExtensionURL();
                if (spaceExtensionURL) {
                    return true;
                }
                else {
                    return false;
                }
            }
            else if (context.extensionURL) {
                return true;
            }
        }
        
        return false;
    },
    
    // spaceExtensionURL format :
    // endpoint://{com.ibm.bspace}bspaceBusinessVariableWidgetRootId/com/ibm/bspace/widgets/bslm/spaceext/FabricSpaceExtension.js#FabricSpaceExtension
    create: function(context) {
        var spaceExtensionURL = context.space ? context.space.getSpaceExtensionURL() : context.extensionURL;
        var endpointAndURL = this._getEndpointAndURL(spaceExtensionURL);
        var url = this._getFullURL(endpointAndURL);
        
        this._loadJavaScript(url);
        var className = this._getSpaceExtensionClassName(endpointAndURL.url);
        var extension = this._createSpaceExtensionImpl(className);
        extension.id = spaceExtensionURL; // id is extension url in space metadata
        return extension;
    },
    
    _getEndpointAndURL: function(spaceExtensionURL) {
        var endpointTag = "endpoint://";
        var index = spaceExtensionURL.indexOf(endpointTag);
        if (index == -1) {
            return {
                "endpoint": null,
                "url": spaceExtensionURL
            };
        }
        
        var start = index + endpointTag.length;
        var firstSlash = spaceExtensionURL.indexOf("/", start);
        var endpointID = firstSlash == -1 ? spaceExtensionURL.substring(start) : spaceExtensionURL.substring(start, firstSlash);
        var url = firstSlash == -1 ? null : spaceExtensionURL.substring(firstSlash);
        return {
            "endpoint": endpointID,
            "url": url
        };
    },
    
    _getEndpointURL: function(spaceExtensionID) {
        var configService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
        var configObject = configService.getConfigObject(com.ibm.mashups.enabler.services.ConfigConstants.ENDPOINT_CONFIG_PROVIDER);
        var endpoints = configObject.getValue(com.ibm.mashups.enabler.services.ConfigConstants.ENDPOINTS);
        if (endpoints) {
            var endpointsArray = endpoints.split(';');
            for (var i = 0; i < endpointsArray.length; i++) {
                if (endpointsArray[i] == spaceExtensionID) {
                    return configObject.getValue(spaceExtensionID + ".url");
                }
            }
        }
        
        return null;
    },
    
    _getLocalServer: function(url) {
        if (url.indexOf("http://") == -1 && url.indexOf("https://") == -1) {
            // this is a relative path, need to get server url and pre-pend to current url.
            var serverHttpUrl = new com.ibm.mm.enabler.utils.HttpUrl("/");
            var server = serverHttpUrl.toString();
            while (server.charAt(server.length - 1) == '/') {
                server = server.substr(0, server.length - 1);
            }
            return server;
        }
        
        return null;
    },
    
    _getFullURL: function(endpointAndURL) {
        var endpointURL = endpointAndURL.endpoint ? this._getEndpointURL(endpointAndURL.endpoint) : null;
        var spaceExtensionURL = endpointAndURL.url;
        
        var fullURL = spaceExtensionURL;
        if (endpointURL && endpointURL.length > 0) {
            if (endpointURL.charAt(endpointURL.length - 1) == '/') {
                if (spaceExtensionURL.length > 0 && spaceExtensionURL.charAt(0) == '/') {
                    spaceExtensionURL = spaceExtensionURL.substring(1);
                }
                fullURL = endpointURL + spaceExtensionURL;
            }
            else {
                if (spaceExtensionURL) {
                    if (spaceExtensionURL.length > 0 && spaceExtensionURL.charAt(0) == '/') {
                        fullURL = endpointURL + spaceExtensionURL;
                    }
                    else {
                        fullURL = endpointURL + '/' + spaceExtensionURL;
                    }
                }
                else {
                    fullURL = endpointURL;
                }
            }
        }
        
        var server = this._getLocalServer(fullURL);
        if (server) {
            fullURL = (fullURL.charAt(0) == '/') ? server + fullURL : server + '/' + fullURL;
        }
        
        var separator = fullURL.lastIndexOf("#");
        if (separator != -1) {
            fullURL = fullURL.substring(0, separator);
        }
        
        return fullURL;
    },
    
    _loadJavaScript: function(url) {
        var externalScript = new com.ibm.mm.enabler.aggregation.javascript.ExternalScriptFilter();
        externalScript.loadExternalScript(url);
    },
    
    _getSpaceExtensionClassName: function(spaceExtensionURL) {
        var urls = spaceExtensionURL.split("#");
        if (urls.length === 0) {
            return null;
        }
        else if (urls.length > 1) {
            var className = urls[1].replace(/^\s+|\s+$/g, "");
            if (className != "") {
                return className;
            }
        }
        
        // fallback to from sript name
        var scriptName = urls[0];
        var lastSlash = scriptName.lastIndexOf('/');
        if (lastSlash == -1) {
            lastSlash = 0;
        }
        else if (lastSlash == scriptName.length - 1) {
            return null;
        }
        else {
            lastSlash = lastSlash + 1;
        }
        
        var jsExtension = scriptName.lastIndexOf(".js");
        if (jsExtension == -1) {
            return scriptName.substring(lastSlash);
        }
        else {
            return scriptName.substring(lastSlash, jsExtension);
        }
    },
    
    _createSpaceExtensionImpl: function(spaceExtClassName) {
        if (spaceExtClassName.indexOf("()") !== -1) {
            spaceExtClassName = spaceExtClassName.substring(0,spaceExtClassName.length - 2);
        }
        var spaceExtObject = new (dojo.getObject(spaceExtClassName))();
        return spaceExtObject;
    },
    
    insert: function(node, nextNode) {
    },
    
    confirmInsert: function(node, nextNode) {
        return "false";
    },
    
    // mostly findExtensionBySpace is used.
    find: function(id) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, id);
    },
    
    _find: function(deferred, sync, id) {
        if (id in this.loadedNodes) {
            // fetch node from cache
            // public callback handling
            if(deferred) { deferred.finish(this.loadedNodes[id], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK); }
            
            return this.loadedNodes[id];
        }
        
        var extension = this.create({
            "extensionURL": id
        });
        this.loadedNodes[id] = extension;
        
        return extension;
    },
    
    findExtensionBySpace: function(spaceNode) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._findExtensionBySpace, spaceNode);
    },
    
    _findExtensionBySpace: function(deferred, sync, spaceNode) {
        var space;
        if (typeof(spaceNode) == "string") {
            var spaceModel = com.ibm.mashups.enabler.space.Factory.getSpaceModel();
            space = spaceModel.find(spaceNode).start();
        }
        else {
            space = spaceNode;
        }
        
        var hasExtension = this.confirmCreate({
            "space": space
        });
        if (!hasExtension) {
            return null;
        }
        
        // extension url is used as ID.
        return this._find(deferred, sync, space.getSpaceExtensionURL());
    }
});

/**
 *
 * Non namespaced space extension for back-compatible with existing JS.
 */
dojo.declare("SpaceExtension", [com.ibm.mashups.enabler.model.space.SpaceExtension], {});


/**
 *
 * Non namespaced space extension result for back-compatible with existing JS.
 */
dojo.declare("SpaceExtensionResult", [com.ibm.mashups.enabler.model.space.SpaceExtensionResult], {});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.SpaceExtensionModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.SpaceExtensionModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.SpaceExtensionModel");








}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.StaticSkinModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.StaticSkinModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.StaticSkinModel_API");
dojo.provide("com.ibm.mashups.enabler.model.StaticSkinModel");

/**
 * @ibm-module ThemeModel
 * @since 2.4
 * @private
 */
dojo.declare("com.ibm.mashups.enabler.model.StaticSkinModel", [com.ibm.mashups.enabler.Locator], {
    
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.StaticSkinModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.StaticSkinModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.StaticSkinModelImpl");




dojo.declare("com.ibm.mm.enabler.theme.StaticSkinModelImpl",com.ibm.mashups.enabler.model.StaticSkinModel,{
    // singleton
    getInstance: function() {
        if (!com.ibm.mm.enabler.theme.StaticSkinModelImpl._instance) {
            com.ibm.mm.enabler.theme.StaticSkinModelImpl._instance = new com.ibm.mm.enabler.theme.StaticSkinModelImpl();
        }
        return com.ibm.mm.enabler.theme.StaticSkinModelImpl._instance;
    },
    
    _skinModel: null,
    
    constructor: function() {
        this._skinModel = new com.ibm.mm.enabler.theme.SkinModelImpl();
    },
    
    find: function(id,themeId) {
       this._skinModel._setThemeId(themeId);
       return this._skinModel.find(id);
    },
    
    invalidate: function() {
    	this._skinModel.invalidate();
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.StaticSkinModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.StaticSkinModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.StaticSkinModel");








}

if(!dojo._hasResource["com.ibm.mashups.enabler.template.TemplateNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.template.TemplateNode"] = true;
dojo.provide("com.ibm.mashups.enabler.template.TemplateNode");




/**
 * Interface representing a template node.
 * @ibm-api
 * @ibm-module TemplateModel
 */
dojo.declare("com.ibm.mashups.enabler.template.TemplateNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.ModifiableLocalized], {
    /**
     * Returns a URL to export the template in zip file format
     * @return {String} link to export the template
     */
    getExportURL: function(){
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.TemplateModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.TemplateModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.TemplateModel_API");
dojo.provide("com.ibm.mashups.enabler.model.TemplateModel");

/**
 * Interface for a template model.
 * @ibm-api
 * @ibm-module TemplateModel
 */
dojo.declare("com.ibm.mashups.enabler.model.TemplateModel", [com.ibm.mashups.enabler.ListModelController, com.ibm.mashups.enabler.Invalidatable, com.ibm.mashups.enabler.SubmittableFormProvider], {
    /**
     * Commits the modifications applied to this model and all dependent models.<br>
     * @return {DeferredOperation} a deferred object used to start this operation. 
     * The return value when executed through the deferred object is <code>null</null>
     */
    commit: function(){
    },
    
    /**
     * Confirms whether exporting the template is possible.
     * @param{com.ibm.mashups.enabler.template.TemplateNode} template template object or template uri (without any scope).
     * Must not be <code>null</code>
     * @return {Boolean} <code>true</code> if the node can be exported, <code>false</code> otherwise.
     */
    confirmExport: function(template){
    },
    
    /**
     * Returns a SubmittableForm object associated with the given id to import a template.
     * Same as <code>getSubmittableForm</code>
     * @param {String} id the id of the html form this object is associated with
     * @return {com.ibm.mashups.enabler.SubmittableForm} SubmittableForm object, maybe <code>null</code>
     */
    getSubmittableImportForm: function(id){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.template.TemplateNodeImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.template.TemplateNodeImpl"] = true;
dojo.provide("com.ibm.mm.enabler.template.TemplateNodeImpl");










dojo.declare("com.ibm.mm.enabler.template.TemplateNodeImpl", [com.ibm.mashups.enabler.template.TemplateNode, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.EditLinkProviderXmlImpl], {
    constructor: function(node){
        this.xmlData = node;
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_TEMPLATE);
        this.prefix = this.serviceJson.idprefix;
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_CA]));
        
        // Override LocalizedImpl to handle composite app titles and descriptions
        this.li_ns = dojo.delegate(this.serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_XML, nsf.NS_CA]));
        
        this.BASE_NLS_TITLE_STRING = "ca:title";
        this.BASE_NLS_DESCRIPTION_STRING = "ca:description";
        this.BASE_NS = this.li_ns.ca;

        // title
        this.TITLE_ELEMENT = "atom:content/*";
        this.TITLE = this.TITLE_ELEMENT + "/" + this.BASE_NLS_TITLE_STRING;

        // description
        this.DESCRIPTION_ELEMENT = "atom:content/*";
        this.DESCRIPTION = this.DESCRIPTION_ELEMENT + "/" + this.BASE_NLS_DESCRIPTION_STRING;
		
    },
    
    getExportURL: function(){
        var expr = "atom:link[@ca:rel='template-export']";
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        return node ? node.getAttribute("href") : null;
    },
    
    toString: function(){
        return com.ibm.mm.enabler.utils.Dom.innerXML(this.xmlData);
    }
    
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.ModelIteratorUnorderedImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.ModelIteratorUnorderedImpl"] = true;
dojo.provide("com.ibm.mm.enabler.ModelIteratorUnorderedImpl");











// this is an implementation of an unorder iterator
dojo.declare("com.ibm.mm.enabler.ModelIteratorUnorderedImpl", [com.ibm.mm.enabler.DeferredIteratorImpl], {
    constructor: function(url, model) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        // the iterator's model
        this.model = model;
        
        // entries in feed order; undefined entries denote potential 
        // nodes to load; null entry marks end of node collection 
        this.entries = [];
        
        // cursor position
        this.cursor = 0;
        
        // number of nodes
        this.sizeValue = null;
        
        // atom feed
        this.ATOM_FEED = "atom:feed";
        
        // atom entry
        this.ATOM_ENTRY = this.ATOM_FEED + "/atom:entry";
        
        // next link
        this.ATOM_NEXT = this.ATOM_FEED + "/atom:link[@rel='next']";
        
        // size indicator
        this.OPENSEARCH_TOTALRESULTS = this.ATOM_FEED + "/opensearch:totalResults";
        
        if (url) {
            // collection url
            this.collectionUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(url, model);
            
            if (model.strategy) {
                for (var x in model.strategy) {
                    if (com.ibm.mm.enabler.utils.Misc.isInstanceOf(model.strategy[x], com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy)) {
                        // max number of entries 
                        this.collectionUrl.setParameter("num", model.strategy[x].getInterval());
                        // start element (only set with paging) 
                        this.collectionUrl.setParameter("start", this.getCursorPosition());
                        break;
                    }
                }
            }
            
            // let iterator implementations customize the url
            this._customizeUrl(this.collectionUrl);
        }
        
        // number of created nodes to include in this iterator
        this.numCreatedNodes = 0;
        
        // compile key-value pairs for each created node included in this 
        // iterator: (createdId, nextId)
        this._createdNodes = {};
        com.ibm.mm.enabler.utils.Misc.forIn(this.model.createdNodes, function(value, createdId) {
            var created = this.model.find(createdId).start();
            this._createdNodes[createdId] = created;
            this.numCreatedNodes++;
        }, this);
        
        // number of removed nodes not to include in this iterator
        this.numRemovedNodes = 0;
        
        // if the url is not present, removed nodes do not count towards size anymore
        if (url) {
            com.ibm.mm.enabler.utils.Misc.forIn(this.model.deletedNodes, function() {
                this.numRemovedNodes++;
            }, this);
        }
    },
    
    // This abstract function must be implemented for each iterator implementation.
    // It is invoked for each atom entry in the collection, where the atom entry 
    // is passed in as xml data. This function must return an instance of a 
    // resource based on the xml data, or null, in case no node is to be created.
    _createNode: function(xmlData) {
        return null;
    },
    
    // This abstract function allows to modify the collection url, e.g. by adding 
    // url parameters. This method is applied on the url created in the constructor 
    // of the base iterator implementation, as well as on all 'next'-link urls, 
    // which are obtained from collection chunks in order to obtain the next chunk. 
    _customizeUrl: function(url) {
    },
    
    hasNext: function(dfr, sync) {
        if (typeof this.entries[this.cursor] == "undefined") {
            var insertionIndex = 0;
            if (this.collectionUrl) {
                // load nodes from server
                this._loadAhead(dfr || null, true);
                insertionIndex = this.cursor;
            }
            else {
                // do not load entries
                this.entries[0] = null;
            }
            
            // reflect number of created/removed nodes in size value
            this.sizeValue = this.sizeValue + this.numCreatedNodes - this.numRemovedNodes;
        }
        
        var candidate = this.entries[this.cursor];
        var hasNext = candidate ? true : false;
        if (hasNext && this.model.deletedNodes && this.model.deletedNodes[candidate]) {
            // do not expose nodes, which are deleted at runtime
            this.cursor++;
            return this.hasNext(dfr, sync);
        }
        return hasNext;
    },
    
    next: function(dfr, sync) {
        return this.hasNext(dfr || null, typeof sync == "undefined" ? true : sync) ? this.model.loadedNodes[this.entries[this.cursor++]] : null;
    },
    
    _insertCreated: function(entries, cursor) {
        for (var createdId in this._createdNodes) { // JSLINT-IGNORE: no prototype properties in here
            entries.splice(cursor, 0, createdId);
            cursor++;
        }
    },
    
    setCursorPosition: function(position) {
        this.cursor = position;
        this.collectionUrl.setParameter("start", position);
    },
    
    getCursorPosition: function() {
        return this.cursor;
    },
    
    size: function() {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._size);
    },
    
    _size: function(dfr, sync) {
        if (!this.sizeValue) {
            this.hasNext(null, true);
        }
        
        // finished callback 
        if (dfr) {
            dfr.finish(this.sizeValue, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
        }
        // convenience
        return this.sizeValue;
    },
    
    start: function(sync) {
        this._loadAhead(this, sync, true);
    },
    
    _loadAhead: function(deferred, sync, iterate) {
        // logging support
        var LOG_METHOD = "_loadAhead";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        
        // handle empty iterator
        if (!this.collectionUrl) {
            this.entries[this.cursor] = null;
            if (deferred) {
                deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
            return;
        }
        
        // service request
        var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(this.collectionUrl, null, null, false, sync);
        serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
            
                // fetch size
                this.sizeValue = com.ibm.mashups.enabler.xml.XPath.evaluateNumber(this.OPENSEARCH_TOTALRESULTS, data, this.model.ns);
                
                // iterate atom entries
                var c = 0;
                var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(this.ATOM_ENTRY, data, this.model.ns);
                if (nodes) {
                    for (var i = 0, l = nodes.length; i < l; i++) {
                        // instantiate node
                        var node = this._createNode(nodes[i]);
                        if (node) {
                            // put id in entry list
                            this.entries[this.cursor + c++] = node.getID();
                            
                            // callback handling
                            if (deferred) {
                                deferred.nextFinish(node);
                            }
                        }
                        else {
                            // an entry for which no node is instantiated  
                            // does not count towards collection size
                            this.sizeValue--;
                        }
                    }
                }
                // check for 'next'-link on feed level
                var next = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(this.ATOM_NEXT, data, this.model.ns);
                if (next) {
                    var url = next.getAttribute("href");
                    
                    // set collection url from 'next'-link
                    this.collectionUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(url, this.model);
                    
                    // let iterator implementations customize the url
                    this._customizeUrl(this.collectionUrl);
                    if (nodes && iterate) {
                        // was last next element
                        this._loadAhead(deferred, sync, iterate);
                    }
                }
                else {
                    // mark end of collection
                    this.entries[this.cursor + c] = null;

                    // inject nodes, which are created at runtime
                    this._insertCreated(this.entries, this.cursor + c);
                    
                    // finished callback (end of collection)
                    if (deferred) {
                        deferred.finish(null, xhr.status);
                    }
                }
            }
            else {
                // finished callback (error)
                if (deferred) {
                    deferred.finish(null, xhr.status);
                }
            }
        }));
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.template.TemplateModelIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.template.TemplateModelIteratorImpl"] = true;
// APIs



// Implementations




dojo.provide("com.ibm.mm.enabler.template.TemplateModelIteratorImpl");

// navigation model iterator
dojo.declare("com.ibm.mm.enabler.template.TemplateModelIteratorImpl", [com.ibm.mm.enabler.ModelIteratorUnorderedImpl], {
    constructor: function() {
    },
    
    _createNode: function(xmlData) {
       var node = new com.ibm.mm.enabler.template.TemplateNodeImpl(xmlData);
       /*
            node._addDirtyCallback(this.model, function() {
                // if navigation node gets dirty, so does the model 
                this._setDirty();
            });
       */     
        // cache node in model
        node = this.model._cacheNode(node);

        return node || null;
    },
    
    _customizeUrl: function(url) {
        // include access control info
        url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);

        // apply caching strategy
        this.model._applyNoCacheStrategy(url);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.template.TemplateModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.template.TemplateModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.template.TemplateModelImpl");

























dojo.declare("com.ibm.mm.enabler.template.TemplateModelImpl", [com.ibm.mashups.enabler.ListModel, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.ConfirmableModelImpl], {

    constructor: function() {
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        var LOG_METHOD = "constructor()";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD);
        }

        this.loadedNodes = {};
        this.createdNodes = {};
        this.deletedNodes = {};
        this.initialContents = {};
        
        this.modelMessages = dojo.i18n.getLocalization("com.ibm.mm.enabler", "modelMessages");
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_TEMPLATE);
        this.prefix = this.serviceJson.idprefix;
        this.nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, this.nsf.getNameSpaces([this.nsf.NS_ATOM, this.nsf.NS_XHTML, this.nsf.NS_OPENSEARCH, this.nsf.NS_CA, this.nsf.NS_APP]));
        
        this.urlType = com.ibm.mashups.enabler.model.url.ModelUrlFactory.TEMPLATE_URL;
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    
    // singleton
    getInstance: function() {
        var instance = com.ibm.mm.enabler.template.TemplateModelImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.template.TemplateModelImpl._instance = new com.ibm.mm.enabler.template.TemplateModelImpl());
    },
    
    invalidate: function() {
        delete com.ibm.mm.enabler.template.TemplateModelImpl._instance;
        // clear all
        
        // get IDs before - concurrency!
        var clearIds = [];
        for (var id in this.loadedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.loadedNodes, id)) {
                clearIds.push(id);
            }
        }
        dojo.forEach(clearIds, this._invalidate, [this, this.loadedNodes, true]);
        clearIds = [];
        for (var id in this.createdNodes) {
            if (Object.prototype.hasOwnProperty.call(this.createdNodes, id)) {
                clearIds.push(id);
            }
        }
        dojo.forEach(clearIds, this._invalidate, [this, this.createdNodes, false]);
        clearIds = [];
        for (var id in this.deletedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.deletedNodes, id)) {
                clearIds.push(id);
            }
        }
        dojo.forEach(clearIds, this._invalidate, [this, this.deletedNodes, false]);

        this.loadedNodes = {};
        this.createdNodes = {};
        this.deletedNodes = {};
        this.initialContents = {};
    },
    _invalidate: function(id) {
        var _that = this[0];
        var _array = this[1];
        
        if (id in _array) {
            delete _array[id];
            if (id in _that.initialContents) {
                delete _that.initialContents[id];
            }
        }
    },
    
    find: function(id) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, id);
    },
    
    _find: function(deferred, sync, id) {
        var LOG_METHOD = "_find(id, deferred, sync)";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [id, deferred, sync]);
        }
        
        if (id in this.deletedNodes) {
            // public callback handling
            if (deferred) {
                deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
            }
            
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD);
            }
            return null;
        }
        
        if (id in this.loadedNodes) {
            // fetch node from cache
            // public callback handling
            if (deferred) {
                deferred.finish(this.loadedNodes[id], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
            
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD, this.loadedNodes[id]);
            }
            
            return this.loadedNodes[id];
        }
        
        // use href info from identifier
        var myUrl = null;
        var href = com.ibm.mm.enabler.utils.Utils.getHrefFromIdentifiable(id);
        if (href) {
            myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(href, this);
        }
        else {
            // construct url otherwise
            myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(this.urlType, this);
            myUrl.setNodes([{
                value: id,
                isID: true
            }]);
        }
        
        // include access control info
        myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
        // apply caching strategy
        this._applyNoCacheStrategy(myUrl);

        var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
        var entry = null;
        var result = {};
        serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
            var LOG_METHOD = "_find_callback(type, data, xhr, args)";
            if (bIsLoggable) {
                this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
            }
            
            var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("//atom:entry", data, this.ns);
            
            var newentry = new com.ibm.mm.enabler.template.TemplateNodeImpl(node);
            this.newid = newentry.getID();
            this.loadedNodes[this.newid] = newentry;
            this.initialContents[this.newid] = newentry.toString();
            
            
            // finished callback 
            if (deferred) {
                deferred.finish(this.loadedNodes[this.newid] || null, xhr.status);
            }
            
            // handle error in sync call
            if (sync && type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                // do not expose HTTP_NOT_FOUND; simply return null in this case
                if (xhr.status != com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND) {
                    result.status = xhr.status;
                }
            }
            
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD);
            }
        }));
        if (result.status) {
            throw new Error(dojo.string.substitute(this.modelMessages.E_COMNODE_NOTFOUND_1, [id, result.status]));
        }
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, this.loadedNodes[this.newid]);
        }
        return this.loadedNodes[this.newid];
    },
    
    create: function(context) {
        // Not implemented here - create is run through import
        return null;
    },
    confirmCreate: function(context) {
        return this.getConfirmCreate();
    },
    
    insert: function(node, nextNode) {
        var id = node.getID();
        this.createdNodes[id] = node;
    },
    confirmInsert: function(node, nextNode) {
        return this.getConfirmInsert(nextNode);
    },
    
    confirmExport: function(node) {
        if (com.ibm.mm.enabler.AnonymousHelper.getAnonymousMode() == com.ibm.mashups.enabler.user.AnonymousMode.ANONYMOUS) {
            return false;
        }
        return true;
    },
    
    commit: function() {
        return new com.ibm.mm.enabler.DeferredOperationImpl(this, this._commit);
    },
    
    _commit: function(deferred, sync) {
        var LOG_METHOD = "_commit(deferred, sync)";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [deferred, sync]);
        }
        
        // initialize callback handling
        this.statusCode = com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK;
        this.requestCount = 0;
        this._acquire();
        
        // HTTP PUT updated nodes
        for (var id in this.loadedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.loadedNodes,id)) {
                // Not implemented here - update is run through import
            }
        }
        
        // HTTP POST created nodes
        for (var created_id in this.createdNodes) {
            if (Object.prototype.hasOwnProperty.call(this.createdNodes,created_id)) {
                // Not implemented here - update is run through import
            }
        }
        this.createdNodes = {};
        
        // HTTP DELETE deleted nodes
        for (var deleted_id in this.deletedNodes) {
            if (Object.prototype.hasOwnProperty.call(this.deletedNodes,deleted_id)) {
                if (bIsLoggable) {
                    this.LOGGER.trace(LOG_METHOD, "Processing delete of template ${0}", deleted_id);
                }
                
                var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.deletedNodes[deleted_id].getEditLink(), this);
                var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                this._acquire();
                serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args) {
                    var LOG_METHOD = "_commit_remove_callback(type, data, xhr, args)";
                    if (bIsLoggable) {
                        this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
                    }
                    
                    // finished callback
                    if (deferred && deferred.operationCallback) {
                        deferred.operationCallback(newentry, com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE, xhr.status, deferred.operationCallbackParameters);
                    }
                    
                    this._release(deferred, xhr.status);
                    
                    if (bIsLoggable) {
                        this.LOGGER.exiting(LOG_METHOD, [type, data, xhr, args]);
                    }
                })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
            }
        }
        this.deletedNodes = {};
        
        // the finishedCallback ist called in _release
        this._release(deferred, 0);
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    
    // acquire semaphore
    _acquire: function() {
        this.requestCount++;
    },
    
    // release semaphore
    _release: function(deferred, statusCode) {
        // set overall status Code
        this.statusCode = Math.max(parseInt(statusCode, 10), this.statusCode);
        // check for full release
        this.requestCount--;
        if (this.requestCount <= 0) {
            if (deferred) {
                deferred.finish(null, this.statusCode);
            }
        }
    },
    
    remove: function(node) {
        var LOG_METHOD = "remove(node)";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, node);
        }
        
        var id = node.getID();
        // mark node as deleted
        if (!(id in this.deletedNodes)) {
            this.deletedNodes[id] = this.loadedNodes[id];
        }
        // mark cache entry
        if (id in this.loadedNodes) {
            delete this.loadedNodes[id];
        }
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, node);
        }
    },
    confirmRemove: function(node) {
        return this.getConfirmRemove(node);
    },
    
    iterator: function() {
        return new com.ibm.mm.enabler.template.TemplateModelIteratorImpl(this.serviceJson.url, this);
    },
    
    _cacheNode: function(node) {
        var id = node.getID();
        if (!(id in this.loadedNodes)) {
            this.loadedNodes[id] = node;
            this.initialContents[id] = node.toString();
        }
        return this.loadedNodes[id];
    },
    _applyNoCacheStrategy: function(url) {
        var s = this._findStrategyByType("com.ibm.mashups.enabler.strategy.NoCacheStrategy");
        if (s) {
            url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_NOCACHE, "true");
        }
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.TemplateModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.TemplateModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.TemplateModel");










}

if(!dojo._hasResource["com.ibm.mashups.enabler.theme.Theme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.theme.Theme"] = true;
dojo.provide("com.ibm.mashups.enabler.theme.Theme");






/**
 * Interface that represents a theme; used in the ThemeModel
 * @ibm-api
 * @ibm-module ThemeModel
 */
dojo.declare("com.ibm.mashups.enabler.theme.Theme", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized, com.ibm.mashups.enabler.DefaultLocalized, com.ibm.mashups.enabler.MetaData], {
    /**
     * Returns the base url of the theme.
     * @type String
     * @return base url of the theme. can never be <code>null</code>
     * @deprecated Since 2.0. Use ThemeModel/ResourceLocator findResourceURL instead
     */
    getBaseUrl: function(){
    },
    
    /**
     * Returns the default skin id for the given theme.
     * @type String
     * @return the default skin id for given theme, <code>null</code> in case of no default skin set.
     * @deprecated use com.ibm.mashups.enabler.ThemeProvider.getDefaultSkinIdentifier instead
     */
    getDefaultSkin: function(){
    },
    
    /**
     * Returns the default skin identifier for this theme.
     * @return {com.ibm.mashups.enabler.Identifiable} the default skin id for this theme, <code>null</code> in case of
     * no default skin set.
     */
    getDefaultSkinIdentifier: function(){
    }
    
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.ThemeModel_API"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.ThemeModel_API"] = true;
dojo.provide("com.ibm.mashups.enabler.model.ThemeModel_API");
dojo.provide("com.ibm.mashups.enabler.model.ThemeModel");

/**
 * Interface for a theme model. This list model describes a list of theme objects.<br>
 * <br>
 * The theme model supports the following strategies:
 * <ul>
 * <li>{@link com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy ListLoadAheadStrategy}</li>
 * <li>{@link com.ibm.mashups.enabler.strategy.HiddenMetaDataLoadingStrategy HiddenMetaDataLoadingStrategy}</li>
 * </ul>
 * @ibm-api
 * @ibm-module ThemeModel
 */
dojo.declare("com.ibm.mashups.enabler.model.ThemeModel", [com.ibm.mashups.enabler.ListModel, com.ibm.mashups.enabler.ResourceLocator], {

    /**
     * Returns the skin model for the given theme.
     * @param {com.ibm.mashups.enabler.theme.Theme} theme the Theme for which to return the skin model;
     * must not be <code>null</code>
     * @type com.ibm.mashups.enabler.model.SkinModel
     * @return skin model for given theme, Never <code>null</code>.
     */
    getSkinModel: function(theme){
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.EmptySkinModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.EmptySkinModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.EmptySkinModelImpl");









// empty skin model
dojo.declare("com.ibm.mm.enabler.theme.EmptySkinModelImpl", [com.ibm.mashups.enabler.model.SkinModel, com.ibm.mm.enabler.model.ModelImpl], {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
    },
    
    iterator: function() {
        return new com.ibm.mm.enabler.EmptyModelIteratorImpl();
    },
    
    find: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, uri);
    },
    
    _find: function(dfr, sync, uri) {
        // finished callback 
        if(dfr) { dfr.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND); }
        // convenience
        return null;
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.ThemeImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.ThemeImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.ThemeImpl");















// theme
dojo.declare("com.ibm.mm.enabler.theme.ThemeImpl", [com.ibm.mashups.enabler.theme.Theme, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.DefaultLocalizedXmlImpl, com.ibm.mm.enabler.MetaDataImpl2], {
    constructor: function(node) {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        this.xmlData = node;
        // service document and initialization
        var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_THEME);
        this.prefix = serviceJson.idprefix;
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM]));
    },
    
    getBaseUrl: function(internal) {
        if (typeof internal == "undefined") {
            var LOG_METHOD = "getBaseUrl";
            this.LOGGER.warning(LOG_METHOD, "Deprecated. Use com.ibm.mashups.enabler.model.ThemeModel.findResourceURL instead.");
        }
        return this._getBaseUrl();
    },
    _getBaseUrl: function() {
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:content/model:theme/model:link[@ext:rel='baseUrl']", this.xmlData, this.ns);
        return node ? node.getAttribute("href") : null;
    },
    
    // deprecated
    getDefaultSkin: function() {
        var id = this.getDefaultSkinIdentifier();
        return id ? id.getID() : null;
    },
    
    getDefaultSkinIdentifier: function() {
        var result = null;
        var expr = "atom:link[@rel='related' and @ext:rel='defaultSkin']";
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        if (node) {
            var id = com.ibm.mm.enabler.utils.Utils.getIdFromExtUri(this.prefix, node);
            var href = node.getAttribute("href");
            result = new com.ibm.mm.enabler.IdentifierImpl(id, {
                "href": href
            });
        }
        return result || null;
    },
    
    toString: function() {
        return com.ibm.mm.enabler.utils.Dom.innerXML(this.xmlData);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.ThemeModelIteratorImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.ThemeModelIteratorImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.ThemeModelIteratorImpl");











// theme model iterator
dojo.declare("com.ibm.mm.enabler.theme.ThemeModelIteratorImpl", [com.ibm.mashups.enabler.Iterator, com.ibm.mm.enabler.DeferredIteratorImpl, com.ibm.mm.enabler.ModelIteratorImpl], {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
    },
    
    _createNode: function(xmlData) {
        var node = new com.ibm.mm.enabler.theme.ThemeImpl(xmlData);
        this.model.loadedNodes[node.getID()] = node;
        return node;
    },
    
    _customizeUrl: function(url) {
        // include access control info
        url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
        // full representation
        url.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);
        
        // include hidden meta data
        this.model._applyHiddenMetaDataLoadingStrategyToModelURL(url);
    }
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.theme.ThemeModelImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.theme.ThemeModelImpl"] = true;
dojo.provide("com.ibm.mm.enabler.theme.ThemeModelImpl");






















dojo.declare("com.ibm.mm.enabler.theme.ThemeModelImpl", [com.ibm.mashups.enabler.model.ThemeModel, com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.strategy.HiddenMetaDataLoadingStrategyHelperImpl], {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        
        // cache of loaded nodes; (name:id, value:node)
        this.loadedNodes = {};
        
        // skin models
        this.skinModels = {};
        
        // service document and initialization
        var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_THEME);
        var nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(serviceJson.namespaces, nsf.getNameSpaces([nsf.NS_ATOM, nsf.NS_THR, nsf.NS_XML, nsf.NS_OPENSEARCH]));
        
        // theme collection url
        this.collectionUrl = serviceJson.url;
    },
    
    // singleton
    getInstance: function() {
        var instance = com.ibm.mm.enabler.theme.ThemeModelImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.theme.ThemeModelImpl._instance = new com.ibm.mm.enabler.theme.ThemeModelImpl());
    },
    
    iterator: function() {
        return new com.ibm.mm.enabler.theme.ThemeModelIteratorImpl(this.collectionUrl, this);
    },
    
    find: function(uri) {
        return new com.ibm.mm.enabler.DeferredImpl(this, this._load, uri);
    },
    
    getSkinModel: function(theme) {
        var themeId = theme.getID();
        if (!(themeId in this.skinModels)) {
            // link to skin collection
            var href;
            var expr = "atom:link[@ext:rel='skin']";
            var skin = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, theme.xmlData, this.ns);
            if (skin) {
                href = skin.getAttribute("href");
                this.skinModels[themeId] = new com.ibm.mm.enabler.theme.SkinModelImpl(href, themeId);
            }
            if (!href) {
                // dont cache empty model
                return new com.ibm.mm.enabler.theme.EmptySkinModelImpl();
            }
        }
        return this.skinModels[themeId];
    },
    
    _load: function(deferred, sync, ident) {
        // logging support
        var LOG_METHOD = "_load";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        
        var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(ident);
        if (id in this.loadedNodes) {
            // finished callback
            if(deferred) { deferred.finish(this.loadedNodes[id], com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK); }
        }
        else {
            var myUrl;
            // use href info from identifier
            var href = com.ibm.mm.enabler.utils.Utils.getHrefFromIdentifiable(ident);
            if (href) {
                myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(href, this);
            }
            else {
                // construct url otherwise
                myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.THEME_URL, this);
                myUrl.setNodes([{
                    value: id,
                    isID: true
                }]);
            }
            // include access control info
            myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
            
            // full representation
            myUrl.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);
            
            // apply hidden meta data strategy
            this._applyHiddenMetaDataLoadingStrategyToModelURL(myUrl);
            
            var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
            serviceReq.read(dojo.hitch(this, function(type, data, xhr, args) {
                if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                    var expr = "atom:feed/atom:entry";
                    var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, data, this.ns);
                    if (node) {
                        this.loadedNodes[id] = new com.ibm.mm.enabler.theme.ThemeImpl(node);
                    }
                }
                // finished/error callback
                if(deferred) { deferred.finish(this.loadedNodes[id] || null, xhr.status); }
            }));
        }
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
        
        return this.loadedNodes[id] || null;
    },
    
    // @deprecated
    findResourceUrl: function(node, name) {
        return this.findResourceURL(node, name);
    },
    findResourceURL: function(node, name){
        var url;
        var xmlResult = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:link[@ext:rel='media']", node.xmlData, node.ns);
        if (xmlResult) {
            var href = xmlResult.getAttribute("href");
            if (href) {
                if (href.lastIndexOf(name) == (href.length - name.length)) {
                    url = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(href, this);
                }
            }
        }
        if (!url) {
            url = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.THEME_MEDIA_URL, this);
            var nodeJSON = [{
                value: node.getID(),
                isID: true
            }, {
                value: name,
                isID: false
            }];
            url.setNodes(nodeJSON);
        }
        return url.getAbsoluteURL();
    }
});

}

if(!dojo._hasResource["com.ibm.mashups.enabler.model.ThemeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.enabler.model.ThemeModel"] = true;
dojo.provide("com.ibm.mashups.enabler.model.ThemeModel");









}

if(!dojo._hasResource["com.ibm.mm.enabler.opensocial"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.opensocial"] = true;
dojo.provide("com.ibm.mm.enabler.opensocial");
/*
 * This is injected on the fly and not fetched through an API. Therefore we need to define the @ibm-module so that the build process is picking it up
 * @ibm-module OSGadget
 */


dojo.declare("com.ibm.mm.enabler.opensocial", null, {
    renderGadget: function(gadgetId, gadgetDefinitionUrl) {
    
        // check if gadget existed before and delete it from shindig and from OAHub container 
        this.removeGadgetInstance(gadgetId);
        
		// update title
		this._updateTitle(gadgetId);
		
        shindig.container.layoutManager = new shindig.FloatLeftLayoutManager(gadgetId);
        // prepare variables to get user attributes
        var userProperties = {};
        userProperties._mum_gadget_ID_ = {
            value: gadgetId
        }; // save mashups ID to use later in shindig.DefaultUserPrefStore.prototype.savePrefs
        this.getGadgetUserPreferences(gadgetId, userProperties);
        
        var gadget0 = shindig.container.createGadget({
            specUrl: gadgetDefinitionUrl.toString(),
            userPrefs: userProperties
        });
        
        gadget0.setServerBase(document.location.protocol + "//" + document.location.host + ibmCfg.enablerConfig["com.ibm.mashups.contextroot.opensocial"] + "/gadgets/");
        var gadgetLocaleObject = com.ibm.mm.enabler.utils.LocaleHelper._getLocaleObj(ibmConfig.displayLocale);
        if (gadgetLocaleObject && gadgetLocaleObject.country) {
            shindig.container.setCountry(gadgetLocaleObject.country);
        }
        if (gadgetLocaleObject && gadgetLocaleObject.language) {
            shindig.container.setLanguage(gadgetLocaleObject.language);
        }
        shindig.container.addGadget(gadget0);
        shindig.container.setView('canvas');
        shindig.container.renderGadget(gadget0);
        
        gadget0.mumReloadGadget = dojo.hitch(gadget0, function(payload) { // gadgets event handler for move events
            if (this.userPrefs && this.userPrefs._mum_gadget_ID_) {
                var mumGadgetId = this.userPrefs._mum_gadget_ID_;
                // checks if the event is related to this gadget instance by comparing the widget/gadget ID with the payload ID
                if (mumGadgetId.value && mumGadgetId.value == payload) {
                    // render gadget again
                    com.ibm.mm.enabler.opensocial.OpenSocial.removeGadgetInstance(payload);
                    com.ibm.mm.enabler.opensocial.OpenSocial.renderGadget(payload, this.specUrl);
                    
                    // unsubscribe to event widgetMove event
                    var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
                    if (eventService) {
                        eventService.unsubscribeEvent(this.gadgetMovedEventId);
                    }
                    
                }
            }
        });
        
        gadget0.removeHubInstance = dojo.hitch(gadget0, function(payload) { // gadgets event handler for page switch events			
            // remove Hub instance
            com.ibm.mm.enabler.opensocial.OpenSocial.removeHubInstace(this.getIframeId()); // remove the hub container for the gadget
            // unsubscribe to event switchPage event
            var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
            if (eventService) {
                eventService.unsubscribeEvent(this.switchPageEventId);
            }
        });
        
        gadget0.removeHubInstanceLayoutChange = dojo.hitch(gadget0, function(payload) { // gadgets event handler for page switch events			
            if (!payload) { //payload is undefined, which  means layout change for all widgets/gadgets is to be saved, so remove hub instance 
                // remove Hub instance
                com.ibm.mm.enabler.opensocial.OpenSocial.removeHubInstace(this.getIframeId()); // remove the hub container for the gadget				
                // unsubscribe to event switchPage event
                var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
                if (eventService) {
                    eventService.unsubscribeEvent(this.layoutRefreshEventId);
                }
            }
        });
        
        gadget0.changeStateHandler = dojo.hitch(gadget0, function(payload) {
            if (this.userPrefs && this.userPrefs._mum_gadget_ID_) {
                var mumGadgetId = this.userPrefs._mum_gadget_ID_;
                if (mumGadgetId.value && payload.id) {
                    // remove gadget if widget changed is this instance OR if newState=maximize
                    if (mumGadgetId.value == payload.id || (payload.newWindowState && payload.newWindowState == "maximize")) {
                        com.ibm.mm.enabler.opensocial.OpenSocial.removeGadgetDomElements(mumGadgetId.value);
                        
                        // unsubscribe to event windowStateChanged event
                        var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
                        if (eventService) {
                            eventService.unsubscribeEvent(this.windowStateChangeActionId);
                        }
                    }
                }
            }
        });
        
        gadget0.stateChangedHandler = dojo.hitch(gadget0, function(payload) {
            if (this.userPrefs && this.userPrefs._mum_gadget_ID_) {
                var mumGadgetId = this.userPrefs._mum_gadget_ID_;
                if (mumGadgetId.value && payload.id) {
                    // re-render this gadget if changed gadget is this one OR oldState=maximize
                    if (mumGadgetId.value == payload.id || (payload.oldWindowState && payload.oldWindowState == "maximize")) {
                        // re-render gadget
                        com.ibm.mm.enabler.opensocial.OpenSocial.renderGadget(mumGadgetId.value, this.specUrl);
                        // unsubscribe to event windowStateChangedEventId event
                        var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
                        if (eventService) {
                            eventService.unsubscribeEvent(this.windowStateChangedEventId);
                        }
                    }
                }
            }
        });
        
        var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
        // subscribes to widget move events to handle Move and Hide actions 
        gadget0.gadgetMovedEventId = eventService.subscribeEvent("com.ibm.mashups.builder.widgetMoved", gadget0, "mumReloadGadget");
        // subscribe to page switch event to remove the corresponding OAHub container instance for the gadget  	
        gadget0.switchPageEventId = eventService.subscribeEvent("com.ibm.mashups.builder.switchPage", gadget0, "removeHubInstance");
        // subscribe to layout refresh to remove the corresponding OAHub container instance for the gadget when edited page is saved
        gadget0.layoutRefreshEventId = eventService.subscribeEvent("com.ibm.mashups.builder.layoutRefresh", gadget0, "removeHubInstanceLayoutChange");
        // subscribe to changewidgetwindowstate to remove the corresponding OAHub container instance for the gadget on a maximize/minimize/restore
        gadget0.windowStateChangeActionId = eventService.subscribeEvent("com.ibm.mashups.iwidget.changewidgetwindowstate", gadget0, "changeStateHandler");
        // subscribe to widgetwindowstatechanged to re-render the gadget on a minimize/restore
        gadget0.windowStateChangedEventId = eventService.subscribeEvent("com.ibm.mashups.iwidget.widgetwindowstatechanged", gadget0, "stateChangedHandler");
        
        // eventService.subscribeEvent("com.ibm.mashups.builder.widgetResized", gadget0, "widgetResizedHandler");
        //eventService.subscribeEvent(com.ibm.mashups.builder.IEvent.WIDGET_DELETED, this, "deleteGadgetObj");		
    },
	_query:function(controlId){
		var layoutNode = dojo.byId(controlId);
		if (layoutNode != null && dojo.hasClass(layoutNode, "iw-Standalone")) {
			return []; // unmanaged such as system widget or context menu widget, so no need to update title
		}
		
		var layoutNodes = dojo.query("#layoutContainers > .component-container > .id-"+ controlId);//check from visible layout container
		if (!!layoutNodes && layoutNodes.length>0) {
			return layoutNodes;
		}
		layoutNodes = dojo.query("#layoutContainers > .hiddenWidgetsDiv > .component-container > .id-" +controlId);//check from hidden container
		if (!!layoutNodes && layoutNodes.length>0) {
			return layoutNodes;
		}
		return dojo.query(".id-" + controlId);//unusual layout 
	},	
	_updateTitle: function(gadgetId) {
	    var pageID = com.ibm.mm.enabler.utils.Model.getCurrentPageID();
	    var modelID = com.ibm.mm.iwidget.Utils.getModelID(gadgetId);
	
		var layoutNode, elem;
		var layoutNodes =  this._query(modelID);
		if (layoutNodes.length !== 1) {
			return;
		}
		layoutNode = layoutNodes[0];
		
		var elems = dojo.query("." + com.ibm.mashups.enabler.runtime.skin.Constants.DYNAMIC_CONTENT_TITLE, layoutNode);
		if (elems.length !== 1) {
			return;
		}
		elem = elems[0];
	
		var title = null;
        var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var currentPage = navModel.find(pageID).start();
		if (navModel.hasLayoutModel(currentPage, true)) {
			// xml
			var layoutModel = navModel.getLayoutModel(currentPage);
			var currentLayoutControl = layoutModel.find(modelID).start();
			// gadgets do not have translated titles in their definition, so should always have a
			// title in the control since the titles are forced in from the catalog for gadgets
			title = currentLayoutControl.getTitle(dojo.locale);
		} else {
			// json
		    if (!((ibmConfig) && (ibmConfig.json) && (ibmConfig.json["resource-service"]))) {
				return;	    	
		    }
	    
			var provider = com.ibm.mm.enabler.persistence.JsonPersistenceProvider;
			if (!provider) {
				return;
			}

			var _this = this;	
			provider.getWidgetData(pageID, modelID, function (result, status) {
				if (status == 200) {
					var widgetData = result;
		
					if (widgetData) {
						// gadgets do not have translated titles in their definition, so should always have a
						// title in the widgetData since the titles are forced in from the catalog for gadgets
						if (widgetData.title !== null && widgetData.title !== undefined) {
							// if there is a customized title passed from the server, then use it
							title = widgetData.title;
						}
					}
				}
				else {
				}
			});		
		}
		
		if (title !== null) {
			elem.innerHTML = title;
		}
	},
    removeGadgetDomElements: function(mumId) {
        var gadgetInstance = null;
        var gadget0 = this.getGadgetObject(mumId);
        if (gadget0 != null) {
            gadgetInstance = gadget0;
            // remove Hub instance
            this.removeHubInstace(gadget0.getIframeId());
        }
        // remove Gadget markup from DOM
        var rootDomNodes = dojo.query("#" + mumId);
        var _gadget_chrome_instance = dojo.query("#" + mumId + " > .gadgets-gadget-chrome");
        if (rootDomNodes.length > 0 && _gadget_chrome_instance.length > 0) {
            if (_gadget_chrome_instance[0].parentNode == rootDomNodes[0]) { // check if it is not drag and drop
                rootDomNodes[0].removeChild(_gadget_chrome_instance[0]);
            }
        }
        return gadgetInstance;
    },
    removeGadgetInstance: function(mumId) {
        var gadgetInstance = this.removeGadgetDomElements(mumId);
        if (gadgetInstance && gadgetInstance.userPrefs && gadgetInstance.userPrefs._mum_gadget_ID_) {
            gadgetInstance.userPrefs._mum_gadget_ID_ = "";
            // TBD remove gadget from shindig container when a public api is available
        }
    },
    getGadgetUserPreferences: function(gadgetId, userProperties) {
        var widgetModel = com.ibm.mashups.enabler.widget.Factory.getWidgetModel();
        var layoutModel = com.ibm.mm.enabler.utils.Model.getCurrentPageLayoutModel();
        var modelID = com.ibm.mm.iwidget.Utils.getModelID(gadgetId);
        var controlIdentifier = new com.ibm.mm.enabler.IdentifierImpl(modelID);
        var layoutControl = layoutModel.find(controlIdentifier).start();
        var myWidgetWindow = widgetModel.getWidgetWindow(layoutControl).start();
        // read widget definition user attributes and save default values on array userProperties
        var widgetDefPreferences = widgetModel.getHierarchicalPreferences(myWidgetWindow);
        if (widgetDefPreferences !== null) {
            widgetDefPreferences = widgetDefPreferences.start();
            if (widgetDefPreferences !== null) {
                var defNames = widgetDefPreferences.getNames();
                for (i = 0; i < defNames.length; ++i) {
                    userProperties[defNames[i]] = {
                        value: widgetDefPreferences.getValue(defNames[i])
                    };
                }
            }
        }
    },
    resizeGadget: function(payload) {
        // TBD - needs to be rewritten 
        var gadgetNodes = dojo.query(".gadgets-gadget", dojo.byId(gadgetId));
        if (gadgetNodes.length > 0) {
            if (gadgetNodes[0].tagName == "IFRAME") {
                gadgetNodes[0].style.height = (payload.newHeight - 40) + "px";
                gadgetNodes[0].style.width = (payload.newWidth - 30) + "px";
            }
        }
    },
    getGadgetObject: function(mumId) {
        var gadgetObj = null;
        if (shindig.container.gadgets_) {
            // iterates through gadgets and searches for the gadget instance with mum gadget id = mumId
            for (var gadgetInstance in shindig.container.gadgets_) {
                if (shindig.container.gadgets_[gadgetInstance] && shindig.container.gadgets_[gadgetInstance].userPrefs && shindig.container.gadgets_[gadgetInstance].userPrefs._mum_gadget_ID_) {
                    var mumGadgetId = shindig.container.gadgets_[gadgetInstance].userPrefs._mum_gadget_ID_;
                    if (mumGadgetId.value && mumGadgetId.value == mumId) {
                        gadgetObj = shindig.container.gadgets_[gadgetInstance];
                        break;
                    }
                }
            }
        }
        return gadgetObj;
    },
    onDemandLoad_OpenSocialLibs: function() {
        if (typeof shindig == "undefined") { // check if loading of shindig files is required
            var args = {
                url: ibmCfg.enablerConfig["com.ibm.mashups.contextroot.opensocial"] + "/gadgets/js/shindig-container:pubsub-2.js?c=1&debug=0",
                sync: true,
                load: function(data, ioArgs) {
                    //status is 0 if it's a local file system resource
                    if (ioArgs.xhr.status == 200 || ioArgs.xhr.status === 0) {
                        if (window.execScript) {
                            window.execScript(ioArgs.xhr.responseText, "JavaScript");
                        }
                        else {
                            rc = dojo.eval(ioArgs.xhr.responseText);
                        }
                    }
                },
                error: function(data, ioArgs) {
                    this.LOGGER.severe("tagservices.osgadgets.processTag", "OpenSocial resource failed to load." + ioArgs); // TBD-create new message
                }
            };
            dojo.xhrGet(args); // loads shindig js
            // args.url = ibmCfg.enablerConfig["com.ibm.mashups.contextroot.enabler"]+"/js/com/ibm/mm/enabler/opensocial.js";
            // dojo.xhrGet(args);
            // loads opensocial.js
            this.initOpenSocial();
        }
    },
    initOpenSocial: function() {
        // initializes pubsub2router variable with reference to current OAHub instance
        var managedhub = com.ibm.mm.enabler.hub.ManagedHubImpl.getInstance();
        gadgets.pubsub2router.init({
            hub: managedhub
        });
        gadgets.rpc.register('set_pref', com.ibm.mm.enabler.opensocial.OpenSocial.mashupsGadgets_SetUserPref);
        shindig.DefaultUserPrefStore.prototype.savePrefs = com.ibm.mm.enabler.opensocial.OpenSocial.openSocial_SavePrefs;
    },
    removeHubInstace: function(iFrameId) {
        // remove Hub instance
        var hubContainer = com.ibm.mm.enabler.hub.ManagedHubImpl.getInstance().getContainer(iFrameId);
        if (hubContainer != null) {
            var hubInstance = com.ibm.mm.enabler.hub.ManagedHubImpl.getInstance();
            hubInstance.removeContainer(hubContainer);
        }
    },
    openSocial_SavePrefs: function(gadget) {
        // save user preferences in Mashups DB
        var gadgetId = null;
        if (gadget && gadget.userPrefs && gadget.userPrefs._mum_gadget_ID_ && gadget.userPrefs._mum_gadget_ID_.value) {
            gadgetId = gadget.userPrefs._mum_gadget_ID_.value;
        }
        else {
            return false;
        }
        // prepare variables to get user attributes
        var widgetModel = com.ibm.mashups.enabler.widget.Factory.getWidgetModel();
        var layoutModel = com.ibm.mm.enabler.utils.Model.getCurrentPageLayoutModel();
        var modelID = com.ibm.mm.iwidget.Utils.getModelID(gadgetId);
        var controlIdentifier = new com.ibm.mm.enabler.IdentifierImpl(modelID);
        var layoutControl = layoutModel.find(controlIdentifier).start();
        var myWidgetWindow = widgetModel.getWidgetWindow(layoutControl).start();
        
        // read widget definition user attributes and save default values on array userProperties
        var widgetDefPreferences = widgetModel.getHierarchicalPreferences(myWidgetWindow);
        if (widgetDefPreferences !== null) {
            widgetDefPreferences = widgetDefPreferences.start();
            var gadgetPrefs = gadget.userPrefs;
            for (var name in gadgetPrefs) {
                if (name !== "_mum_gadget_ID_") {
                    widgetDefPreferences.setValue(name, gadgetPrefs[name].value);
                }
            }
            widgetModel.commit().start();
        }
    },
    mashupsGadgets_SetUserPref: function(editToken, name, value) {
        // workaround for shindig defect - override this function until it gets fixed in shindig 2.0 stream
        // the override takes place in tagservices.osgadgets.initOpenSocial()
        var id = shindig.container.gadgetService.getGadgetIdFromModuleId(this.f);
        var gadget = shindig.container.getGadget(id);
        for (var i = 1, j = arguments.length; i < j; i += 2) {
            if (!gadget.userPrefs[arguments[i]]) {
                gadget.userPrefs[arguments[i]] = {
                    value: arguments[i + 1]
                };
            }
            else {
                gadget.userPrefs[arguments[i]].value = arguments[i + 1];
            }
        }
        gadget.saveUserPrefs();
    }
});

// singleton
com.ibm.mm.enabler.opensocial.OpenSocial = new com.ibm.mm.enabler.opensocial();

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.SpaceNodeXmlImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.SpaceNodeXmlImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.persistence.xml.SpaceNodeXmlImpl");




























dojo.declare("com.ibm.mm.enabler.space.persistence.xml.SpaceNodeXmlImpl", [com.ibm.mm.enabler.persistence.xml.XmlPersistenceImpl, com.ibm.mm.enabler.persistence.xml.IdentifiableXmlImpl, com.ibm.mm.enabler.persistence.xml.LocalizedXmlImpl, com.ibm.mm.enabler.persistence.xml.DefaultLocalizedXmlImpl, com.ibm.mm.enabler.MetaDataImpl, com.ibm.mm.enabler.persistence.xml.EditLinkProviderXmlImpl, com.ibm.mm.enabler.TransformableImpl, com.ibm.mm.enabler.space.persistence.meta.SpaceNodeMetaImpl, com.ibm.mm.enabler.DirtyFlagProviderImpl, com.ibm.mm.enabler.ac.OwnerAspect, com.ibm.mm.enabler.space.SpaceExtensionAspect], {
    constructor: function(node){
        // node is taken care of in the XmlPersistenceImpl
        this.owner = null;
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_SPACE);
        this.prefix = this.serviceJson.idprefix;
        this.nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, this.nsf.getNameSpaces([this.nsf.NS_ATOM, this.nsf.NS_CA]));
        
        // Override LocalizedImpl to handle composite app titles and descriptions
        this.li_ns = dojo.mixin(this.nsf.getNameSpaces([this.nsf.NS_ATOM, this.nsf.NS_XML, this.nsf.NS_CA]), this.serviceJson.namespaces);
        
        // base
        this.BASE_NLS_TITLE_STRING = "ca:title";
        this.BASE_NLS_DESCRIPTION_STRING = "ca:description";
        this.BASE_NS = this.li_ns.ca;
        
        // title
        this.TITLE_ELEMENT = "atom:content/ca:application";
        this.TITLE = this.TITLE_ELEMENT + "/" + this.BASE_NLS_TITLE_STRING;
        
        // description
        this.DESCRIPTION_ELEMENT = "atom:content/ca:application";
        this.DESCRIPTION = this.DESCRIPTION_ELEMENT + "/" + this.BASE_NLS_DESCRIPTION_STRING;
        
        // track changed properties(old values) in space: title, description, metaData, owner
        this.changedProperties = {};
        this.sourceSpaceID = null;
        this.latestLocale = "en";
    },
    getOwner: function(){
        if (!this.owner) {
            var expr = "atom:content/ca:application/ca:owner";
            var owner = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
            if (owner) {
                var um = com.ibm.mashups.enabler.user.Factory.getUserModel();
                var ownerID = com.ibm.mm.enabler.utils.Dom.getAttributeWithNS(owner, "ca:id", "id", this.ns.ca);
                this.owner = um.find(ownerID).start();
            }
        }
        return this.owner;
    },
    getOwnerDeferred: function(){
        return new com.ibm.mm.enabler.DeferredImpl(this, function(deferred, sync){
            if (!this.owner) {
                var expr = "atom:content/ca:application/ca:owner";
                var owner = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
                if (owner) {
                    var um = com.ibm.mashups.enabler.user.Factory.getUserModel();
                    var ownerID = com.ibm.mm.enabler.utils.Dom.getAttributeWithNS(owner, "ca:id", "id", this.ns.ca);
                    um.find(ownerID).setFinishedCallback(dojo.hitch(this, function(result, status, parameter){
                        if (status == com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK) {
                            this.owner = result;
                        }
                        deferred.finish(result, status);
                    })).start(sync);
                }
                else {
                    deferred.finish(null, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_NOT_FOUND);
                }
            }
            else {
                deferred.finish(this.owner, com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK);
            }
        });
    },
    setOwner: function(user){
        // track old owner for update
        if (!this.changedProperties.owner) {
            this.changedProperties.owner = this.getOwner();
        }
        
        this.owner = user;
        var expr = "atom:content/ca:application/ca:owner";
        var owner = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        if (owner) {
            com.ibm.mm.enabler.utils.Utils.setAttributeWithNS(owner, "ca:id", this.ns.ca, user.getID());
        }
        else {
            expr = "atom:content/ca:application";
            var application = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
            if (application) {
                // create owner  element
                var tn = com.ibm.mm.enabler.utils.Dom.createElement(this.xmlData.ownerDocument, "ca:owner", this.ns.ca);
                com.ibm.mm.enabler.utils.Utils.setAttributeWithNS(tn, "ca:id", this.ns.ca, user.getID());
                application.appendChild(tn);
            }
        }
        this.setDirty();
    },
    
    getExportSpaceURL: function(){
        var expr = "atom:link[@ca:rel='application-export']";
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        return node ? node.getAttribute("href") : null;
    },
    
    getExportTemplateURL: function(){
        var expr = "atom:link[@ca:rel='template-export']";
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        return node ? node.getAttribute("href") : null;
    },
    
    setHidden: function(hide){
        var name = this._getHiddenMetaNameDynamic();
        this.setMetaData(name, (hide) ? "true" : "false");
    },
    
    confirmSetHidden: function(hide){
        var ret1 = this.confirmSetMetaData(this._getHiddenMetaNameFixed(), (hide) ? "true" : "false");
        var ret2 = this.confirmSetMetaData(this._getHiddenMetaNameDynamic(), (hide) ? "true" : "false");
        return (ret1 && ret2);
    },
    
    getRootNavigation: function(){
        var result;
        var expr = "atom:link[@ca:rel='navigation']";
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(expr, this.xmlData, this.ns);
        if (node) {
            var id = com.ibm.mm.enabler.utils.Utils.getIdFromExtUri(this.prefix, node);
            var href = node.getAttribute("href");
            result = new com.ibm.mm.enabler.IdentifierImpl(id, {
                "href": href
            });
        }
        return result || null;
    },
    
    /**
     * If template will use current titles/descriptions of this space. Update them if needed.
     */
    saveAsTemplate: function(){
        return new com.ibm.mm.enabler.DeferredOperationImpl(this, this._saveAsTemplate);
    },
    
    _saveAsTemplate: function(deferred, sync){
        //POST to template:collection with feed containing space id
        var entry = com.ibm.mm.enabler.utils.Utils.createNode("atom:entry", this.ns.atom);
        entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:id", this.ns.atom));
        entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:title", this.ns.atom));
        
        var updated = com.ibm.mm.enabler.utils.Utils.createNode("atom:updated", this.ns.atom);
        var time = new Date();
        com.ibm.mm.enabler.utils.Dom.textContent(updated, time.toGMTString());
        entry.appendChild(updated);
        
        var content = com.ibm.mm.enabler.utils.Utils.createNode("atom:content", this.ns.atom);
        content.setAttribute("type", "application/xml");
        
        var template = com.ibm.mm.enabler.utils.Utils.createNode("ca:template", this.ns.ca);
        com.ibm.mm.enabler.utils.Utils.setAttributeWithNS(template, "creation-context:app-id", this.ns["creation-context"], this.getID());
        
        var locales = this.getLocales();
        for (var i = 0, l = locales.length; i < l; i++) {
            var title = com.ibm.mm.enabler.utils.Utils.createNode("ca:title", this.ns.ca);
            title.setAttribute("xml:lang", locales[i]);
            com.ibm.mm.enabler.utils.Dom.textContent(title, this.getTitle(locales[i]));
            template.appendChild(title);
            
            var description = com.ibm.mm.enabler.utils.Utils.createNode("ca:description", this.ns.ca);
            description.setAttribute("xml:lang", locales[i]);
            com.ibm.mm.enabler.utils.Dom.textContent(description, this.getDescription(locales[i]));
            template.appendChild(description);
        }
        
        content.appendChild(template);
        entry.appendChild(content);
        
        var templateServiceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_TEMPLATE);
        var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(templateServiceJson.url, this.spaceModel);
        var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
        serviceReq.create(com.ibm.mm.enabler.utils.Dom.innerXML(entry), dojo.hitch(this, function(type, data, xhr, args){
            // public callback handling
            if (deferred && deferred.operationCallback) {
                deferred.operationCallback(entry, com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE, xhr.status, deferred.operationCallbackParameters);
            }
            
            // internal callback handling
            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                if (deferred.errorCallback) {
                    deferred.errorCallback(xhr.status, deferred.errorCallbackParameters);
                }
            }
        }));
    },
    
    setTitle: function(title, locale){
        this.latestLocale = locale; // remember locale and use it for life cycle callback
        if (!this.changedProperties.title) {
            this.changedProperties.title = {};
        }
        if (!this.changedProperties.title[locale]) {
            this.changedProperties.title[locale] = this.getTitle(locale); // save current
        }
        // call super
        this.inherited(arguments);
    },
    
    removeTitle: function(locale){
        if (!this.changedProperties.title) {
            this.changedProperties.title = {};
        }
        if (!this.changedProperties.title[locale]) {
            this.changedProperties.title[locale] = this.getTitle(locale); // save current
        }
        // call super
        this.inherited(arguments);
    },
    
    removeTitles: function(){
        if (!this.changedProperties.title) {
            this.changedProperties.title = {};
        }
        // save all current titles
        var locales = this.getLocales();
        for (var locale in locales) {
            if (!this.changedProperties.title[locale]) {
                this.changedProperties.title[locale] = this.getTitle(locale);
            }
        }
        // call super
        this.inherited(arguments);
    },
    
    setDescription: function(desc, locale){
        if (!this.changedProperties.description) {
            this.changedProperties.description = {};
        }
        if (!this.changedProperties.description[locale]) {
            this.changedProperties.description[locale] = this.getDescription(locale); // save current
        }
        // call super
        this.inherited(arguments);
    },
    
    removeDescription: function(locale){
        if (!this.changedProperties.description) {
            this.changedProperties.description = {};
        }
        if (!this.changedProperties.description[locale]) {
            this.changedProperties.description[locale] = this.getDescription(locale); // save current           
        }
        // call super
        this.inherited(arguments);
    },
    
    removeDescriptions: function(){
        if (!this.changedProperties.description) {
            this.changedProperties.description = {};
        }
        // save all current descriptions
        var locales = this.getLocales();
        for (var locale in locales) {
            if (!this.changedProperties.description[locale]) {
                this.changedProperties.description[locale] = this.getDescription(locale);
            }
        }
        // call super
        this.inherited(arguments);
    },
    
    setMetaData: function(name, value){
        if (!this.changedProperties.metadata) {
            this.changedProperties.metadata = {};
        }
        if (!this.changedProperties.metadata[name]) {
            this.changedProperties.metadata[name] = this.getMetaData(name); // save current
        }
        // call super
        this.inherited(arguments);
    },
    
    removeMetaData: function(name){
        if (!this.changedProperties.metadata) {
            this.changedProperties.metadata = {};
        }
        if (!this.changedProperties.metadata[name]) {
            this.changedProperties.metadata[name] = this.getMetaData(name); // save current
        }
        // call super
        this.inherited(arguments);
    },
    
    isFavorite: function(){
        if (this.isFavoriteChanged()) {
            return this.favoriteOverwrite;
        }
        var ret = com.ibm.mashups.enabler.xml.XPath.evaluateBoolean("atom:link[@ca:rel='favorite']", this.xmlData, this.ns);
        return ret;
    },
    
    setFavorite: function(favorite){
        this.favoriteOverwrite = favorite;
        // now check if we can reset ourselves
        var ret = com.ibm.mashups.enabler.xml.XPath.evaluateBoolean("atom:link[@ca:rel='favorite']", this.xmlData, this.ns);
        if (ret == this.favoriteOverwrite) {
            delete this.favoriteOverwrite;
        }
    },
    
    isFavoriteChanged: function(){
        return (typeof this.favoriteOverwrite != "undefined");
    },
    
    // this one is used by the xml model
    _favoriteUpdateCommited: function(url){
        if (this.isFavoriteChanged()) {
            var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:link[@ca:rel='favorite']", this.xmlData, this.ns);
            if (this.favoriteOverwrite) {
                if (!node) {
                    var linkNode = com.ibm.mm.enabler.utils.Utils.createLinkNode(url, "related", this.nsf.getNameSpaceUri(this.nsf.NS_ATOM));
                    com.ibm.mm.enabler.utils.Utils.setAttributeWithNS(linkNode, "ca:rel", this.nsf.getNameSpaceUri(this.nsf.NS_CA), "favorite");
                    this.xmlData.appendChild(linkNode);
                }
            }
            else {
                if (node) {
                    this.xmlData.removeChild(node);
                }
            }
            delete this.favoriteOverwrite;
        }
    },
    
    getSpaceExtensionURL: function(){
        return this.getMetaData("{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.space.extension_url");
    },
    
    _setCopiedFrom: function(sourceSpaceID){
        this.sourceSpaceID = sourceSpaceID;
    },
    
    _getCopiedFrom: function(){
        return this.sourceSpaceID;
    },
    
    _diffSpaceChanges: function(){
        var delta = {};
        var locale;
        // owner
        if ("owner" in this.changedProperties) {
            var oldID = (!this.changedProperties.owner) ? "" : this.changedProperties.owner.getID();
            var newOwner = this.getOwner();
            var newID = (!newOwner) ? "" : newOwner.getID();
            if (newID != oldID) {
                delta.owner = newID;
            }
        }
        
        // title
        if ("title" in this.changedProperties) {
            for (locale in this.changedProperties.title) {
                if (Object.prototype.hasOwnProperty.call(this.changedProperties.title, locale)) {
                    var newTitle = this.getTitle(locale);
                    if (newTitle != this.changedProperties.title[locale]) {
                        if (!delta.title) {
                            delta.title = {};
                        }
                        delta.title[locale] = newTitle;
                    }
                }
            }
        }
        
        // description
        if ("description" in this.changedProperties) {
            for (locale in this.changedProperties.description) {
                if (Object.prototype.hasOwnProperty.call(this.changedProperties.description, locale)) {
                    var newDescription = this.getDescription(locale);
                    if (newDescription != this.changedProperties.description[locale]) {
                        if (!delta.description) {
                            delta.description = {};
                        }
                        delta.description[locale] = newDescription;
                    }
                }
            }
        }
        
        // metadata
        if ("metadata" in this.changedProperties) {
            for (var name in this.changedProperties.metadata) {
                if (Object.prototype.hasOwnProperty.call(this.changedProperties.metadata, name)) {
                    var value = this.getMetaData(name);
                    if (value != this.changedProperties.metadata[name]) {
                        if (!delta.metadata) {
                            delta.metadata = {};
                        }
                        delta.metadata[name] = value;
                    }
                }
            }
        }
        
        // finished, clear changed properties.
        this.changedProperties = {};
        return delta;
    },
    
    _getFavoriteURL: function(){
        var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:link[@ca:rel='favorite']", this.xmlData, this.ns);
        return node ? node.getAttribute("href") : null;
    }
    
});

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.FactoryImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.FactoryImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.persistence.xml.FactoryImpl");



dojo.declare("com.ibm.mm.enabler.space.persistence.xml.FactoryImpl", null, {

    createSpaceNodePersistence: function(data){
        // create persistence implementation
        return new com.ibm.mm.enabler.space.persistence.xml.SpaceNodeXmlImpl(data);
    }
});

// factory
com.ibm.mm.enabler.space.persistence.xml.Factory = new com.ibm.mm.enabler.space.persistence.xml.FactoryImpl();

}

if(!dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl"] = true;
dojo.provide("com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl");

/*
 * This is injected on the fly and not fetched through an API. Therefore we need to define the @ibm-module so that the build process is picking it up
 * @ibm-module SpaceModelXml
 */
































dojo.declare("com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl", [com.ibm.mm.enabler.model.ModelImpl, com.ibm.mm.enabler.ConfirmableModelImpl, com.ibm.mm.enabler.SubmittableFormProviderImpl, com.ibm.mm.enabler.strategy.HiddenMetaDataLoadingStrategyHelperImpl, com.ibm.mm.enabler.ServiceDocConsumer], {

    constructor: function(){
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        var LOG_METHOD = "constructor()";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD);
        }
        
        // client id counter
        this.cid = 0;
        
        this.modelMessages = dojo.i18n.getLocalization("com.ibm.mm.enabler", "modelMessages");
        this.enforcementHelper = com.ibm.mm.enabler.utils.APIEnforcementHelper;
        this.xmlDom = com.ibm.mm.enabler.utils.Dom.createDocument();
        this.spaceExtensionModel = com.ibm.mashups.enabler.space.Factory.getSpaceExtensionModel();
        this.urlType = com.ibm.mashups.enabler.model.url.ModelUrlFactory.SPACE_URL;
        this._ATOM_FEED_ENTRY = "atom:feed/atom:entry";
        
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_SPACE);
        this.prefix = this.serviceJson.idprefix;
        this.nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, this.nsf.getNameSpaces([this.nsf.NS_ATOM, this.nsf.NS_XHTML, this.nsf.NS_OPENSEARCH, this.nsf.NS_CA, this.nsf.NS_APP]));
        
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    },
    
    _initServiceDoc: function(){
        this.inherited("_initServiceDoc", arguments);
        // service document and initialization
        this.serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_SPACE);
        this.prefix = this.serviceJson.idprefix;
        this.nsf = com.ibm.mm.enabler.model.NameSpaceFactory;
        this.ns = dojo.delegate(this.serviceJson.namespaces, this.nsf.getNameSpaces([this.nsf.NS_ATOM, this.nsf.NS_XHTML, this.nsf.NS_OPENSEARCH, this.nsf.NS_CA, this.nsf.NS_APP]));
    },
    
    // singleton
    _getInstance: function(){
        var instance = com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl._instance;
        return instance ? instance : (com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl._instance = new com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl());
    },
    
    invalidate: function(){
        delete com.ibm.mm.enabler.space.persistence.xml.SpaceModelXmlImpl._instance;
    },
    
    find: function(id){
        this.enforcementHelper.enforceDefinedNotNull(id, this.modelMessages.E_PARAM_ISNULL_OR_UNDEFINED_3, ["id", this.declaredClass, "find(id)"]);
        this._initServiceDoc();
        return new com.ibm.mm.enabler.DeferredImpl(this, this._find, id);
    },
    
    _find: function(deferred, sync, _id){
        var LOG_METHOD = "_find(_id, deferred, sync)";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, [_id, deferred, sync]);
        }
        
        var id = com.ibm.mm.enabler.utils.Utils.getIdFromIdentifiable(_id);
        
        // use href info from identifier
        var myUrl = null;
        var href = com.ibm.mm.enabler.utils.Utils.getHrefFromIdentifiable(_id);
        if (href) {
            myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(href, this);
        }
        else {
            // construct url otherwise
            myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(this.urlType, this);
            myUrl.setNodes([{
                value: id,
                isID: true
            }]);
        }
        
        // include access control info
        myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
        
        // apply hidden meta data strategy
        this._applyHiddenMetaDataLoadingStrategyToModelURL(myUrl);
        
        // apply caching strategy
        this._applyNoCacheStrategy(myUrl);
        
        var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
        var retVal = null;
        serviceReq.read(dojo.hitch(this, function(type, data, xhr, args){
            var LOG_METHOD = "_find_callback(type, data, xhr, args)";
            if (bIsLoggable) {
                this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
            }
            this._initServiceDoc();
            
            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                var node = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("//atom:entry", data, this.ns);
                retVal = com.ibm.mm.enabler.space.persistence.xml.Factory.createSpaceNodePersistence(node);
            }
            
            // finished callback 
            if (deferred) {
                deferred.finish(retVal, xhr.status);
            }
            
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD);
            }
        }));
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD, retVal);
        }
        return retVal;
    },
    
    create: function(context){
        this._initServiceDoc();
        // create atom:entry
        var result = com.ibm.mm.enabler.utils.Atom.createEntry("New Space", "ca:application", this.ns.ca);
        if (context) {
            if (context.clonedSpace) {
                com.ibm.mm.enabler.utils.Dom.setAttributeWithNS(this.xmlDom, result.modelElement, "creation-context:app-id", "app-id", this.ns["creation-context"], context.clonedSpace);
            }
            else 
                if (context.templateID) {
                    result.modelElement.setAttribute("ca:template-id", context.templateID);
                }
        }
        
        var f = com.ibm.mm.enabler.space.persistence.xml.Factory;
        var space = f.createSpaceNodePersistence(result.entryElement);
        space.setID((context && context.cid) ? context.cid : "" + this.cid++);
        
        if (context && context.clonedSpace) {
            space._setCopiedFrom(context.clonedSpace);
        }
        
        space._setDirty();
        return space;
    },
    confirmCreate: function(context){
        return this.getConfirmCreate();
    },
    
    commit: function(ctx){
        return new com.ibm.mm.enabler.DeferredOperationImpl(this, function(deferred, sync){
            var LOG_METHOD = "_commit(deferred, sync)";
            var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
            if (bIsLoggable) {
                this.LOGGER.entering(LOG_METHOD, [deferred, sync]);
            }
            this._initServiceDoc();
            
            var nodes = ctx.getNodes();
            var loadedNodes = nodes.loadedNodes;
            var createdNodes = nodes.createdNodes;
            var deletedNodes = nodes.deletedNodes;
            var createdFavorites = nodes.createdFavoriteNodes;
            var deletedFavorites = nodes.deletedFavoriteNodes;
            
            // initialize callback handling
            this.statusCode = com.ibm.mm.enabler.model.HttpStatusCodes.HTTP_OK;
            this.requestCount = 0;
            this._acquire();
            
            var space, extension, result, myUrl, serviceReq, entry, link, id;
            var failedNodes = {};
            
            // HTTP PUT updated nodes
            for (id in loadedNodes) {
                if (Object.prototype.hasOwnProperty.call(loadedNodes, id)) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Checking if loaded space ${0} is modified", id);
                    }
                    space = loadedNodes[id];
                    if (space.isDirty()) {
                        if (bIsLoggable) {
                            this.LOGGER.trace(LOG_METHOD, "Space is modified, updating");
                        }
                        // onUpdateSpace event
                        extension = this.spaceExtensionModel.findExtensionBySpace(space).start();
                        if (extension) {
                            result = extension.onSpaceUpdate(space.getID(), dojo.toJson(space._diffSpaceChanges(), true), null);
                            if (result && result.getState() != "OK") {
                                if (bIsLoggable) {
                                    this.LOGGER.trace(LOG_METHOD, "Space update aborted: " + space.getID());
                                    this.LOGGER.trace(LOG_METHOD, "Space extension update problem: " + result.getLocaleMessage());
                                    this.LOGGER.trace(LOG_METHOD, "Solution: " + result.getLocaleSolution());
                                }
                                
                                continue;
                            }
                        }
                        
                        entry = com.ibm.mm.enabler.utils.Utils.createFeed("space:" + this.prefix + space.getID(), "IBM Lotus Mashups Space Feed", loadedNodes[id], this.ns);
                        
                        // now let's create the url
                        myUrl = new com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(space.getEditLink(), this);
                        // include access control info
                        myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
                        
                        serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, true);
                        this._acquire();
                        serviceReq.update(entry, dojo.hitch(this, function(type, data, xhr, args){
                            if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_ERROR) {
                                if (bIsLoggable) {
                                    this.LOGGER.warning(LOG_METHOD, this.modelMessages.E_XHR_REQUEST_FAILED_2, [xhr, data || "n/a"]);
                                }
                            }
                            else {
                                var updatedEntry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry(this._ATOM_FEED_ENTRY, data, this.ns);
                                var id = space.getID();
                                if (updatedEntry) {
                                    // update space node xml data
                                    loadedNodes[id].setXml(updatedEntry);
                                    ctx.nodeUpdated(loadedNodes[id]);
                                }
                                else {
                                    // if there is no entry in the response, then remove the space node
                                    // this might happen, if someone set a new owner and removes the read permissions of the old owner
                                    ctx.nodeDeleted(id);
                                }
                                // operation callback on base deferred
                                if (deferred && deferred.operationCallback) {
                                    deferred.operationCallback(loadedNodes[id], com.ibm.mashups.enabler.DeferredOperation.OPERATION_MODIFY, xhr.status, deferred.operationCallbackParameters);
                                }
                                this._release(deferred, xhr.status);
                            }
                        })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                    }
                }
            }
            
            // HTTP POST created nodes
            for (var count in createdNodes) {
                if (Object.prototype.hasOwnProperty.call(createdNodes, count)) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Processing new space");
                    }
                    space = createdNodes[count];
                    var oldid = space.getID();
                    var sourceSpaceID = space._getCopiedFrom(); // not null, then onSpaceCopy even
                    var latestLocale = space.latestLocale;
                    if (sourceSpaceID) { // see if extension allow it to be copied
                        extension = this.spaceExtensionModel.findExtensionBySpace(sourceSpaceID).start();
                        if (extension) {
                            result = extension.canCopySpace(space.getID(), null);
                            if (result && result.getState() != "OK") {
                                if (bIsLoggable) {
                                    this.LOGGER.trace(LOG_METHOD, "Space copy aborted: " + sourceSpaceID);
                                    this.LOGGER.trace(LOG_METHOD, "Space extension does not allow to copy this space.");
                                }
                                
                                continue;
                            }
                        }
                    }
                    
                    // now let's create the url
                    myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.serviceJson.url, this);
                    // include access control info
                    myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
                    
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, true);
                    this._acquire();
                    serviceReq.create(space.toString(), dojo.hitch(this, function(type, data, xhr, args){
                        var LOG_METHOD = "_commit_create_callback(type, data, xhr, args)";
                        if (bIsLoggable) {
                            this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
                        }
                        var newentry = null;
                        if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                            var entry = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("//atom:entry", data, this.ns);
                            if (entry) {
                                newentry = com.ibm.mm.enabler.space.persistence.xml.Factory.createSpaceNodePersistence(entry);
                                ctx.nodeCreated(oldid, newentry);
                                // now we need to update the favorite entry with the cid
                                if (oldid in createdFavorites) {
                                    createdFavorites[newentry.getID()] = newentry;
                                    // we need to transfer the favorite setting between enodes so that it is correctly setup in the in memory XML
                                    if (createdFavorites[oldid].isFavoriteChanged()) {
                                        newentry.setFavorite(createdFavorites[oldid].isFavorite());
                                    }
                                    delete createdFavorites[oldid];
                                }
                                
                                // onCopyspace or onCreateSpace
                                extension = this.spaceExtensionModel.findExtensionBySpace(newentry).start();
                                if (extension) {
                                    var userLocale = (!latestLocale) ? "en" : latestLocale;
                                    var id = newentry.getID();
                                    var title = newentry.getTitle(userLocale);
                                    var description = newentry.getDescription(userLocale);
                                    var sourceSpaceID = args;
                                    result = sourceSpaceID ? extension.onSpaceCopy(sourceSpaceID, id, title, description, null) : extension.onSpaceCreate(id, title, description, null);
                                    if (result && result.getState() != "OK") {
                                        if (bIsLoggable) {
                                            this.LOGGER.trace(LOG_METHOD, "Space create/copy aborted: " + id);
                                            this.LOGGER.trace(LOG_METHOD, "Space extension update problem: " + result.getLocaleMessage());
                                            this.LOGGER.trace(LOG_METHOD, "Solution: " + result.getLocaleSolution());
                                        }
                                        
                                        failedNodes[id] = newentry;
                                    }
                                }
                            }
                        }
                        
                        // public callback handling
                        if (deferred && deferred.operationCallback) {
                            deferred.operationCallback(newentry, com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE, xhr.status, deferred.operationCallbackParameters);
                        }
                        
                        this._release(deferred, xhr.status);
                        
                        if (bIsLoggable) {
                            this.LOGGER.exiting(LOG_METHOD);
                        }
                    }), sourceSpaceID); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
            
            // Deleted those node failed during copy or create events, no onSpaceDelete event will be fired for them.
            for (var failedID in failedNodes) {
                if (Object.prototype.hasOwnProperty.call(failedNodes, failedID)) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Processing delete of space ${0}", id);
                    }
                    myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(failedNodes[failedID].getEditLink(), this);
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                    this._acquire();
                    serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args){
                        var LOG_METHOD = "_commit_remove_callback(type, data, xhr, args)";
                        if (bIsLoggable) {
                            this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
                        }
                        
                        // finished callback
                        if (deferred && deferred.operationCallback) {
                            deferred.operationCallback(null, com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE, xhr.status, deferred.operationCallbackParameters);
                        }
                        
                        this._release(deferred, xhr.status);
                        
                        if (bIsLoggable) {
                            this.LOGGER.exiting(LOG_METHOD, [type, data, xhr, args]);
                        }
                    })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
            
            // HTTP DELETE deleted nodes
            for (id in deletedNodes) {
                if (Object.prototype.hasOwnProperty.call(deletedNodes, id)) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Processing delete of space ${0}", id);
                    }
                    // onSpaceDelete event
                    var spaceToDelete = ctx.getModelNode(loadedNodes[id]);
                    extension = this.spaceExtensionModel.findExtensionBySpace(spaceToDelete).start();
                    if (extension) {
                        var deleteID = spaceToDelete.getID();
                        result = extension.canDeleteSpace(deleteID, null);
                        if (result && result.getState() != "OK") {
                            if (bIsLoggable) {
                                this.LOGGER.trace(LOG_METHOD, "Space delete aborted: " + spaceToDelete.getID());
                                this.LOGGER.trace(LOG_METHOD, "Space extension update problem: " + result.getLocaleMessage());
                                this.LOGGER.trace(LOG_METHOD, "Solution: " + result.getLocaleSolution());
                            }
                            
                            continue;
                        }
                        
                        result = extension.onSpaceDelete(deleteID, null);
                        if (result && result.getState() != "OK") {
                            if (bIsLoggable) {
                                this.LOGGER.trace(LOG_METHOD, "Space delete aborted: " + spaceToDelete.getID());
                                this.LOGGER.trace(LOG_METHOD, "Space extension update problem: " + result.getLocaleMessage());
                                this.LOGGER.trace(LOG_METHOD, "Solution: " + result.getLocaleSolution());
                            }
                            
                            continue;
                        }
                    }
                    
                    myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(deletedNodes[id].getEditLink(), this);
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                    this._acquire();
                    serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args){
                        var LOG_METHOD = "_commit_remove_callback(type, data, xhr, args)";
                        if (bIsLoggable) {
                            this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
                        }
                        
                        var sc = com.ibm.mm.enabler.model.HttpStatusCodes;
                        if (xhr.status == sc.HTTP_NOT_FOUND || xhr.status == sc.HTTP_OK) {
                            ctx.nodeDeleted(id);
                        }
                        
                        // finished callback
                        if (deferred && deferred.operationCallback) {
                            deferred.operationCallback(null, com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE, xhr.status, deferred.operationCallbackParameters);
                        }
                        
                        this._release(deferred, xhr.status);
                        
                        if (bIsLoggable) {
                            this.LOGGER.exiting(LOG_METHOD, [type, data, xhr, args]);
                        }
                    })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
            
            // HTTP DELETE deleted favorites
            for (var id in deletedFavorites) {
                if (deletedFavorites[id]) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Processing delete of space-favorite ${0}", id);
                    }
                    myUrl = null;
                    
                    link = deletedFavorites[id]._getFavoriteURL();
                    if (typeof(link) == "string") {
                        myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(link, this);
                    }
                    else 
                        if (!link) {
                            myUrl = new com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.SPACE_FAVORITE_URL, this);
                            myUrl.setNodes([{
                                value: id,
                                isID: true
                            }]);
                        }
                        else {
                            myUrl = link;
                        }
                    
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                    this._acquire();
                    serviceReq.remove(dojo.hitch(this, function(type, data, xhr, args){
                        var LOG_METHOD = "_commit_remove_callback(type, data, xhr, args)";
                        if (bIsLoggable) {
                            this.LOGGER.entering(LOG_METHOD, [type, data, xhr, args]);
                        }
                        if (deletedFavorites[id]) {
                            deletedFavorites[id]._favoriteUpdateCommited();
                        }
                        // finished callback
                        if (deferred && deferred.operationCallback) {
                            deferred.operationCallback(null, com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE, xhr.status, deferred.operationCallbackParameters);
                        }
                        this._release(deferred, xhr.status);
                        if (bIsLoggable) {
                            this.LOGGER.exiting(LOG_METHOD, [type, data, xhr, args]);
                        }
                    })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
            
            // HTTP POST created favorites
            for (var id in createdFavorites) {
                if (Object.prototype.hasOwnProperty.call(createdFavorites, id) && createdFavorites[id]) {
                    if (bIsLoggable) {
                        this.LOGGER.trace(LOG_METHOD, "Processing create of space-favorite ${0}", id);
                    }
                    entry = com.ibm.mm.enabler.utils.Utils.createNode("atom:entry", this.nsf.getNameSpaceUri(this.nsf.NS_ATOM));
                    entry.appendChild(com.ibm.mm.enabler.utils.Utils.createNode("atom:id", this.nsf.getNameSpaceUri(this.nsf.NS_ATOM)));
                    
                    var title = com.ibm.mm.enabler.utils.Utils.createNode("atom:title", this.nsf.getNameSpaceUri(this.nsf.NS_ATOM));
                    com.ibm.mm.enabler.utils.Dom.textContent(title, "Adding a favorite space " + id);
                    entry.appendChild(title);
                    
                    var content = com.ibm.mm.enabler.utils.Utils.createNode("atom:content", this.nsf.getNameSpaceUri(this.nsf.NS_ATOM));
                    content.setAttribute("type", "application/xml");
                    
                    var favorite = com.ibm.mm.enabler.utils.Utils.createNode("ca:application-favorite", this.nsf.getNameSpaceUri(this.nsf.NS_CA));
                    com.ibm.mm.enabler.utils.Utils.setAttributeWithNS(favorite, "ca:app-id", this.nsf.getNameSpaceUri(this.nsf.NS_CA), id);
                    
                    content.appendChild(favorite);
                    entry.appendChild(content);
                    
                    myUrl = null;
                    
                    link = createdFavorites[id]._getFavoriteURL();
                    if (typeof(link) == "string") {
                        myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(link, this);
                    }
                    else 
                        if (!link) {
                            myUrl = new com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.SPACE_FAVORITE_URL, this);
                            myUrl.setNodes([{
                                value: id,
                                isID: true
                            }]);
                        }
                        else {
                            myUrl = link;
                        }
                    
                    serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                    this._acquire();
                    serviceReq.create(com.ibm.mm.enabler.utils.Dom.innerXML(entry), dojo.hitch(this, function(type, data, xhr, args){
                        if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                            if (createdFavorites[id]) {
                                createdFavorites[id]._favoriteUpdateCommited(myUrl.getAbsoluteURL());
                            }
                        }
                        this._release(deferred, xhr.status);
                    })); // JSLINT-IGNORE: Enabler Team decided to keep this paradigma from dojo in tact
                }
            }
            
            // the finishedCallback is triggered within the _release
            this._release(deferred, 0);
            
            if (bIsLoggable) {
                this.LOGGER.exiting(LOG_METHOD);
            }
        });
    },
    
    // get submittable form
    getSubmittableImportForm: function(id){
        this._initServiceDoc();
        var model = null;
        var url;
        if (id == "spaceForm") {
            url = new com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.SPACE_URL, null);
            url.setNodes([{
                value: "collection",
                isID: false
            }]);
            url.setParameter("mode", "import");
            url.setParameter("mime-type", "text/html");
            url = url.toProxifiedString();
            /*"/mum/mycontenthandler?uri=space:collection&mode=import"*/
            model = new com.ibm.mm.enabler.SubmittableFormImpl(url, "post", id);
        }
        else 
            if (id == "pageForm") {
                var serviceJson = com.ibm.mm.enabler.model.ServiceDocumentModel.getCollectionData(com.ibm.mm.enabler.model.ServiceDocumentModel.SERVICE_RESOURCE);
                url = new com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(serviceJson.url, this);
                url.setParameter("mode", "import");
                url.setParameter("mime-type", "text/html");
                // post the spaceId to server.
                if (arguments[1]) {
                    url.setParameter("spaceId", arguments[1]);
                }
                url = url.toProxifiedString();
                /*"/mum/mycontenthandler?uri=resource:collection&mode=import"*/
                model = new com.ibm.mm.enabler.SubmittableFormImpl(url, "post", id);
            }
        return model;
    },
    
    // acquire semaphore
    _acquire: function(){
        this.requestCount++;
    },
    
    // release semaphore
    _release: function(deferred, statusCode){
        // set overall status Code
        this.statusCode = Math.max(parseInt(statusCode, 10), this.statusCode);
        // check for full release
        this.requestCount--;
        if (this.requestCount <= 0) {
            if (deferred) {
                deferred.finish(null, this.statusCode);
            }
        }
    },
    
    remove: function(node){
        // noop
    },
    
    
    confirmRemove: function(node){
        var allowedByExtension = true;
        var extension = this.spaceExtensionModel.findExtensionBySpace(node).start();
        if (extension) {
            var spaceID = typeof(node) == "string" ? node : node.getID();
            var result = extension.canDeleteSpace(spaceID, null);
            allowedByExtension = (!result || result.getState() == "OK");
        }
        
        return this.getConfirmRemove(node) && allowedByExtension;
    },
    
    _applyNoCacheStrategy: function(url){
        var s = this._findStrategyByType("com.ibm.mashups.enabler.strategy.NoCacheStrategy");
        if (s) {
            url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_NOCACHE, "true");
        }
    },
    
    _getIteratorPersistenceHandler: function(){
        this._initServiceDoc();
        var that = this;
        return {
            // result is a json object
            // nodes: the nodes preloaded
            // size: the size of the iterator
            // loadFurtherNodes: defines whether we are at the end of the iterator or not
            loadAhead: function(context){
                return new com.ibm.mm.enabler.DeferredImpl(that, function(deferred, sync, node){
                    // logging support
                    var LOG_METHOD = "loadAhead";
                    var bIsLoggable = this.LOGGER.isLoggable(that.LOG_LEVEL);
                    if (bIsLoggable) {
                        this.LOGGER.entering(LOG_METHOD, arguments);
                    }
                    var retVal = null;
                    
                    // service request
                    var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(context.url, null, null, false, sync);
                    serviceReq.read(dojo.hitch(this, function(type, data, xhr, args){
                        var id;
                        var f = com.ibm.mm.enabler.space.persistence.xml.Factory;
                        if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                            retVal = {
                                nodes: [],
                                loadFurtherNodes: false
                            };
                            // fetch size
                            retVal.size = com.ibm.mashups.enabler.xml.XPath.evaluateNumber("atom:feed/opensearch:totalResults", data, this.ns);
                            // iterate favorites node entries
                            var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath("atom:feed/atom:entry", data, this.ns);
                            if (nodes) {
                                for (var i = 0; i < nodes.length; i++) {
                                    // instantiate node
                                    var node = f.createSpaceNodePersistence(nodes[i]);
                                    retVal.nodes.push(node);
                                }
                            }
                            // check for 'next'-link on feed level
                            var next = com.ibm.mashups.enabler.xml.XPath.evaluateEntry("atom:feed/atom:link[@rel='next']", data, this.ns);
                            if (next) {
                                // tell the iterator that it can load more nodes
                                retVal.loadFurtherNodes = true;
                                
                                var url = next.getAttribute("href");
                                // set collection url from 'next'-link
                                context.url = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(url, this);
                                
                                // include access control info
                                context.url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
                                // apply caching strategy
                                this._applyNoCacheStrategy(context.url);
                            }
                            if (deferred) {
                                deferred.finish(retVal, xhr.status);
                            }
                        }
                        else {
                            // error callback
                            if (deferred) {
                                deferred.finish(null, xhr.status);
                            }
                        }
                    }));
                    // exit trace
                    if (bIsLoggable) {
                        this.LOGGER.exiting(LOG_METHOD, retVal);
                    }
                    return retVal;
                });
            },
            getLoadAheadContext: function(context, iterator){
                if (!context) { // start at the beginning
                    context = {};
                    // collection url
                    context.url = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(that.serviceJson.url, that);
                    if (that.strategy) {
                        for (var x in that.strategy) {
                            if (com.ibm.mm.enabler.utils.Misc.isInstanceOf(that.strategy[x], com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy)) {
                                // max number of entries 
                                context.url.setParameter("num", that.strategy[x].getInterval());
                                // start element (only set with paging) 
                                context.url.setParameter("start", iterator.getCursorPosition());
                                break;
                            }
                        }
                    }
                    // include access control info
                    context.url.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
                    // apply caching strategy
                    that._applyNoCacheStrategy(context.url);
                }
                return context;
            },
            setCursorPosition: function(context, iterator){
                if ((context) && (context.url)) {
                    context.url.setParameter("start", iterator.getCursorPosition());
                }
                return context;
            }
        };
    },
    
    _getFavoriteIteratorPersistenceHandler: function(){
        var that = this;
        return {
            // result is a json object
            // nodes: the nodes preloaded
            // size: the size of the iterator
            // loadFurtherNodes: defines whether we are at the end of the iterator or not
            loadAhead: function(context){
                return new com.ibm.mm.enabler.DeferredImpl(that, function(deferred, sync, node){
                    // logging support
                    var LOG_METHOD = "loadAhead";
                    var bIsLoggable = this.LOGGER.isLoggable(that.LOG_LEVEL);
                    if (bIsLoggable) {
                        this.LOGGER.entering(LOG_METHOD, arguments);
                    }
                    var retVal = null;
                    
                    // url parameter
                    var myUrl = com.ibm.mashups.enabler.model.url.ModelUrlFactory.getModelURL(this.serviceJson.url, this);
                    // favorites filter
                    myUrl.setParameter("filter", "is-favorite=true");
                    // include access control info
                    myUrl.addParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_ASPECT_VALUE_ACCESS_CONTROL);
                    // full representation
                    myUrl.setParameter(com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION, com.ibm.mm.enabler.model.url.ModelUrlConstants.PARAMETER_REPRESENTATION_VALUE_FULL);
                    
                    // service request
                    var serviceReq = new com.ibm.mm.enabler.services.ModelRestServiceRequest(myUrl, null, null, false, sync);
                    serviceReq.read(dojo.hitch(this, function(type, data, xhr, args){
                        var id;
                        var f = com.ibm.mm.enabler.space.persistence.xml.Factory;
                        if (type == com.ibm.mm.enabler.services.ModelRestServiceRequest.XHR_STATUS_LOAD) {
                            retVal = {
                                nodes: [],
                                loadFurtherNodes: false
                            };
                            // iterate favorites node entries
                            var expr = "atom:feed/atom:entry";
                            var nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(expr, data, this.ns);
                            if (nodes) {
                                for (var i = 0; i < nodes.length; i++) {
                                    // instantiate node
                                    var node = f.createSpaceNodePersistence(nodes[i]);
                                    retVal.nodes.push(node);
                                }
                            }
                            retVal.size = retVal.nodes.length;
                            if (deferred) {
                                deferred.finish(retVal, xhr.status);
                            }
                        }
                        else {
                            // error callback
                            if (deferred) {
                                deferred.finish(null, xhr.status);
                            }
                        }
                    }));
                    // exit trace
                    if (bIsLoggable) {
                        this.LOGGER.exiting(LOG_METHOD, retVal);
                    }
                    return retVal;
                });
            },
            getLoadAheadContext: function(context, result){
                return context;
            },
            setCursorPosition: function(context, iterator){
                return context;
            }
        };
    }
});

}

if(!dojo._hasResource["tagservices.osgadgets"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["tagservices.osgadgets"] = true;
dojo.provide("tagservices.osgadgets");

dojo.provide("com.ibm.mm.livetext.osgadgets");
/*
 * This is injected on the fly and not fetched through an API. Therefore we need to define the @ibm-module so that the build process is picking it up
 * @ibm-module OSGadget
 */




/* Livetext tag used for loading openSocial gadget, for the tag to work you need to have Shindig container 
 // loaded as part of the bootstrap
 */
dojo.declare("tagservices.osgadgets", null, {
    constructor: function() {
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger(this.declaredClass);
        this.LOG_LEVEL = com.ibm.mashups.enabler.logging.LogLevel.TRACE;
    },
    processTag: function(tag) {
        // logging support
        var LOG_METHOD = "processTag";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
		
        var gadgetId = tag.getAttribute("id");
		
		com.ibm.mm.enabler.opensocial.OpenSocial.onDemandLoad_OpenSocialLibs();	

		// retrieve gadget definition url
        var temp = dojo.query(".iw-Definition", tag)[0];
        if (!temp) {
            temp = dojo.query(".mm_Definition", tag)[0];
        }        
        // retrieve and appropriately form gadget definition url
        var gadgetUrlFromDef = new com.ibm.mm.enabler.utils.HttpUrl(temp.getAttribute("href"));        
				
		com.ibm.mm.enabler.opensocial.OpenSocial.renderGadget(gadgetId,gadgetUrlFromDef);		
    },
    unprocessTag: function(tag) {
        var ns = ["iw-", "mm_"];
        for (var n = 0; n < ns.length; n++) {
            // search the content nodes
            var contentNodes = dojo.query("." + ns[n] + iwConstants.CSSCLASS_INSTANCE.iwContent, tag);
            
            // iterate through them and delete them
            for (var i = 0; i < contentNodes.length; i++) {
                com.ibm.mm.enabler.utils.Dom.destroyNode(contentNodes[i]);
            }
        }
    }	
});

}


dojo.i18n._preloadLocalizations("com.ibm.mm.enabler.nls.enabler", ["ROOT","ar","ca","cs","da","de","de-de","el","en","en-gb","en-us","es","es-es","fi","fi-fi","fr","fr-fr","he","he-il","hr","hu","it","it-it","ja","ja-jp","kk","ko","ko-kr","nb","nl","nl-be","nl-nl","no","pl","pt","pt-br","pt-pt","ro","ro-ro","ru","sk","sl","sv","th","tr","uk","xx","zh","zh-cn","zh-tw"]);
